package com.mortals.xhx.common.code;

import java.util.HashMap;
import java.util.Map;

/**
 * 认证方式
 */
public enum AuthTypeEnum {
	
	//设备接入平台的安全认证方式。产品创建成功后，认证方式不可变更。
	//1，设备密钥：使用物联网平台为设备生成的DeviceSecret进行设备认证签名计算。使用DeviceSecret签名计算，
	//2，ID²：ID²认证提供设备与物联网平台的双向身份认证能力，通过建立轻量化的安全链路（iTLS）来保障数据的安全性。
	//3，X.509证书：使用X.509数字证书进行设备身份认证。',
	
	/** 1：设备密钥 */
	DEV_KEY(1, "设备密钥"),
    /** 2：ID² */
	ITLS(2, "ID²"),
    /** 3：ID² */
	X_509(3, "X.509证书");

    private int value;
    private String msg;

    AuthTypeEnum(int value, String msg) {
        this.value = value;
        this.msg = msg;
    }

    public int getValue() {
        return this.value;
    }

    public static Map<String, String> getEnumMap() {
        Map<String, String> resultMap = new HashMap<>();
        AuthTypeEnum[] contractTypeEnums = AuthTypeEnum.values();
        for (AuthTypeEnum contractTypeEnum : contractTypeEnums) {
            resultMap.put(String.valueOf(contractTypeEnum.value), contractTypeEnum.msg);
        }
        return resultMap;
    }
}
