DROP TABLE IF EXISTS `mortals_xhx_connect_limit`;
CREATE TABLE `mortals_xhx_connect_limit`(
	`id` bigint(20) NOT NULL  auto_increment  COMMENT '主键ID，主键，自增长',
	`deviceConnectLimit` smallint(1) NOT NULL  default '1' COMMENT '设备接入限制，使用同一个设备证书信息（相同的Productkey、DeviceName）。在同一时间，只能和物联网平台服务器建立一个连接。默认:1',
	`customerDeviceMQTTConnectLimit` smallint(3) NOT NULL  default '500' COMMENT '用户下每秒所有设备MQTT连接请求接入次数，每秒最大MQTT连接请求数默认:500',
	`deviceConnectRequestLimit` smallint(1) NOT NULL  default '5' COMMENT '一个设备每分钟的最大连接请求次数。，默认:5',
	`deviceSubNumLimit` smallint(3) NOT NULL  default '100' COMMENT '一个设备的最大订阅数，默认:100',
	`customerDeviceToPlatformRequestLimit` smallint(5) NOT NULL  default '10000' COMMENT '每秒所有设备端向物联网平台发送的请求数。，默认:10000',
	`customerPlatformToDeviceRequestLimit` smallint(4) NOT NULL  default '2000' COMMENT '每秒由物联网平台向设备端发送的请求数。，默认:2000',
	`deviceSubmitRequestQos0Limit` smallint(2) NOT NULL  default '30' COMMENT '一个设备每秒最多可上报的消息数量，默认:30',
	`deviceSubmitRequestQos1Limit` smallint(2) NOT NULL  default '10' COMMENT '一个设备每秒最多可上报的消息数量，默认:10',
	`deviceReveiceMessageLimit` smallint(2) NOT NULL  default '50' COMMENT '一个设备接收下行消息的最大限制，默认:50',
	`connectThroughputLimit` smallint(4) NOT NULL  default '1024' COMMENT '一个连接每秒的吞吐量最大限制。，默认:1024K',
	`messageQoS1StorageMaxLimit` smallint(2) NOT NULL  default '7' COMMENT 'QoS1消息的最大存储时间。。，默认:7天',
	`messageMQTTLenghtLimit` smallint(3) NOT NULL  default '256' COMMENT 'MQTT消息长度，MQTT单个发布消息最大长度。默认:256K',
	`MQTTHearbeatLimit` smallint(4) NOT NULL  default '300' COMMENT 'MQTT保活，MQTT连接心跳时间为30秒至1,200秒。心跳时间不在此区间内，服务器将会拒绝连接。默认:300秒',
	`createTime` datetime NOT NULL  COMMENT '创建时间',
	`updateTime` datetime NOT NULL  COMMENT '更新时间',
	PRIMARY KEY  (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='连接通信限制表';

DROP TABLE IF EXISTS `mortals_xhx_consumer_group`;
CREATE TABLE `mortals_xhx_consumer_group`(
	`id` bigint(20) NOT NULL  auto_increment  COMMENT '主键ID，主键，自增长',
	`customerId` bigint(20) NOT NULL  COMMENT '用户ID',
	`consumerGroupName` varchar(30) NOT NULL  COMMENT '消费组名称，消费组名称支持中文、英文字母、日文、数字和下划线（_），长度范围为4~30个字符',
	`createTime` datetime NOT NULL  COMMENT '创建时间',
	`updateTime` datetime NOT NULL  COMMENT '更新时间',
	PRIMARY KEY  (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='服务端订阅消费组表';

DROP TABLE IF EXISTS `mortals_xhx_consumer_sub`;
CREATE TABLE `mortals_xhx_consumer_sub`(
	`id` bigint(20) NOT NULL  auto_increment  COMMENT '主键ID，主键，自增长',
	`customerId` bigint(20) NOT NULL  COMMENT '用户ID',
	`productId` bigint(20) NOT NULL  COMMENT '产品',
	`consumerGroupId` bigint(20) NOT NULL  COMMENT '消费组ID',
	`pushMessType` tinyint(2) NOT NULL  COMMENT '推送消息类型，服务端要订阅的消息类型。目前，服务端可订阅的设备消息类型包括：1，设备上报消息',
	`subType` tinyint(2) NOT NULL  COMMENT '订阅类型，1，AMQP2，MNS',
	`createTime` datetime NOT NULL  COMMENT '创建时间',
	`updateTime` datetime NOT NULL  COMMENT '更新时间',
	PRIMARY KEY  (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='服务端订阅表';

DROP TABLE IF EXISTS `mortals_xhx_customer`;
CREATE TABLE `mortals_xhx_customer`(
	`id` bigint(20) NOT NULL  auto_increment  COMMENT '主键ID，主键，自增长',
	`loginName` varchar(30) NOT NULL  COMMENT '用户账号',
	`password` varchar(32) NOT NULL  COMMENT '密码',
	`custType` tinyint(2) NOT NULL  COMMENT '客户类型',
	`tradeId` tinyint(2) NOT NULL  COMMENT '行业类型',
	`custName` varchar(32) NOT NULL  COMMENT '客户名称',
	`address` varchar(64) default NULL  COMMENT '地址',
	`contactTelphone` varchar(11) NOT NULL  COMMENT '联系电话',
	`status` tinyint(2) NOT NULL  COMMENT '状态',
	`accessKey` varchar(12) NOT NULL  COMMENT '颁发给用户的访问服务所用的密钥ID',
	`createTime` datetime NOT NULL  COMMENT '创建时间',
	`updateTime` datetime NOT NULL  COMMENT '更新时间',
	PRIMARY KEY  (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='客户表';

DROP TABLE IF EXISTS `mortals_xhx_device`;
CREATE TABLE `mortals_xhx_device`(
	`id` bigint(20) NOT NULL  auto_increment  COMMENT '主键ID，主键，自增长',
	`customerId` bigint(20) NOT NULL  COMMENT '用户ID',
	`productId` bigint(20) NOT NULL  COMMENT '产品',
	`deviceName` varchar(32) NOT NULL  COMMENT '设备名称，设置设备名称。 设备名称在产品内具有唯一性。支持英文字母、数字、短划线（-）、下划线（_）、at（@）、英文句号（.）和英文冒号（:），长度限制为4~32个字符。',
	`deviceRemark` varchar(64) default NULL  COMMENT '设备备注，设置备注名称。支持中文、英文字母、日文、数字和下划线（_），长度限制为4~64个字符，',
	`deviceSecret` varchar(64) NOT NULL  COMMENT 'DeviceSecret，平台为设备颁发的设备密钥，用于认证加密。需与DeviceName成对使用。',
	`productKey` varchar(64) NOT NULL  COMMENT 'productKey，平台为产品颁发的全局唯一标识符',
	`connectStatus` tinyint(2) NOT NULL  default '2' COMMENT '连接状态，0：离线  1：在线 2：未激活，默认2',
	`useStatus` tinyint(2) NOT NULL  default '1' COMMENT '启用状态，0：停用  1：启用 2: 测试，默认1',
	`onlineTime` datetime default NULL  COMMENT '最近上线时间',
	`offlineTime` datetime default NULL  COMMENT '最近离线时间',
	`createTime` datetime NOT NULL  COMMENT '创建时间',
	`updateTime` datetime NOT NULL  COMMENT '更新时间',
	PRIMARY KEY  (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='设备表';

DROP TABLE IF EXISTS `mortals_xhx_device_topic`;
CREATE TABLE `mortals_xhx_device_topic`(
	`id` bigint(20) NOT NULL  auto_increment  COMMENT '主键ID，主键，自增长',
	`customerId` bigint(20) NOT NULL  COMMENT '用户ID',
	`productId` bigint(20) NOT NULL  COMMENT '产品',
	`deviceTopicHandle` tinyint(2) NOT NULL  COMMENT '设备操作权限，设备对该Topic的操作权限，可设置为发布、订阅、发布和订阅。',
	`topic` varchar(255) NOT NULL  COMMENT 'Topic类，设备的具体是topic信息，根据产品topic信息生产',
	`deviceTopicRemark` varchar(64) default NULL  COMMENT '描述',
	`createTime` datetime NOT NULL  COMMENT '创建时间',
	`updateTime` datetime NOT NULL  COMMENT '更新时间',
	`deviceId` bigint(20) NOT NULL  COMMENT '设备ID',
	`deviceName` varchar(32) NOT NULL  COMMENT '设备名称',
	`qos` tinyint(2) NOT NULL  COMMENT 'qos',
	`productKey` varchar(64) NOT NULL  COMMENT 'productKey，平台为产品颁发的全局唯一标识符',
	PRIMARY KEY  (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='设备topic表';

DROP TABLE IF EXISTS `mortals_xhx_product`;
CREATE TABLE `mortals_xhx_product`(
	`id` bigint(20) NOT NULL  auto_increment  COMMENT '主键ID，主键，自增长',
	`customerId` bigint(20) NOT NULL  COMMENT '用户ID',
	`productName` varchar(30) NOT NULL  COMMENT '产品名称，为产品命名。产品名称在账号内具有唯一性。例如，可以填写为产品型号。支持中文、英文字母、日文、数字、下划线（_）、短划线（-）、at（@）和英文圆括号，长度限制4~30个字符。',
	`productType` tinyint(2) NOT NULL  COMMENT '所属品类，相当于产品模板。1，标准品类：平台已为标准品类预定义了功能模板。例如，能源管理 > 电表品类已预定义用电量、电压、电流、总累积量等电表标准功能。选择该品类，创建的产品具有预定义的功能。您可以在该产品的产品详情页功能定义页签下，编辑、修改、新增功能。2，自定义品类：产品创建成功后，需根据实际需要，自定义物模型。',
	`nodeType` tinyint(2) NOT NULL  COMMENT '节点类型，产品下设备的类型。1，直连设备：具有IP地址，可直接连接平台，且不能挂载子设备，但可作为子设备挂载到网关下的设备。2，网关子设备：不直接连接平台，而是通过网关设备接入平台的设备。3，网关设备：可以挂载子设备的直连设备。网关具有子设备管理模块，可以维持子设备的拓扑关系，将与子设备的拓扑关系同步到云端。',
	`connGateProtocol` tinyint(2) default NULL  COMMENT '接入网关协议，节点类型选择为网关子设备的参数。表示该产品下的设备作为子设备与网关的通讯协议类型。自定义：表示子设备和网关之间是其它标准或私有协议。1,Modbus：表示子设备和网关之间的通讯协议是Modbus。2,OPC UA：表示子设备和网关之间的通讯协议是OPC UA。3,ZigBee：表示子设备和网关之间的通讯协议是ZigBee。4,BLE：表示子设备和网关之间的通讯协议是BLE。',
	`networkType` tinyint(2) NOT NULL  COMMENT '连网方式，直连设备和网关设备的连网方式。1,Wi-Fi2,蜂窝（2G/3G/4G/5G）3,以太网4,LoRaWAN5,其他',
	`networkProof` varchar(30) default NULL  COMMENT '入网凭证',
	`dataType` tinyint(2) NOT NULL  COMMENT '数据格式，设备上下行的数据格式。1，标准数据格式：是平台为开发者提供的设备与云端的数据交换协议，采用JSON格式。2，透传/自定义：选择透传/自定义。需在定义数据解析脚本，将上行的自定义格式的数据转换为标准数据格式；将下行的标准数据格式数据解析为设备自定义格式，设备才能与云端进行通信。',
	`checkType` tinyint(2) NOT NULL  COMMENT '校验类型，设备上报物模型数据时，平台将按设置的数据校验方式，根据物模型定义进行数据校验。1,弱校验：只校验设备数据的idetifier和dataType字段，不校验其他字段，流转全量数据。2,免校验：对上报的数据不进行校验，流转全量数据。',
	`authType` tinyint(2) NOT NULL  COMMENT '认证方式，设备接入平台的安全认证方式。产品创建成功后，认证方式不可变更。1，设备密钥：使用物联网平台为设备生成的DeviceSecret进行设备认证签名计算。使用DeviceSecret签名计算，2，ID²：ID²认证提供设备与物联网平台的双向身份认证能力，通过建立轻量化的安全链路（iTLS）来保障数据的安全性。3，X.509证书：使用X.509数字证书进行设备身份认证。',
	`productRemark` varchar(64) default NULL  COMMENT '产品备注',
	`productKey` varchar(64) NOT NULL  COMMENT 'productKey，平台为产品颁发的全局唯一标识符',
	`productStatus` tinyint(2) NOT NULL  COMMENT '产品状态，1，开发中2，发布',
	`productTSL` text NOT NULL  COMMENT '产品下发布的物模型',
	`qos` tinyint(2) NOT NULL  COMMENT 'qos',
	`createTime` datetime NOT NULL  COMMENT '创建时间',
	`updateTime` datetime NOT NULL  COMMENT '更新时间',
	PRIMARY KEY  (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='产品表';

DROP TABLE IF EXISTS `mortals_xhx_product_device_limit`;
CREATE TABLE `mortals_xhx_product_device_limit`(
	`id` bigint(20) NOT NULL  auto_increment  COMMENT '主键ID，主键，自增长',
	`productLabelNum` smallint(3) NOT NULL  default '100' COMMENT '产品标签数量，产品最多可以添加的标签数。默认:100',
	`deviceLabelNum` smallint(3) NOT NULL  default '100' COMMENT '设备标签数量，设备最多可以添加的标签数。默认:100',
	`productNum` smallint(4) NOT NULL  default '1000' COMMENT '产品数量，一个客户下最多可以创建的产品数。默认:1000',
	`productDeviceNum` int(7) NOT NULL  default '1000000' COMMENT '产品下设备数量，一个产品最多可以添加的设备数。默认:1000000',
	`customerDeviceNum` int(8) NOT NULL  default '10000000' COMMENT '用户下的设备数量，一个用户最多可以添加的设备数。默认:10000000',
	`gateChildrenDeviceNum` smallint(4) NOT NULL  default '1500' COMMENT '网关下子设备的数量，一个网关下最多添加的子设备数。默认:1500',
	`createTime` datetime NOT NULL  COMMENT '创建时间',
	`updateTime` datetime NOT NULL  COMMENT '更新时间',
	PRIMARY KEY  (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='产品和设备限制表';

DROP TABLE IF EXISTS `mortals_xhx_product_function_item`;
CREATE TABLE `mortals_xhx_product_function_item`(
	`id` bigint(20) NOT NULL  auto_increment  COMMENT '主键ID，主键，自增长',
	`functionName` varchar(30) NOT NULL  COMMENT '功能名称',
	`functionIdentifier` varchar(50) NOT NULL  COMMENT '标识符',
	`eventType` tinyint(2) default NULL  COMMENT '事件类型，事件拥有1，信息2告警故障',
	`readOrWrite` tinyint(2) default NULL  COMMENT '读写类型，属性拥有1，读写2只读',
	`invokingType` tinyint(2) default NULL  COMMENT '调用方式，服用拥有1，同步2.异步',
	`productFunctionType` tinyint(2) NOT NULL  COMMENT '产品功能类型，1，属性2，事件3，服务',
	`productId` bigint(20) NOT NULL  COMMENT '产品ID',
	`createTime` datetime NOT NULL  COMMENT '创建时间',
	`updateTime` datetime NOT NULL  COMMENT '更新时间',
	`productKey` varchar(64) NOT NULL  COMMENT 'productKey，平台为产品颁发的全局唯一标识符',
	`customerId` bigint(20) NOT NULL  COMMENT '用户ID',
	`productFunctionStatus` tinyint(2) NOT NULL  COMMENT '发布状态，1，草稿2，发布3，废弃',
	`productFunctionMethod` varchar(64) default NULL  COMMENT '请求方法',
	PRIMARY KEY  (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='产品物模型节点表';

DROP TABLE IF EXISTS `mortals_xhx_product_function_item_detail`;
CREATE TABLE `mortals_xhx_product_function_item_detail`(
	`id` bigint(20) NOT NULL  auto_increment  COMMENT '主键ID，主键，自增长',
	`dataType` tinyint(2) NOT NULL  COMMENT '数据类型',
	`dataTypeDetail` text NOT NULL  COMMENT '数据类型明细信息',
	`parameterName` varchar(30) NOT NULL  COMMENT '参数名称',
	`detailIdentifier` varchar(50) NOT NULL  COMMENT '标识符',
	`parameterType` tinyint(2) NOT NULL  COMMENT '参数类型，1，输出2，输入',
	`createTime` datetime NOT NULL  COMMENT '创建时间',
	`updateTime` datetime NOT NULL  COMMENT '更新时间',
	`productId` bigint(20) NOT NULL  COMMENT '产品ID',
	`productKey` varchar(64) NOT NULL  COMMENT 'productKey，平台为产品颁发的全局唯一标识符',
	`functionItemId` bigint(20) NOT NULL  COMMENT '物模型节点主键',
	PRIMARY KEY  (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='产品物模型节点明细表';

DROP TABLE IF EXISTS `mortals_xhx_product_topic`;
CREATE TABLE `mortals_xhx_product_topic`(
	`id` bigint(20) NOT NULL  auto_increment  COMMENT '主键ID，主键，自增长',
	`customerId` bigint(20) NOT NULL  COMMENT '用户ID',
	`productId` bigint(20) NOT NULL  COMMENT '产品',
	`deviceTopicHandle` tinyint(2) NOT NULL  COMMENT '设备操作权限，设备对该Topic的操作权限，可设置为发布、订阅、发布和订阅。',
	`topic` varchar(255) NOT NULL  COMMENT 'Topic类，将Topic类填充完整。类目命名只能包含字母、数字和下划线（_），每级类目不能为空。',
	`productTopicRemark` varchar(64) default NULL  COMMENT '描述',
	`createTime` datetime NOT NULL  COMMENT '创建时间',
	`updateTime` datetime NOT NULL  COMMENT '更新时间',
	`qos` tinyint(2) NOT NULL  COMMENT 'qos',
	`productKey` varchar(64) NOT NULL  COMMENT 'productKey，平台为产品颁发的全局唯一标识符',
	PRIMARY KEY  (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='产品topic表';

DROP TABLE IF EXISTS `mortals_xhx_product_topic_limit`;
CREATE TABLE `mortals_xhx_product_topic_limit`(
	`id` bigint(20) NOT NULL  auto_increment  COMMENT '主键ID，主键，自增长',
	`productCreateTopicNumLimit` smallint(2) NOT NULL  default '50' COMMENT '自定义Topic类数量，一个产品最多可以定义50个Topic类。默认:50',
	`topicTopLimit` smallint(2) NOT NULL  default '7' COMMENT 'Topic类目，一个Topic中最多可包含多少个层级类目，即Topic中斜杠的最大数量。默认:7',
	`deviceSubRequestMaxNumLimit` smallint(2) NOT NULL  default '8' COMMENT '订阅数，一个订阅请求的最大订阅数。默认:8',
	`deviceSubOrUnSubTimeLimit` smallint(2) NOT NULL  default '10' COMMENT '订阅操作生效时间，订阅和取消订阅都是操作10秒后生效，一次订阅永久生效。默认:10秒',
	`createTime` datetime NOT NULL  COMMENT '创建时间',
	`updateTime` datetime NOT NULL  COMMENT '更新时间',
	PRIMARY KEY  (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='产品topic类限制表';

DROP TABLE IF EXISTS `mortals_xhx_product_tsl_limit`;
CREATE TABLE `mortals_xhx_product_tsl_limit`(
	`id` bigint(20) NOT NULL  auto_increment  COMMENT '主键ID，主键，自增长',
	`productTSLNum` smallint(2) NOT NULL  default '10' COMMENT '产品下物模型的个数，一个产品最多可包含物模型模块（默认模块和自定义模块）总数。默认:10',
	`TSLFunctionNum` smallint(3) NOT NULL  default '300' COMMENT '一个TSL的功能数，一个物模型模块最多可添加的功能总数。（属性，服务，事件的总和），默认:300',
	`TSLStructParameterNum` smallint(3) NOT NULL  default '300' COMMENT 'Struct参数个数，struct类的属性最多可添加的参数个数，默认:300',
	`TSLDataTpyeEnumNum` smallint(3) NOT NULL  default '100' COMMENT 'enum个数，当功能的数据类型为enum时，枚举项的最多个数。默认:100',
	`TSLDataTpyeTextLength` smallint(3) NOT NULL  default '10240' COMMENT '文本长度，当功能的数据类型为text时，数据最大长度。默认:10240字符',
	`TSLDataTpyeArrayNum` smallint(3) NOT NULL  default '512' COMMENT '数组长度，当功能的数据类型为array时，数组内的元素最多个数。默认:512个',
	`TSLServiceEventParameterNum` smallint(3) NOT NULL  default '300' COMMENT '所有服务和事件入参和出参的参数总个数，所有服务和事件中最多可添加入参和出参的总数。默认:300个',
	`TSLServiceInParameterNum` smallint(3) NOT NULL  default '100' COMMENT '一个服务入参数个数，一个服务中最多可添加的入参个数。默认:100个',
	`TSLServiceOutParameterNum` smallint(3) NOT NULL  default '100' COMMENT '一个服务出参数个数，一个服务中最多可添加的出参个数。默认:100个',
	`TSLEventOutParameterNum` smallint(3) NOT NULL  default '300' COMMENT '一个事件出参个数，一个事件中最多可添加的出参个数。默认:300个',
	`TSLDraftVersionNum` smallint(3) NOT NULL  default '10' COMMENT '单个TSL保存的草稿数，一个物模型最多保存最近版本的个数。默认:10个',
	`TSLExportJsonFileSize` smallint(3) NOT NULL  default '256' COMMENT '导入一个文件大小，一个物模型最多保存最近版本的个数。默认:256K',
	`TSLExportMoreJsonFileSize` smallint(3) NOT NULL  default '2' COMMENT '导入多个文件大小，导入物模型时，多个JSON格式文件压缩的ZIP文件大小的上限。默认:2.5M',
	`createTime` datetime NOT NULL  COMMENT '创建时间',
	`updateTime` datetime NOT NULL  COMMENT '更新时间',
	PRIMARY KEY  (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='物模型功能限制表';

DROP TABLE IF EXISTS `mortals_xhx_service_sub_amqp_limit`;
CREATE TABLE `mortals_xhx_service_sub_amqp_limit`(
	`id` bigint(20) NOT NULL  auto_increment  COMMENT '主键ID，主键，自增长',
	`authTimeout` smallint(2) NOT NULL  default '15' COMMENT '认证超时，建立连接之后，需要立刻发送认证请求。如果15秒内没有认证成功，服务器将主动关闭连接。默认:15秒',
	`messageTimeout` smallint(3) NOT NULL  default '300' COMMENT '数据超时，服务端与物联网平台建立连接时，需传入心跳时间（AMQP协议参数idle-timeout），取值范围为30~300秒。 如果超过心跳时间，连接上无任何帧通信，物联网平台将关闭连接。建立连接后，用户服务端需在心跳时间内发送PING包来维持连接。若没有在心跳时间内发送PING包，物联网平台将断开连接默认:300秒',
	`consumerGroupMessageStorageMaxLimit` int(9) NOT NULL  default '1000000' COMMENT '消息保存条数，一个消费组最多可堆积消息条数。默认:1000000',
	`messageStorageTimeLimit` smallint(1) NOT NULL  default '1' COMMENT '消息保存时长，消息保存时长。默认:1天',
	`pushMessageLimit` int(6) NOT NULL  default '1000' COMMENT '实时消息推送限流，一个连接限流，默认:1000 TPS',
	`pushHeapUpMessageLimit` smallint(4) NOT NULL  default '200' COMMENT '堆积消息推送限流，一个消费组限流，默认:200 TPS。',
	`productConsumerGroupNumLimit` smallint(2) NOT NULL  default '10' COMMENT '产品可关联的消费组数量，默认:10',
	`consumerGroupProductNumLimit` smallint(3) NOT NULL  default '100' COMMENT '消费组可关联的产品数，默认:100',
	`customerConsumerGroupNumLimit` smallint(4) NOT NULL  default '1000' COMMENT '消费组个数限制，默认:1000',
	`consumerGroupHaveConsumerNumLimit` smallint(2) NOT NULL  default '64' COMMENT '消费端个数限制，默认:64',
	`consumerGroupConsumerConNumLimit` smallint(3) NOT NULL  default '100' COMMENT '连接次数限制，一个消费组1分钟内，消费端请求连接不超过100次。默认:100',
	`createTime` datetime NOT NULL  COMMENT '创建时间',
	`updateTime` datetime NOT NULL  COMMENT '更新时间',
	PRIMARY KEY  (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='AMQP服务端订阅限制表';

DROP TABLE IF EXISTS `mortals_xhx_device_log`;
CREATE TABLE `mortals_xhx_device_log`(
	`id` bigint(20) NOT NULL  auto_increment  COMMENT '主键ID，主键，自增长',
	`deviceName` varchar(32) NOT NULL  COMMENT '设备名称，设置设备名称。',
	`productKey` varchar(64) NOT NULL  COMMENT 'productKey，平台为产品颁发的全局唯一标识符',
	`logType` tinyint(2) NOT NULL  COMMENT '日志类型，0：上报事件  1：下发服务',
	`content` varchar(5000) default NULL  COMMENT '内容',
	`createTime` datetime NOT NULL  COMMENT '创建时间',
	PRIMARY KEY  (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='设备日志表';


INSERT INTO `mortals_xhx_menu` VALUES ('1', '产品管理', '', '0', '20', '1', '0', '0','1','goods', '', '', '0', '0', '3', null, null, null);
INSERT INTO `mortals_xhx_menu` VALUES ('2', '设备管理', '', '0', '30', '1', '0', '0','1', 'menu', '', '', '0', '0', '3', null, null, null);
INSERT INTO `mortals_xhx_menu` VALUES ('3', '客户管理', '', '0', '40', '1', '0', '0','1', 'mobile-phone', '', '', '0', '0', '3', null, null, null);
INSERT INTO `mortals_xhx_menu` VALUES ('4', '服务订阅', '', '0', '50', '1', '0', '0','1', 'service', '', '', '0', '0', '3', null, null, null);
INSERT INTO `mortals_xhx_menu` VALUES ('5', '监控运维', '', '0', '60', '1', '0', '0','1', 'bell', '', '', '0', '0', '3', null, null, null);
INSERT INTO `mortals_xhx_menu` VALUES ('6', '限制管理', '', '0', '70', '1', '0', '0','1', 'share', '', '', '0', '0', '3', null, null, null);
INSERT INTO `mortals_xhx_menu` VALUES (101, '产品列表', '/product/list', 1, 10, 1, 0, 1,'1', '', NULL, NULL, 0, 0, 3, '2019-02-19 16:08:09', 1, 'admin');
INSERT INTO `mortals_xhx_menu` VALUES (102, '产品物模型', '/product/function/item/list', 1, 20, 1, 0, 1,'1', '', NULL, NULL, 0, 0, 3, '2019-02-19 16:08:09', 1, 'admin');
INSERT INTO `mortals_xhx_menu` VALUES (103, '产品物模型详情', '/product/function/item/detail/list', 1, 30, 1, 0, 1,'1', '', NULL, NULL, 0, 0, 3, '2019-02-19 16:08:09', 1, 'admin');
INSERT INTO `mortals_xhx_menu` VALUES (104, '产品Topic', '/product/topic/list', 1, 40, 1, 0, 1, '1','', NULL, NULL, 0, 0, 3, '2019-02-19 16:08:09', 1, 'admin');
INSERT INTO `mortals_xhx_menu` VALUES (201, '设备列表', '/device/list', 2, 10, 1, 0, 1,'1', '', NULL, NULL, 0, 0, 3, '2019-02-19 16:08:09', 1, 'admin');
INSERT INTO `mortals_xhx_menu` VALUES (202, '设备Topic', '/device/topic/list', 2, 20, 1, 0, 1, '1','', NULL, NULL, 0, 0, 3, '2019-02-19 16:08:09', 1, 'admin');
INSERT INTO `mortals_xhx_menu` VALUES (203, '设备日志', '/device/log/list', 2, 30, 1, 0, 1,'1', '', NULL, NULL, 0, 0, 3, '2019-02-19 16:08:09', 1, 'admin');
INSERT INTO `mortals_xhx_menu` VALUES (301, '客户列表', '/customer/list', 3, 10, 1, 0, 1,'1', '', NULL, NULL, 0, 0, 3, '2019-02-19 16:08:09', 1, 'admin');
INSERT INTO `mortals_xhx_menu` VALUES (401, '服务订阅组', '/consumer/group/list', 4, 10, 1, 0, 1,'1', '', NULL, NULL, 0, 0, 3, '2019-02-19 16:08:09', 1, 'admin');
INSERT INTO `mortals_xhx_menu` VALUES (402, '服务订阅', '/consumer/sub/list', 4, 20, 1, 0, 1, '1','', NULL, NULL, 0, 0, 3, '2019-02-19 16:08:09', 1, 'admin');
INSERT INTO `mortals_xhx_menu` VALUES (601, '连接通信限制', '/connect/limit/list', 6, 10, 1, 0, 1,'1', '', NULL, NULL, 0, 0, 3, '2019-02-19 16:08:09', 1, 'admin');
INSERT INTO `mortals_xhx_menu` VALUES (602, '产品设备限制', '/product/device/limit/list', 6, 20, 1, 0, 1,'1', '', NULL, NULL, 0, 0, 3, '2019-02-19 16:08:09', 1, 'admin');
INSERT INTO `mortals_xhx_menu` VALUES (603, '产品topic类限制', '/product/topic/limit/list', 6, 30, 1, 0, 1,'1', '', NULL, NULL, 0, 0, 3, '2019-02-19 16:08:09', 1, 'admin');
INSERT INTO `mortals_xhx_menu` VALUES (604, '物模型功能限制', '/product/tsl/limit/list', 6, 40, 1, 0, 1,'1', '', NULL, NULL, 0, 0, 3, '2019-02-19 16:08:09', 1, 'admin');
INSERT INTO `mortals_xhx_menu` VALUES (605, 'AMQP服务订阅限制', '/service/sub/amqp/limit/list', 6, 50, 1, 0, 1,'1', '', NULL, NULL, 0, 0, 3, '2019-02-19 16:08:09', 1, 'admin');

