package com.mortals;

import com.mortals.mqtt.broker.common.config.BrokerConfig;
import com.mortals.mqtt.broker.common.config.NettyConfig;
import com.mortals.mqtt.broker.common.helper.MixAll;
import com.mortals.mqtt.broker.services.MqttBrokerService;
import org.apache.commons.cli.*;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.Level;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.core.LoggerContext;
import org.apache.logging.log4j.core.config.Configuration;
import org.apache.logging.log4j.core.config.LoggerConfig;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.util.Map;
import java.util.Objects;
import java.util.Properties;

/**
 * 启动类
 */
public class MQServer {
    private static final Logger log = LoggerFactory.getLogger(MQServer.class);


    public static void main(String[] args) {
        try {
            //String filePath = MQServer.class.getClassLoader().getResource("conf//mq.properties").getPath();
            //System.out.println("filePath:"+filePath);


            //启动mqtt service
            startMqttService(args);
            //todo 启动coap service

            //todo 启动http服务

        } catch (Exception e) {
            log.error("mqtt start failure,cause = " ,e);
            System.exit(-1);
        }
    }

    public static MqttBrokerService startMqttService(String[] args) throws Exception {
        //构建初始选项
        Options options = buildOptions();
        //初始化命令行解释器，解析启动参数
        CommandLineParser parser = new DefaultParser();
        CommandLine commandLine = parser.parse(options, args);
        String mqttHome = null;
        String logLevel = null;
        BrokerConfig brokerConfig = new BrokerConfig();
        NettyConfig nettyConfig = new NettyConfig();
        if (commandLine != null) {
            mqttHome = commandLine.getOptionValue("h");
            logLevel = commandLine.getOptionValue("l");
        }

        //获取工作路径
        if (StringUtils.isEmpty(mqttHome)) {
            mqttHome = brokerConfig.getMqttHome();
        }
        if (StringUtils.isEmpty(mqttHome)) {
            throw new Exception("please set MQTT_HOME.");
        }
        //读取配置文件
        String mqttConfigPath = mqttHome + File.separator + "conf" + File.separator + "mq.properties";
        //初始化mqtt配置
        initConfig(mqttConfigPath, brokerConfig, nettyConfig);
        brokerConfig.setMqttHome(mqttHome);
        // 日志配置加载
        try {
            LoggerContext context = (LoggerContext) LogManager.getContext(false);
            File file = new File(mqttHome + File.separator + "conf" + File.separator + "log4j2.xml");
            context.setConfigLocation(file.toURI());
            Configuration configuration = context.getConfiguration();
            Map<String, LoggerConfig> loggerConfigMap = configuration.getLoggers();
            Level newLevel = logLevel == null ? null : Level.getLevel(logLevel);
            if (newLevel == null) {
                newLevel = Level.INFO;
            }
            for (LoggerConfig value : loggerConfigMap.values()) {
                value.setLevel(newLevel);
            }
            //更新日志配置
            context.updateLoggers(configuration);
        } catch (Exception ex) {
            System.err.print("Log4j2 load error,ex:" + ex);
        }


      //  final MqId mqId = new MqId();
      //  System.out.println("This MQ ID is " + mqId.getMqId());

        // 启动mqtt服务，线程等
        MqttBrokerService mqttBrokerService = new MqttBrokerService(brokerConfig, nettyConfig);
        mqttBrokerService.start();

        Runtime.getRuntime().addShutdownHook(new Thread(new Runnable() {
            @Override
            public void run() {
                mqttBrokerService.shutdown();
            }
        }));

        return mqttBrokerService;
    }

    private static Options buildOptions() {
        Options options = new Options();
        //设置home
        Option opt = new Option("h", true, "mqttHome,eg: /wls/xxx");
        opt.setRequired(false);
        options.addOption(opt);
        //设置propertites
        opt = new Option("c", true, "mq.properties path,eg: /wls/xxx/xxx.properties");
        opt.setRequired(false);
        options.addOption(opt);


        //设置日志等级
        opt = new Option("l", true, "DEBUG");
        opt.setRequired(false);
        options.addOption(opt);

        return options;
    }

    /**
     * convert properties to java config class
     *
     * @param mqttConfigPath
     * @param brokerConfig
     * @param nettyConfig
     */
    private static void initConfig(String mqttConfigPath, BrokerConfig brokerConfig, NettyConfig nettyConfig) {
        Properties properties = new Properties();
        BufferedReader bufferedReader = null;
        try {
            bufferedReader = new BufferedReader(new FileReader(mqttConfigPath));
            properties.load(bufferedReader);
            MixAll.properties2POJO(properties, brokerConfig);
            MixAll.properties2POJO(properties, nettyConfig);
        } catch (FileNotFoundException e) {
            System.out.println("mq.properties cannot find,cause + " + e + ",path:" + mqttConfigPath);
        } catch (IOException e) {
            System.out.println("Handle mqttConfig IO exception,cause = " + e);
        } finally {
            try {
                if (Objects.nonNull(bufferedReader)) {
                    bufferedReader.close();
                }
            } catch (IOException e) {
                System.out.println("Handle mqttConfig IO exception,cause = " + e);
            }
        }
    }

    public static void close() {

    }

}
