/**
 * Copyright © 2016-2021 The Thingsboard Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.mortals.coap;

import lombok.extern.slf4j.Slf4j;
import org.eclipse.californium.core.CoapServer;
import org.eclipse.californium.core.server.resources.Resource;

import java.net.UnknownHostException;
import java.util.concurrent.ConcurrentMap;
import java.util.concurrent.ScheduledExecutorService;

@Slf4j
public class DefaultCoapServerService implements CoapServerService {

//    @Autowired
//    private CoapServerContext coapServerContext;

    private CoapServer server;

  //  private TbCoapDtlsCertificateVerifier tbDtlsCertificateVerifier;

    private ScheduledExecutorService dtlsSessionsExecutor;

    public void init() throws UnknownHostException {
        createCoapServer();
    }

    public void shutdown() {
        if (dtlsSessionsExecutor != null) {
            dtlsSessionsExecutor.shutdownNow();
        }
        log.info("Stopping CoAP server!");
        server.destroy();
        log.info("CoAP server stopped!");
    }

    @Override
    public CoapServer getCoapServer() throws UnknownHostException {
        if (server != null) {
            return server;
        } else {
            return createCoapServer();
        }
    }

    @Override
    public ConcurrentMap<String, TbCoapDtlsSessionInfo> getDtlsSessionsMap() {

        return null;

       // return tbDtlsCertificateVerifier != null ? tbDtlsCertificateVerifier.getTbCoapDtlsSessionIdsMap() : null;
    }

    @Override
    public long getTimeout() {
        return 0L;
        //return coapServerContext.getTimeout();
    }

    private CoapServer createCoapServer() throws UnknownHostException {
        server = new CoapServer();
//
//        CoapEndpoint.Builder noSecCoapEndpointBuilder = new CoapEndpoint.Builder();
//        InetAddress addr = InetAddress.getByName(coapServerContext.getHost());
//        InetSocketAddress sockAddr = new InetSocketAddress(addr, coapServerContext.getPort());
//        noSecCoapEndpointBuilder.setInetSocketAddress(sockAddr);
//        noSecCoapEndpointBuilder.setNetworkConfig(NetworkConfig.getStandard());
//        CoapEndpoint noSecCoapEndpoint = noSecCoapEndpointBuilder.build();
//        server.addEndpoint(noSecCoapEndpoint);

        //udp加密
//        if (isDtlsEnabled()) {
//            CoapEndpoint.Builder dtlsCoapEndpointBuilder = new CoapEndpoint.Builder();
//            TbCoapDtlsSettings dtlsSettings = coapServerContext.getDtlsSettings();
//            DtlsConnectorConfig dtlsConnectorConfig = dtlsSettings.dtlsConnectorConfig();
//            DTLSConnector connector = new DTLSConnector(dtlsConnectorConfig);
//            dtlsCoapEndpointBuilder.setConnector(connector);
//            CoapEndpoint dtlsCoapEndpoint = dtlsCoapEndpointBuilder.build();
//            server.addEndpoint(dtlsCoapEndpoint);
//            if (dtlsConnectorConfig.isClientAuthenticationRequired()) {
//                tbDtlsCertificateVerifier = (TbCoapDtlsCertificateVerifier) dtlsConnectorConfig.getAdvancedCertificateVerifier();
//                dtlsSessionsExecutor = Executors.newSingleThreadScheduledExecutor();
//                dtlsSessionsExecutor.scheduleAtFixedRate(this::evictTimeoutSessions, new Random().nextInt((int) getDtlsSessionReportTimeout()), getDtlsSessionReportTimeout(), TimeUnit.MILLISECONDS);
//            }
//        }
        Resource root = server.getRoot();
        //TbCoapServerMessageDeliverer messageDeliverer = new TbCoapServerMessageDeliverer(root);
       // server.setMessageDeliverer(messageDeliverer);

        server.start();
        return server;
    }

//    private boolean isDtlsEnabled() {
//        return coapServerContext.getDtlsSettings() != null;
//    }
//
//    private void evictTimeoutSessions() {
//        tbDtlsCertificateVerifier.evictTimeoutSessions();
//    }
//
//    private long getDtlsSessionReportTimeout() {
//        return tbDtlsCertificateVerifier.getDtlsSessionReportTimeout();
//    }

}
