package com.mortals.common.cluster.dispatcher.akka;

import akka.actor.typed.ActorRef;
import akka.actor.typed.ActorSystem;
import akka.actor.typed.Behavior;
import akka.actor.typed.javadsl.Behaviors;
import akka.actor.typed.pubsub.Topic;
import akka.actor.typed.pubsub.Topic.Command;
import com.alibaba.fastjson.JSON;
import com.mortals.mqtt.broker.common.config.BrokerConfig;
import com.mortals.mqtt.broker.common.log.LogUtil;
import com.mortals.mqtt.broker.common.log.MqttLogger;
import com.mortals.common.cluster.dispatcher.ClusterEventHandler;
import com.mortals.common.cluster.dispatcher.EventConsumeHandler;
import com.mortals.common.cluster.dispatcher.event.Event;
import com.typesafe.config.Config;
import com.typesafe.config.ConfigFactory;
import org.slf4j.Logger;

import java.io.File;
import java.util.List;


/**
 * akka集群的分布式订阅与发布
 */
public class AkkaClusterEventHandler implements ClusterEventHandler {

    private static final Logger log = MqttLogger.eventLog;
    private ActorRef<Command<Event>> topic;
    private ActorRef<Event> subscriber;
    private EventConsumeHandler eventConsumeHandler;

    @Override
    public void start(BrokerConfig brokerConfig) {
        //读取配置文件信息
        //Config config = ConfigFactory.load("conf/akka");
        String path = brokerConfig.getMqttHome() + File.separator + "conf" + File.separator + "akka.conf";
        //System.out.println("path:"+path);
        Config config =  ConfigFactory.parseFile(new File(path));

        //System.out.println(config.atKey("akka").atKey("remote").atKey("artery").atKey("canonical").atKey("port").toString());
        //Config config = ConfigFactory.load("conf/akka");
        //创建 Akka system 容器 格式固定
        Behavior<Void> initBehavior = Behaviors.setup(
            context -> {
                //创建一个topic 建议一个actor只创建一个topic
                topic = context.spawn(Topic.create(Event.class, "mqtt-event"), "MqttEvent");
                //创建订阅者
                subscriber = context.spawn(Subscriber.create(this.eventConsumeHandler), "MqttEventSubscriber");
                //发布通知给所有订阅者
                topic.tell(Topic.subscribe(subscriber));

                context.spawn(ClusterListener.create(), "ClusterListener");
                return Behaviors.empty();
            });


      //  log.info("akka config {}",config.atKey("akka").toString());
        ActorSystem.create(initBehavior, "MqttDispatcherSystem", config);

    }

    @Override
    public void shutdown() {
        //退订
        topic.tell(Topic.unsubscribe(subscriber));

    }

    @Override
    public boolean sendEvent(Event event) {
        LogUtil.debug(log, "[AKKA]PUBLISH EVENT:{}", JSON.toJSONString(event));
        //发布事件
        topic.tell(Topic.publish(event));
        return true;
    }

    @Override
    public void setEventConsumeHandler(EventConsumeHandler eventConsumeHandler) {
        this.eventConsumeHandler = eventConsumeHandler;
    }

    @Override
    public List<Event> pollEvent(int maxPollNum) {
        //todo 拉取事件
        return null;
    }
}
