package com.mortals.common.cluster.dispatcher.akka;

import akka.actor.typed.Behavior;
import akka.actor.typed.javadsl.AbstractBehavior;
import akka.actor.typed.javadsl.ActorContext;
import akka.actor.typed.javadsl.Behaviors;
import akka.actor.typed.javadsl.Receive;
import com.mortals.mqtt.broker.common.log.MqttLogger;
import com.mortals.mqtt.broker.common.log.LogUtil;
import com.mortals.common.cluster.dispatcher.EventConsumeHandler;
import com.mortals.common.cluster.dispatcher.event.Event;
import org.slf4j.Logger;


/**
 * 封装订阅行为
 */
public class Subscriber extends AbstractBehavior<Event> {

    private static final Logger log = MqttLogger.eventLog;

    private EventConsumeHandler eventConsumeHandler;


    private Subscriber(ActorContext<Event> context, EventConsumeHandler eventConsumeHandler) {
        super(context);
        this.eventConsumeHandler = eventConsumeHandler;
    }

    public static Behavior<Event> create(EventConsumeHandler eventConsumeHandler) {
        //创建actor 行为
        return Behaviors.setup(context -> new Subscriber(context, eventConsumeHandler));
    }

    @Override
    public Receive<Event> createReceive() {
        //创建接收后，执行onReceive方法
        return newReceiveBuilder().onMessage(Event.class, this::onReceive).build();
    }

    private Behavior<Event> onReceive(Event event) {
        LogUtil.debug(log, "[Subscriber:{}] onReceive:{}", getContext().getSelf().path(), event);
        //执行消费事件
        this.eventConsumeHandler.consumeEvent(event);
        return this;
    }
}
