package com.mortals.common.queue.kafka;

import com.mortals.mqtt.broker.common.config.BrokerConfig;
import com.mortals.common.queue.*;
import org.apache.kafka.clients.consumer.ConsumerConfig;
import org.apache.kafka.clients.producer.ProducerConfig;
import org.apache.kafka.common.serialization.ByteArrayDeserializer;
import org.apache.kafka.common.serialization.StringDeserializer;

import java.util.Properties;

/**
 * @author: finegirl
 * @date: 2021/5/21 14:58
 * @description: kafka消息队列工厂类
 **/
public class KafkaMsgQueueFactory implements MessageQueueFactory {
    @Override
    public TbQueueProducer<TbQueueMsg> createMsgProducer(BrokerConfig brokerConfig) {
        TbKafkaProducerTemplate.TbKafkaProducerTemplateBuilder<TbQueueMsg> builder = TbKafkaProducerTemplate.builder();
        //初始化生产者
        Properties props = new Properties();
        props.put(ProducerConfig.BOOTSTRAP_SERVERS_CONFIG, brokerConfig.getKafkaServerAddr());
        props.put(ProducerConfig.ACKS_CONFIG, brokerConfig.getKafkaAcks());
        props.put(ProducerConfig.RETRIES_CONFIG, brokerConfig.getKafkaRetries());
        props.put(ProducerConfig.BATCH_SIZE_CONFIG, brokerConfig.getKafkaBatchSize());
        props.put(ProducerConfig.LINGER_MS_CONFIG, brokerConfig.getKafkaLingerMs());
        props.put(ProducerConfig.BUFFER_MEMORY_CONFIG, brokerConfig.getKafkaBufferMemory());
        props.put(ProducerConfig.KEY_SERIALIZER_CLASS_CONFIG, brokerConfig.getKafkaKeySerializerClass());
        props.put(ProducerConfig.VALUE_SERIALIZER_CLASS_CONFIG, brokerConfig.getKafkaValueSerializerClass());
        return builder.props(props).build();

    }

    @Override
    public TbQueueConsumer<TbQueueMsg> createMsgConsumer(BrokerConfig brokerConfig) {
        Properties props = new Properties();

        props.put(ConsumerConfig.BOOTSTRAP_SERVERS_CONFIG, brokerConfig.getKafkaServerAddr());

        props.put(ConsumerConfig.AUTO_OFFSET_RESET_CONFIG,"earliest");
        //group.id相同的属于同一个消费者组
        // props.put(ConsumerConfig.GROUP_ID_CONFIG, brokerConfig.getGroupId());
        props.put(ConsumerConfig.ENABLE_AUTO_COMMIT_CONFIG, brokerConfig.getAutoCommit());
        //自动提交offset,每1s提交一次
        props.put(ConsumerConfig.AUTO_COMMIT_INTERVAL_MS_CONFIG, brokerConfig.getAutoCommitInterval());
//            props.put(ConsumerConfig.KEY_DESERIALIZER_CLASS_CONFIG, brokerConfig.getKafkaKeySerializerClass());
//            props.put(ConsumerConfig.VALUE_DESERIALIZER_CLASS_CONFIG, brokerConfig.getKafkaValueSerializerClass());

        props.put(ConsumerConfig.KEY_DESERIALIZER_CLASS_CONFIG, StringDeserializer.class);
        props.put(ConsumerConfig.VALUE_DESERIALIZER_CLASS_CONFIG, ByteArrayDeserializer.class);

        TbKafkaConsumerTemplate.TbKafkaConsumerTemplateBuilder<TbQueueMsg> builder = TbKafkaConsumerTemplate.builder();
        builder.clientId(brokerConfig.getClientId());
        builder.topic(brokerConfig.getKafkaConsumerTopic());
        builder.groupId(brokerConfig.getGroupId());
        builder.decoder(msg -> new DefaultTbQueueMsg(msg));
        builder.props(props);

        return builder.build();
    }
}
