/**
 * Copyright © 2016-2021 The Thingsboard Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.mortals.common.scheduler;

import java.util.concurrent.Callable;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;

public interface SchedulerComponent {


    /**
     * 带延迟时间的调度，只执行一次
     * 调度之后可通过Future.get()阻塞直至任务执行完毕
     */
    ScheduledFuture<?> schedule(Runnable command, long delay, TimeUnit unit);
    /**
     * 带延迟时间的调度，只执行一次
     * 调度之后可通过Future.get()阻塞直至任务执行完毕，并且可以获取执行结果
     */
    <V> ScheduledFuture<V> schedule(Callable<V> callable, long delay, TimeUnit unit);
    /**
     * 带延迟时间的调度，循环执行，固定频率
     */
    ScheduledFuture<?> scheduleAtFixedRate(Runnable command, long initialDelay, long period, TimeUnit unit);
    /**
     * 带延迟时间的调度，循环执行，固定延迟
     */
    ScheduledFuture<?> scheduleWithFixedDelay(Runnable command, long initialDelay, long delay, TimeUnit unit);

}
