package com.mortals.mqtt.broker.bootstrap;

import com.mortals.mqtt.broker.common.log.MqttLogger;
import com.mortals.mqtt.broker.common.log.LogUtil;
import com.mortals.mqtt.broker.common.model.Message;
import com.mortals.common.cluster.dispatcher.InnerMessageDispatcher;
import com.mortals.mqtt.broker.bootstrap.netty.ChannelEventListener;
import com.mortals.mqtt.broker.bootstrap.session.ConnectManager;
import com.mortals.mqtt.broker.bootstrap.util.NettyUtil;
import com.mortals.mqtt.broker.store.MessageStore;
import com.mortals.mqtt.broker.store.SessionState;
import com.mortals.mqtt.broker.store.SessionStore;
import io.netty.channel.Channel;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;

/**
 * 通道生命周期事件
 */
public class ClientLifeCycleHookService implements ChannelEventListener {

    private static final Logger log = MqttLogger.clientTraceLog;
    private MessageStore messageStore;
    private SessionStore sessionStore;
    private InnerMessageDispatcher innerMessageDispatcher;

    public ClientLifeCycleHookService(MessageStore messageStore,SessionStore sessionStore, InnerMessageDispatcher innerMessageDispatcher) {
        this.messageStore = messageStore;
        this.sessionStore=sessionStore;
        this.innerMessageDispatcher = innerMessageDispatcher;
    }

    @Override
    public void onChannelConnect(String remoteAddr, Channel channel) {
        LogUtil.info(log, "[ChannelConnect] remoteAddr -> {} ", remoteAddr);

    }

    @Override
    public void onChannelClose(String remoteAddr, Channel channel) {
        LogUtil.info(log, "[ChannelConnect] remoteAddr -> {} ", remoteAddr);
        //获取通道用户id,获取遗愿消息。
        String clientId = NettyUtil.getClientId(channel);

        if (StringUtils.isNotEmpty(clientId)) {
            SessionState sessionState = new SessionState(SessionState.StateEnum.OFFLINE, System.currentTimeMillis());
            sessionStore.storeSession(clientId, sessionState);
            //非正常关闭，查看是否有遗愿消息
            Message willMessage = messageStore.getWillMessage(clientId);
            if (willMessage != null) {
                innerMessageDispatcher.appendMessage(willMessage);
            }
        }


    }

    @Override
    public void onChannelIdle(String remoteAddr, Channel channel) {
    }

    @Override
    public void onChannelException(String remoteAddr, Channel channel) {
        String clientId = NettyUtil.getClientId(channel);
        ConnectManager.getInstance().removeClient(clientId);
        //更新数据库session在线状态

        LogUtil.warn(log, "[ClientLifeCycleHook] -> {} channelException,close channel and remove ConnectCache!", clientId);
    }
}
