package com.mortals.mqtt.broker.bootstrap.netty;

import com.mortals.mqtt.broker.common.log.MqttLogger;
import com.mortals.mqtt.broker.common.log.LogUtil;
import org.slf4j.Logger;

import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.TimeUnit;

public class NettyEventExecutor implements Runnable {

    private static final Logger log = MqttLogger.remotingLog;

    private LinkedBlockingQueue<NettyEvent> eventQueue = new LinkedBlockingQueue<>();
    private final int maxSize = 100000;
    private ChannelEventListener listener;
    boolean stopped = false;
    private Thread thread;

    public NettyEventExecutor(ChannelEventListener channelEventListener) {
        this.listener = channelEventListener;
    }

    public void putNettyEvent(final NettyEvent nettyEvent) {
        if (this.eventQueue.size() <= maxSize) {
            this.eventQueue.add(nettyEvent);
        } else {
            LogUtil.warn(log, "[NettyEvent] -> event queue size[{}] enough, so drop this event {}", this.eventQueue.size(), nettyEvent.toString());
        }
    }

    @Override
    public void run() {
        while (!this.stopped) {
            try {
                NettyEvent nettyEvent = this.eventQueue.poll(3000, TimeUnit.MILLISECONDS);
                if (nettyEvent != null && listener != null) {
                    switch (nettyEvent.getEventType()) {
                        case CONNECT:
                            listener.onChannelConnect(nettyEvent.getRemoteAddr(), nettyEvent.getChannel());
                            break;
                        case CLOSE:
                            listener.onChannelClose(nettyEvent.getRemoteAddr(), nettyEvent.getChannel());
                            break;
                        case EXCEPTION:
                            listener.onChannelException(nettyEvent.getRemoteAddr(), nettyEvent.getChannel());
                            break;
                        case IDLE:
                            listener.onChannelIdle(nettyEvent.getRemoteAddr(), nettyEvent.getChannel());
                            break;
                        default:
                            break;
                    }
                }
            } catch (Throwable t) {
                LogUtil.warn(log, "[NettyEvent] -> service has exception. ", t);
            }
        }
        LogUtil.info(log, "[NettyEvent] -> NettyEventExecutor service end");
    }

    public void start() {
        this.thread = new Thread(this);
        this.thread.start();
    }

    public void shutdown() {
        this.stopped = true;
    }
}
