package com.mortals.mqtt.broker.bootstrap.netty;

import com.mortals.mqtt.broker.common.helper.Pair;
import com.mortals.mqtt.broker.common.log.LogUtil;
import com.mortals.mqtt.broker.common.log.MqttLogger;
import com.mortals.mqtt.broker.processor.RequestProcessor;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInboundHandlerAdapter;
import io.netty.handler.codec.mqtt.MqttMessage;
import io.netty.handler.codec.mqtt.MqttMessageType;
import lombok.AllArgsConstructor;
import org.slf4j.Logger;

import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.RejectedExecutionException;

/**
 * @author: finegirl
 * @date: 2021/5/12 13:30
 * @description:
 **/

@AllArgsConstructor
public class NettyMqttHandler extends ChannelInboundHandlerAdapter {
    private static final Logger log = MqttLogger.remotingLog;
    private Map<MqttMessageType, Pair<RequestProcessor, ExecutorService>> processorTable;
    @Override
    public void channelRead(ChannelHandlerContext ctx, Object obj) {
        MqttMessage mqttMessage = (MqttMessage) obj;
        if (mqttMessage != null && mqttMessage.decoderResult().isSuccess()) {
            MqttMessageType messageType = mqttMessage.fixedHeader().messageType();
            LogUtil.debug(log, "[Remoting] -> receive mqtt code,type:{},name:{}", messageType.value(), messageType.name());
            Runnable runnable = () -> processorTable.get(messageType).getObject1().processRequest(ctx, mqttMessage);
            try {
                processorTable.get(messageType).getObject2().submit(runnable);
            } catch (RejectedExecutionException ex) {
                LogUtil.warn(log, "Reject mqtt request,cause={}", ex.getMessage());
            }
        } else {
            ctx.close();
        }
    }
}
