package com.mortals.mqtt.broker.bootstrap.util;

import io.netty.channel.Channel;
import io.netty.handler.codec.mqtt.MqttMessage;
import io.netty.util.AttributeKey;

import java.io.IOException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.SocketAddress;
import java.util.concurrent.atomic.AtomicInteger;

public class NettyUtil {

    private static final String CLIENT_ID = "clientId";
    private static final String PRODUCT_ID = "productId";
    private static final String CLEARSTART = "clearStart";

    private static final AttributeKey<String> clientIdAttr = AttributeKey.valueOf(CLIENT_ID);
    private static final AttributeKey<String> productAttr = AttributeKey.valueOf(PRODUCT_ID);
    private static final AttributeKey<Boolean> clearStartAttr = AttributeKey.valueOf(CLEARSTART);

    //通道最大发送数据限制
    private static final AttributeKey<Long> MAX_PACKET_SIZE_SEND = AttributeKey.valueOf("Restriction.MaxPacketSize.Send");


    /**
     * broker收到connect连接的时间
     */
    public static final AttributeKey<Long> CONNECT_RECEIVED_TIMESTAMP = AttributeKey.valueOf("Connect.Received.Timestamp");


    /**
     * session失效时间
     */
    public static final AttributeKey<Long> CLIENT_SESSION_EXPIRY_INTERVAL = AttributeKey.valueOf("ClientSession.ExpiryInterval");


    /**
     * 消息从队列中获取，但未发送成功的数量
     */
    public static final AttributeKey<AtomicInteger> IN_FLIGHT_MESSAGES = AttributeKey.valueOf("Client.InFlightMessages");


    public static final AttributeKey<Boolean> DISCONNECT_EVENT_LOGGED = AttributeKey.valueOf("Disconnect.Event.Logged");


    public static final void setClientId(Channel channel, String clientId) {
        channel.attr(clientIdAttr).set(clientId);
    }

    public static final String getClientId(Channel channel) {
        return channel.attr(clientIdAttr).get();
    }


    public static final void setClearStart(Channel channel, Boolean clearStart) {
        channel.attr(clearStartAttr).set(clearStart);
    }

    public static final Boolean getClearStart(Channel channel) {
        return channel.attr(clearStartAttr).get();
    }


    public static final void setSessionExpiry(Channel channel, Long interval) {
        channel.attr(CLIENT_SESSION_EXPIRY_INTERVAL).set(interval);
    }

    public static final Long getSessionExpiry(Channel channel) {
        return channel.attr(CLIENT_SESSION_EXPIRY_INTERVAL).get();
    }


    public static final void setProductId(Channel channel, String productId) {
        channel.attr(productAttr).set(productId);
    }

    public static final String getProductId(Channel channel) {
        return channel.attr(productAttr).get();
    }

    //校验消息是否为mqtt消息
    public static MqttMessage validateMessage(Object message) throws IOException, ClassCastException {
        MqttMessage msg = (MqttMessage) message;
        if (msg.decoderResult() != null && msg.decoderResult().isFailure()) {
            throw new IOException("invalid massage", msg.decoderResult().cause());
        }
        if (msg.fixedHeader() == null) {
            throw new IOException("Unknown packet, no fixedHeader present, no cause provided");
        }
        return msg;
    }


    public static String getChannelIP(final Channel channel) {

        final InetAddress inetAddress = getChannelAddress(channel);

        if (null != inetAddress) {
            return inetAddress.getHostAddress();
        }

        return null;
    }


    public static InetAddress getChannelAddress(final Channel channel) {

        final SocketAddress socketAddress = channel.remoteAddress();
        if (null != socketAddress) {
            //If this is not an InetAddress, we're treating this as if there's no address
            if (socketAddress instanceof InetSocketAddress) {
                return ((InetSocketAddress) socketAddress).getAddress();
            }
        }
        return null;
    }


}
