package com.mortals.mqtt.broker.common.config;

import com.codahale.metrics.MetricRegistry;
import com.mortals.MQServer;
import lombok.Data;

import java.io.File;

/**
 * broker基础配置
 */
@Data
public class BrokerConfig {

    // 配置conf文件的所在位置，logback，properties文件等所在位置
    private String mqttHome = MQServer.class.getClassLoader().getResource("").getPath();

    private String logLevel = "INFO";
    protected static final String PRE = "mqtt.broker.";
    private String version = "1.0.0";
    private boolean anonymousEnable = true;

    private int pollThreadNum = Runtime.getRuntime().availableProcessors() * 2;

    // 采用拉消息方式时，一次最多拉的消息数目
    private int maxPollEventNum = 10;
    private int pollWaitInterval = 10;//ms

    // plugin class config
    private String sessionStoreClass = "com.mortails.mqtt.broker.store.rdb.RDBSessionStore";
    private String messageStoreClass = "com.mortals.mqtt.broker.store.rdb.RDBMessageStore";
    private String authValidClass = "com.mortals.mqtt.broker.services.auth.DefaultAuthValid";
    private String clusterEventHandlerClass = "com.mortals.common.cluster.dispatcher.rdb.RDBClusterEventHandler";
    //消息生产与消费类
    private String msgQueueClass = "com.mortals.common.queue.kafka.KafkaMsgQueueFactory";

    /**
     * redis相关默认配置
     */
    private String redisHost = "127.0.0.1";
    private int redisPort = 6379;
    private Boolean pool=false;
    private int redisDb=0;
    private String redisPassword = "";
    private int maxWaitMills = 60 * 1000;
    private boolean testOnBorrow = true;
    private int minIdle = 20;
    private int maxTotal = 200;
    private int maxIdle = 50;

    /**
     * db相关默认配置
     */
    private String driver = "com.mysql.jdbc.Driver";
    private String url
            = "jdbc:mysql://localhost:3306/mqtt?characterEncoding=utf8&autoReconnect=true&failOverReadOnly=false"
            + "&maxReconnects=10&useSSL=false";
    private String username = "root";
    private String password = "12345678";

    // 是否启用高性能模式，高性能模式下：入栈消息，出栈消息等过程消息都会默认采用内存缓存，若为false，则会用具体实现的存储缓存这一阶段的消息
    private boolean highPerformance = true;


    /**
     * kafka相关默认配置
     */
    //是否启用kafka消息转发
    private boolean kafkaBrokerEnabled= true;
    //kafka地址 127.0.0.1:9092,127.0.0.1:9093，192.168.0.251:9092
    private String kafkaServerAddr="192.168.0.251:9092";
    //acks回令 如果必须等待回令,那么设置acks为all,否则，设置为-1,等待回令会有性能损耗
    private String kafkaAcks="-1";
    //重试次数
    private Integer kafkaRetries=3;
    //批量提交大小
    private Integer kafkaBatchSize=16384;
    //提交延迟等待时间（等待时间内可以追加提交）
    private Integer kafkaLingerMs=1;
    //缓存大小
    private Integer kafkaBufferMemory=33554432;
    //Key序列化
    private String kafkaKeySerializerClass="org.apache.kafka.common.serialization.StringSerializer";
    //Value序列化
    private String kafkaValueSerializerClass="org.apache.kafka.common.serialization.ByteArraySerializer";
    //配置策略类
    private String kafkaPartitionerClass="cn.wizzer.iot.mqtt.server.store.kafka.SimplePartitioner";
    //默认发送主题
    private String kafkaProducerTopic="mqtt_publish";
    //默认消费主题
    private String kafkaConsumerTopic="mqtt_consume";
    //消费消息间隔时间，单位毫秒
    private Long pollDuration;
    private String clientId;
    private String groupId;
    private String autoCommit;
    private String autoCommitInterval;

    private MetricRegistry metricRegistry = new MetricRegistry();

}
