package com.mortals.mqtt.broker.common.helper;

import com.mortals.mqtt.broker.common.log.MqttLogger;
import org.slf4j.Logger;

import java.math.BigInteger;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @author: finegirl
 * @date: 2021/5/11 9:40
 * @description:
 **/
public final class DateUtils {
    private static final Logger logger = MqttLogger.otherLog;
   // private static Logger logger = Logger.getLogger(DateUtils.class);
    public static final int YEAR_RETURN = 0;
    public static final int MONTH_RETURN = 1;
    public static final int DAY_RETURN = 2;
    public static final int HOUR_RETURN = 3;
    public static final int MINUTE_RETURN = 4;
    public static final int SECOND_RETURN = 5;
    public static final String P_yyyyMMdd = "yyyyMMdd";
    public static final String P_yyyyMM = "yyyyMM";
    public static final String P_yyyy_MM_dd_HH_mm_ss = "yyyy-MM-dd HH:mm:ss";
    public static final String P_yyyy_MM_dd_HH_mm = "yyyy-MM-dd HH:mm";
    public static final String P_yyyy_MM_dd = "yyyy-MM-dd";
    public static final String P_yyyy_M_d = "yyyy-M-d";
    public static final String P_yyyyMd = "yyyyMd";
    public static final String P_yyyy_MM = "yyyy-MM";
    public static final String P_yyyy_MM_dd_HH_mm_ss_SSS = "yyyy-MM-dd HH:mm:ss:SSS";
    public static final String P_yyyyMMddHHmmssSSS = "yyyyMMddHHmmssSSS";
    public static final String P_HH_mm_ss = "HH:mm:ss";
    public static final String P_HHmmssSSS = "HHmmssSSS";

    public DateUtils() {
    }

    public static long getBetween(Date beginDate, Date endDate, int returnPattern) {
        Calendar beginCalendar = Calendar.getInstance();
        Calendar endCalendar = Calendar.getInstance();
        beginCalendar.setTime(beginDate);
        endCalendar.setTime(endDate);
        switch(returnPattern) {
            case 0:
                return getByField(beginCalendar, endCalendar, 1);
            case 1:
                return getByField(beginCalendar, endCalendar, 1) * 12L + getByField(beginCalendar, endCalendar, 2);
            case 2:
                return getTime(beginDate, endDate) / 86400000L;
            case 3:
                return getTime(beginDate, endDate) / 3600000L;
            case 4:
                return getTime(beginDate, endDate) / 60000L;
            case 5:
                return getTime(beginDate, endDate) / 1000L;
            default:
                return 0L;
        }
    }

    public static long getBetween(String beginTime, String endTime, String formatPattern, int returnPattern) throws ParseException {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat(formatPattern);
        Date beginDate = simpleDateFormat.parse(beginTime);
        Date endDate = simpleDateFormat.parse(endTime);
        Calendar beginCalendar = Calendar.getInstance();
        Calendar endCalendar = Calendar.getInstance();
        beginCalendar.setTime(beginDate);
        endCalendar.setTime(endDate);
        switch(returnPattern) {
            case 0:
                return getByField(beginCalendar, endCalendar, 1);
            case 1:
                return getByField(beginCalendar, endCalendar, 1) * 12L + getByField(beginCalendar, endCalendar, 2);
            case 2:
                return getTime(beginDate, endDate) / 86400000L;
            case 3:
                return getTime(beginDate, endDate) / 3600000L;
            case 4:
                return getTime(beginDate, endDate) / 60000L;
            case 5:
                return getTime(beginDate, endDate) / 1000L;
            default:
                return 0L;
        }
    }

    private static long getByField(Calendar beginCalendar, Calendar endCalendar, int calendarField) {
        return (long)(endCalendar.get(calendarField) - beginCalendar.get(calendarField));
    }

    private static long getTime(Date beginDate, Date endDate) {
        return endDate.getTime() - beginDate.getTime();
    }

    public static final Date StrToDateTime(String str, String pattern) {
        Date returnDate = null;
        if (pattern == null) {
            pattern = "yyyy-MM-dd HH:mm:ss";
        }

        if (str != null && str.trim().length() != 0) {
            SimpleDateFormat sdf = new SimpleDateFormat(pattern);

            try {
                returnDate = sdf.parse(str);
                return returnDate;
            } catch (Exception var5) {
                logger.debug("时间格式化异常-->" + str + "-->" + pattern + "-->" + var5.getMessage());
                return returnDate;
            }
        } else {
            return null;
        }
    }

    public static final Date StringToDateTime(String str) {
        StringBuffer tmpStr = new StringBuffer();
        tmpStr.append(getThisYear().substring(0, 2)).append(str.substring(0, 2)).append("-").append(str.substring(2, 4)).append("-").append(str.substring(4, 6)).append(" ").append(str.substring(6, 8)).append(":").append(str.substring(8, 10));
        return StrToDateTime(tmpStr.toString());
    }

    public static final String getThisYear() {
        return getCurrDateTime("yyyy");
    }

    public static final Date StrToDateTime(String str) {
        if (str != null && str.trim().length() > 0) {
            int strLength = str.length();
            if (strLength < 11) {
                str = str + " 00:00:00";
            } else if (strLength > 11 && strLength < 14) {
                str = str + ":00:00";
            } else if (strLength > 14 && strLength < 17) {
                str = str + ":00";
            }
        }

        return StrToDateTime(str, "yyyy-MM-dd HH:mm:ss");
    }

    public static final Date StrToDate(String str) {
        return StrToDateTime(str, "yyyy-MM-dd");
    }

    public static final String getDateTimeStr(Date date, String pattern) {
        if (pattern == null || pattern.trim().length() == 0) {
            pattern = "yyyyMMdd";
        }

        return getDateTime(date, pattern);
    }

    public static final String getStrDate(Date date) {
        return getDateTime(date, "yyyy-MM-dd");
    }

    public static final String getDateTime(Date date, String pattern) {
        if (date == null) {
            return "";
        } else {
            if (pattern == null || pattern.trim().length() == 0) {
                pattern = "yyyy-MM-dd HH:mm:ss";
            }

            SimpleDateFormat sdf = new SimpleDateFormat(pattern);
            return sdf.format(date);
        }
    }

    public static final String getStrYearMonth(Date date) {
        return getDateTime(date, "yyyyMM");
    }

    public static final int getNumYearMonth(Date date) {
        String yearMonth = getStrYearMonth(date);
        return yearMonth == "" ? 0 : Integer.parseInt(yearMonth);
    }

    public static final String getStrDateTime(Date date) {
        return getDateTime(date, "yyyy-MM-dd HH:mm:ss");
    }

    public static final String getCurrStrDate() {
        return getCurrDateTime("yyyy-MM-dd");
    }

    public static final String getCurrStrDateTime() {
        return getCurrDateTime("yyyy-MM-dd HH:mm:ss");
    }

    public static final String getCurrDateTime(String pattern) {
        return getDateTime(new Date(), pattern);
    }

    public static final Date getCurrDate() {
        return StrToDateTime(getCurrStrDate(), "yyyy-MM-dd");
    }

    public static final Date getCurrDatetime() {
        return StrToDateTime(getCurrStrDateTime(), "yyyy-MM-dd HH:mm:ss");
    }

    public static String conver2Date(String str) {
        String date = null;
        if (str != null && str.trim().length() != 0) {
            String pattern = "yyyy-MM-dd HH:mm:ss";
            date = convertDateStrToStr(str, "yyyy-MM-dd HH:mm:ss", pattern);
            if (date != null) {
                return date;
            } else {
                date = convertDateStrToStr(str, "yyyy-MM-dd HH:mm", pattern);
                if (date != null) {
                    return date;
                } else {
                    date = convertDateStrToStr(str, "yyyy-MM-dd", pattern);
                    if (date != null) {
                        return date;
                    } else {
                        date = convertDateStrToStr(str, "yyyyMMdd", pattern);
                        if (date != null) {
                            return date;
                        } else {
                            date = convertDateStrToStr(str, "yyyy-M-d", pattern);
                            if (date != null) {
                                return date;
                            } else {
                                date = convertDateStrToStr(str, "yyyyMd", pattern);
                                return date != null ? date : date;
                            }
                        }
                    }
                }
            }
        } else {
            return date;
        }
    }

    public static String convertDateStrToStr(String dateStr, String oldPattern, String newPattern) {
        try {
            SimpleDateFormat nowDate = new SimpleDateFormat(oldPattern);
            return getDateTime(nowDate.parse(dateStr), newPattern);
        } catch (Exception var4) {
            logger.debug("时间格式化异常-->" + dateStr + ",old=" + oldPattern + ",new=" + newPattern + "-->" + var4.getMessage());
            return null;
        }
    }

    public static String addCurrDateToStr(String pattern, int amount) {
        Calendar currCal = Calendar.getInstance();
        currCal.add(5, amount);
        SimpleDateFormat nowDate = new SimpleDateFormat(pattern);
        return nowDate.format(currCal.getTime());
    }

    public static Date getNextHalfMonthDate(Date date, String pattern) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        int day = cal.get(5);
        if (day < 16) {
            cal.set(5, 16);
        } else {
            cal.set(5, 1);
            cal.set(2, cal.get(2) + 1);
        }

        return StrToDateTime(getDateTime(cal.getTime(), pattern), pattern);
    }

    public static Date getNextMonthFirstDate(Date date, String pattern) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        cal.set(2, cal.get(2) + 1);
        cal.set(5, 1);
        return StrToDateTime(getDateTime(cal.getTime(), pattern), pattern);
    }

    public static Date getMonthLastDate(Date date, String pattern) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(getNextMonthFirstDate(date, pattern));
        cal.set(5, cal.get(5) - 1);
        return StrToDateTime(getDateTime(cal.getTime(), pattern), pattern);
    }

    public static Date getMonthFirstDate(Date date, String pattern) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        cal.set(5, 1);
        return StrToDateTime(getDateTime(cal.getTime(), pattern), pattern);
    }

    public static Date getLastMonthFirstDate(Date date, String pattern) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        cal.set(2, cal.get(2) - 1);
        cal.set(5, 1);
        return StrToDateTime(getDateTime(cal.getTime(), pattern), pattern);
    }

    public static Date getLastMonthLastDate(Date date, String pattern) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(getMonthFirstDate(date, pattern));
        cal.set(5, cal.get(5) - 1);
        return StrToDateTime(getDateTime(cal.getTime(), pattern), pattern);
    }

    public static boolean isSameDay(Date day1, Date day2) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        String ds1 = sdf.format(day1);
        String ds2 = sdf.format(day2);
        return ds1.equals(ds2);
    }

    public static Date addCurrDate(int amount) {
        Calendar currCal = Calendar.getInstance();
        currCal.add(5, amount);
        return currCal.getTime();
    }

    public static Date addDate(Date date, int amount) {
        Calendar currCal = Calendar.getInstance();
        currCal.setTime(date);
        currCal.add(5, amount);
        return currCal.getTime();
    }

    public static long getIdleDay(String startDateStr, String stopDateStr) {
        Date sDate = StrToDate(startDateStr);
        Date eDate = StrToDate(stopDateStr);
        return getIdleDay(sDate, eDate);
    }

    public static long getIdleDay(Date sDate, Date eDate) {
        if (sDate != null && eDate != null) {
            long idleDay = (eDate.getTime() - sDate.getTime()) / 1000L / 3600L / 24L;
            return idleDay;
        } else {
            return -1L;
        }
    }

    public static boolean isLegalHHMM(String hhss) {
        String reg = "(^([0-1][0-9])|2[0-3]):[0-5][0-9]$";
        Pattern pattern = Pattern.compile(reg);
        Matcher matcher = pattern.matcher(hhss);
        return matcher.matches();
    }

    public static String getPreviousMonth() {
        Calendar cal = Calendar.getInstance();
        cal.add(2, -1);
        return getDateTime(cal.getTime(), "yyyy-MM");
    }

    public static String getCurrMonth() {
        return getCurrDateTime("yyyy-MM");
    }

    public static String getPreviousMonth(String month) {
        return getPreviousMonth(StrToDateTime(month, "yyyy-MM"));
    }

    public static int addMonth(int month, int num) {
        return Integer.parseInt(addMonth(Integer.toString(month), num, "yyyyMM"));
    }

    public static String addMonth(String month, int num) {
        return addMonth(StrToDateTime(month, "yyyy-MM"), num);
    }

    public static String addMonth(String month, int num, String pattern) {
        return addMonth(StrToDateTime(month, pattern), num, pattern);
    }

    public static String addMonth(Date date, int num) {
        return addMonth(date, num, "yyyy-MM");
    }

    public static String addMonth(Date date, int num, String pattern) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        cal.add(2, num);
        return getDateTime(cal.getTime(), pattern);
    }

    public static String getPreviousMonth(Date date) {
        return addMonth((Date)date, -1);
    }

    public static String getEndMonth(String month) {
        Date date = StrToDateTime(month + "-01", "yyyy-MM-dd");
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        cal.add(2, 1);
        cal.add(5, -1);
        return getDateTime(cal.getTime(), "yyyy-MM-dd");
    }

    public static int getCurrWeekOfYear(Date date) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        return cal.get(3);
    }

    public static Date getDate(Long time) {
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss:SSS");
        String d = format.format(time);
        Date date = null;

        try {
            date = format.parse(d);
        } catch (ParseException var5) {
            logger.debug("时间格式化异常-->" + time + "-->" + var5.getMessage());
        }

        return date;
    }

    public static boolean currDateMiddleOfDate(Date before, Date after) {
        if (before == null && after == null) {
            return true;
        } else {
            Date currDate = new Date();
            return (before == null || before != null && currDate.after(before)) && (after == null || after != null && currDate.before(after));
        }
    }

    public static String convertTime2Str(long time) {
        if (time == 0L) {
            return "00:00:00";
        } else {
            Calendar cal = Calendar.getInstance();
            cal.set(0, 0, 0, 0, 0, (int)time);
            return getDateTime(cal.getTime(), "HH:mm:ss");
        }
    }

    public static String convertTime2Str(long time, String pattern) {
        if (pattern == null || pattern.trim().length() == 0) {
            pattern = "yyyy-MM-dd HH:mm:ss";
        }

        Calendar cal = Calendar.getInstance();
        cal.setTimeInMillis(time);
        return getDateTime(cal.getTime(), pattern);
    }

    public static boolean isDateBefore(Date date2) throws ParseException {
        Date date = new Date();
        return date.before(date2);
    }

    public static int convertStr2Time(String time) {
        if (time != null && time.trim().length() != 0) {
            try {
                Date currDate = StrToDateTime(time, "HH:mm:ss");
                Calendar cal = Calendar.getInstance();
                cal.setTime(currDate);
                return cal.get(10) * 3600 + cal.get(12) * 60 + cal.get(13);
            } catch (Exception var3) {
                logger.debug("时间格式化异常-->" + time + "-->" + var3.getMessage());
                return 0;
            }
        } else {
            return 0;
        }
    }

    public static BigInteger getDateNum(Date date) {
        if (date == null) {
            return new BigInteger("0");
        } else {
            SimpleDateFormat format = new SimpleDateFormat("yyyyMMddHHmmssSSS");
            String dd = format.format(date);
            BigInteger num = new BigInteger(dd);
            return num;
        }
    }

    public static int getTimeNumber(Date involveTime) {
        if (involveTime == null) {
            return 0;
        } else {
            SimpleDateFormat format = new SimpleDateFormat("HHmmssSSS");
            String dd = format.format(involveTime);
            int num = Integer.valueOf(dd);
            return num;
        }
    }

    public static long getCountDown() {
        Date date = new Date();
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm");
        SimpleDateFormat format2 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String dd = format.format(date);
        StringBuffer bf = new StringBuffer(dd);
        bf.append(":00");
        String ddd = bf.toString();

        try {
            date = format2.parse(ddd);
        } catch (ParseException var10) {
            logger.debug("时间格式化异常-->" + ddd + "-->" + var10.getMessage());
        }

        long now = date.getTime();
        long time = now + 60000L;
        return time;
    }

    public static Date getTodayLastTime() {
        Calendar cal = Calendar.getInstance();
        cal.set(11, 23);
        cal.set(12, 59);
        cal.set(13, 59);
        cal.set(14, 999);
        return cal.getTime();
    }

    public static Date getFirstTime(Date date) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        cal.set(11, 0);
        cal.set(12, 0);
        cal.set(13, 0);
        cal.set(14, 0);
        return cal.getTime();
    }

    public static Date getLastTime(Date date) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        cal.set(11, 23);
        cal.set(12, 59);
        cal.set(13, 59);
        cal.set(14, 999);
        return cal.getTime();
    }

    public static Date getTomorrowFirstTime() {
        Calendar cal = Calendar.getInstance();
        cal.add(5, 1);
        cal.set(11, 0);
        cal.set(12, 0);
        cal.set(13, 0);
        cal.set(14, 0);
        return cal.getTime();
    }

    public static Date getTomorrowFirstTimeWithDelay(int second) {
        Calendar cal = Calendar.getInstance();
        cal.add(5, 1);
        cal.set(11, 0);
        cal.set(12, 0);
        cal.set(13, 0);
        cal.set(14, 0);
        cal.add(13, second);
        return cal.getTime();
    }

    public static List<String[]> splitTime(Date beginDate, Date endDate, int splitType, String pattern) {
        Calendar start = Calendar.getInstance(Locale.CHINESE);
        start.setFirstDayOfWeek(2);
        start.setTime(beginDate);
        start.set(11, 0);
        start.set(12, 0);
        start.set(13, 0);
        start.set(14, 0);
        Calendar end = Calendar.getInstance(Locale.CHINESE);
        end.setFirstDayOfWeek(2);
        end.setTime(endDate);
        end.set(11, 0);
        end.set(12, 0);
        end.set(13, 0);
        end.set(14, 0);
        int growType = 6;
        switch(splitType) {
            case 1:
                growType = 6;
                break;
            case 2:
                growType = 3;
                break;
            case 3:
                growType = 2;
                break;
            case 4:
                growType = 1;
        }

        ArrayList retList = new ArrayList();

        while(start.compareTo(end) <= 0) {
            String[] values = new String[]{getDateTime(start.getTime(), pattern), null};
            int startValue = start.get(growType);
            start.add(growType, 1);
            if (start.after(end)) {
                start.setTime(end.getTime());
            }

            int endValue = start.get(growType);
            if (startValue == endValue) {
                values[1] = getDateTime(start.getTime(), pattern);
            } else {
                while(startValue != endValue) {
                    start.add(6, -1);
                    endValue = start.get(growType);
                    if (startValue == endValue) {
                        values[1] = getDateTime(start.getTime(), pattern);
                        break;
                    }
                }
            }

            start.add(6, 1);
            retList.add(values);
        }

        return retList;
    }

    public static Date addCurrDate(int cycleType, int cycleValue, boolean naturalCycle) {
        Calendar cal = Calendar.getInstance();
        switch(cycleType) {
            case 1:
                cal.add(11, cycleValue);
                if (naturalCycle) {
                    cal.set(12, 0);
                    cal.add(12, -1);
                    cal.set(13, 59);
                    cal.set(14, 999);
                }
                break;
            case 2:
                cal.add(6, cycleValue);
                if (naturalCycle) {
                    cal.add(6, -1);
                    cal.set(11, 23);
                    cal.set(12, 59);
                    cal.set(13, 59);
                    cal.set(14, 999);
                }
                break;
            case 3:
                cal.add(3, cycleValue);
                if (naturalCycle) {
                    cal.set(7, 2);
                    cal.add(6, -1);
                    cal.set(11, 23);
                    cal.set(12, 59);
                    cal.set(13, 59);
                    cal.set(14, 999);
                }
                break;
            case 4:
                cal.add(2, cycleValue);
                if (naturalCycle) {
                    cal.set(5, 1);
                    cal.add(6, -1);
                    cal.set(11, 23);
                    cal.set(12, 59);
                    cal.set(13, 59);
                    cal.set(14, 999);
                }
                break;
            case 5:
                cal.add(1, cycleValue);
                if (naturalCycle) {
                    cal.set(2, 0);
                    cal.set(5, 1);
                    cal.add(6, -1);
                    cal.set(11, 23);
                    cal.set(12, 59);
                    cal.set(13, 59);
                    cal.set(14, 999);
                }
        }

        return cal.getTime();
    }

    public static Date addCurrDateForLastCycleTime(int cycleType, int cycleValue) {
        return addCurrDate(cycleType, cycleValue + 1, true);
    }

    public static Date addCurrDateForLastDayTime(int cycleType, int cycleValue) {
        Calendar cal = Calendar.getInstance();
        switch(cycleType) {
            case 1:
                cal.add(11, cycleValue);
                cal.set(12, 59);
                cal.set(13, 59);
                cal.set(14, 999);
                break;
            case 2:
                cal.add(6, cycleValue);
                cal.set(11, 23);
                cal.set(12, 59);
                cal.set(13, 59);
                cal.set(14, 999);
                break;
            case 3:
                cal.add(3, cycleValue);
                cal.set(11, 23);
                cal.set(12, 59);
                cal.set(13, 59);
                cal.set(14, 999);
                break;
            case 4:
                cal.add(2, cycleValue);
                cal.set(11, 23);
                cal.set(12, 59);
                cal.set(13, 59);
                cal.set(14, 999);
                break;
            case 5:
                cal.add(1, cycleValue);
                cal.set(11, 23);
                cal.set(12, 59);
                cal.set(13, 59);
                cal.set(14, 999);
        }

        return cal.getTime();
    }

    public static void main(String[] args) {
        int cycleType = 5;
        int cycleValue = 1;

        for(String pattern = "yyyy-MM-dd HH:mm:ss.SSS"; cycleType > 0; --cycleType) {
            System.out.println(getDateTime(addCurrDateForLastDayTime(cycleType, cycleValue), pattern));
        }

    }
}
