/*
 * Copyright 2019-present HiveMQ GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.mortals.mqtt.broker.metrics.common;

import com.mortals.mqtt.broker.bootstrap.util.NettyUtil;
import com.mortals.mqtt.broker.dropping.MessageDroppedService;
import io.netty.channel.ChannelDuplexHandler;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelPromise;
import io.netty.handler.codec.mqtt.MqttMessage;
import io.netty.handler.codec.mqtt.MqttMessageType;
import io.netty.handler.codec.mqtt.MqttPublishMessage;
import io.netty.handler.codec.mqtt.MqttQoS;

@ChannelHandler.Sharable
public class GlobalMQTTMessageCounter extends ChannelDuplexHandler {

    private final  MetricsHolder metricsHolder;

    private MessageDroppedService messageDroppedService;

    public GlobalMQTTMessageCounter(final  MetricsHolder metricsHolder,final MessageDroppedService messageDroppedService) {
        this.metricsHolder = metricsHolder;
        this.messageDroppedService=messageDroppedService;
    }

    @Override
    public void channelRead(final ChannelHandlerContext ctx, final  Object message) throws Exception {
        metricsHolder.getIncomingMessageCounter().inc();
        MqttMessage msg = (MqttMessage) message;
        MqttMessageType messageType = msg.fixedHeader().messageType();
        switch (messageType) {
            case PUBLISH:
                metricsHolder.getIncomingPublishCounter().inc();
                break;
            case SUBSCRIBE:
                metricsHolder.getSubscriptionCounter().inc();
                break;
            case CONNECT:
                metricsHolder.getIncomingConnectCounter().inc();
                break;
            case DISCONNECT:
                metricsHolder.getClosedConnectionsCounter().inc();
                break;
            default:
                break;
        }

        super.channelRead(ctx, message);
    }

    @Override
    public void write(final ChannelHandlerContext ctx, final  Object message, final  ChannelPromise promise) throws Exception {
        //判断是否写成功
        if (!ctx.channel().isWritable()) {
            if (message instanceof MqttMessage) {
                MqttMessage msg = (MqttMessage) message;

                MqttMessageType messageType = msg.fixedHeader().messageType();
                String clientId = NettyUtil.getClientId(ctx.channel());
                switch (messageType) {
                    case PUBLISH:
                        MqttPublishMessage publishMessage = (MqttPublishMessage) msg;
                        MqttQoS qos = publishMessage.fixedHeader().qosLevel();
                        if(qos.value()== MqttQoS.AT_MOST_ONCE.value()){
                           messageDroppedService.notWritable(clientId, publishMessage.variableHeader().topicName(), qos.value());

                       }
                }
            }
        }else {
            metricsHolder.getOutgoingMessageCounter().inc();
            if (message instanceof MqttMessage) {
                MqttMessage msg = (MqttMessage) message;
                MqttMessageType messageType = msg.fixedHeader().messageType();
                switch (messageType) {
                    case PUBLISH:
                        metricsHolder.getOutgoingPublishCounter().inc();
                }
            }
        }

        super.write(ctx, message, promise);
    }

}
