/*
 * Copyright 2019-present HiveMQ GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.mortals.mqtt.broker.metrics.common;

import com.codahale.metrics.Gauge;
import com.codahale.metrics.Metric;



public class MQMetric<T extends Metric> {

    private final String name;
    private final Class<? extends Metric> clazz;


    private MQMetric(final String name, final Class<? extends Metric> clazz) {
        this.name = name;
        this.clazz = clazz;
    }

    public static <T extends Metric> MQMetric<T> valueOf(final String name, final Class<T> metricClass) {
       // checkNotNull(name, "Name cannot be null");

        return new MQMetric<>(name, metricClass);
    }

    public static MQMetric<Gauge<Number>> gaugeValue(final String name) {
        //checkNotNull(name, "Name cannot be null");

        return new MQMetric<>(name, Gauge.class);
    }

    public String name() {
        return name;
    }

    public Class<? extends Metric> getClazz() {
        return clazz;
    }
}