/*
 * Copyright 2019-present HiveMQ GmbH
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.mortals.mqtt.broker.metrics.common;

import com.codahale.metrics.Counter;
import com.codahale.metrics.MetricRegistry;
import lombok.Getter;

import static com.mortals.mqtt.broker.metrics.common.MQMetrics.*;


@Getter
public class MetricsHolder {
    private final MetricRegistry metricRegistry;
    private final Counter incomingMessageCounter;
    private final Counter outgoingMessageCounter;

    private final Counter incomingConnectCounter;

    private final Counter incomingPublishCounter;
    private final Counter outgoingPublishCounter;

    private final Counter droppedMessageCounter;

    private final Counter subscriptionCounter;

    private final Counter closedConnectionsCounter;

    public MetricsHolder(final MetricRegistry metricRegistry) {

        this.metricRegistry = metricRegistry;

        incomingMessageCounter = metricRegistry.counter(INCOMING_MESSAGE_COUNT.name());
        outgoingMessageCounter = metricRegistry.counter(OUTGOING_MESSAGE_COUNT.name());

        incomingConnectCounter = metricRegistry.counter(INCOMING_CONNECT_COUNT.name());

        incomingPublishCounter = metricRegistry.counter(INCOMING_PUBLISH_COUNT.name());
        outgoingPublishCounter = metricRegistry.counter(OUTGOING_PUBLISH_COUNT.name());

        droppedMessageCounter = metricRegistry.counter(DROPPED_MESSAGE_COUNT.name());

        closedConnectionsCounter = metricRegistry.counter(CONNECTIONS_CLOSED_COUNT.name());

        subscriptionCounter = metricRegistry.counter(SUBSCRIPTIONS_CURRENT.name());
    }

}
