package com.mortals.mqtt.broker.processor;



import com.mortals.mqtt.broker.services.MqttBrokerService;
import com.mortals.mqtt.broker.common.model.Message;
import com.mortals.mqtt.broker.store.SessionStore;
import com.mortals.mqtt.broker.store.memory.InflowMessageHandler;
import com.mortals.mqtt.broker.store.memory.OutflowMessageHandler;
import com.mortals.mqtt.broker.store.memory.OutflowSecMessageHandler;

import java.util.Collection;

public abstract class MemoryMessageHandler {

    // high performance message handle
    private InflowMessageHandler inflowMessageHandler;
    private OutflowMessageHandler outflowMessageHandler;
    private OutflowSecMessageHandler outflowSecMessageHandler;
    private boolean      highPerformance;
    private SessionStore sessionStore;

    public MemoryMessageHandler(MqttBrokerService mqttBrokerService){
        this.highPerformance = mqttBrokerService.getBrokerConfig().isHighPerformance();
        this.sessionStore = mqttBrokerService.getSessionStore();
        this.inflowMessageHandler = mqttBrokerService.getInflowMessageHandler();
        this.outflowMessageHandler = mqttBrokerService.getOutflowMessageHandler();
        this.outflowSecMessageHandler = mqttBrokerService.getOutflowSecMessageHandler();
    }

    protected boolean cacheInflowMsg(String clientId, Message message) {
        if (highPerformance) {
            return this.inflowMessageHandler.cacheInflowMsg(clientId,message);
        }
        return this.sessionStore.cacheInflowMsg(clientId,message);
    }

    protected Message releaseInflowMsg(String clientId,int msgId){
        if (highPerformance) {
            return this.inflowMessageHandler.releaseInflowMsg(clientId,msgId);
        }
        return sessionStore.releaseInflowMsg(clientId,msgId);
    }

    protected Collection<Message> getAllInflowMsg(String clientId){
        if (highPerformance) {
            return this.inflowMessageHandler.getAllInflowMsg(clientId);
        }
        return sessionStore.getAllInflowMsg(clientId);
    }

    protected boolean cacheOutflowMsg(String clientId,Message message) {
        //判断是否将消息保存到内存中还是持久化
        if (highPerformance) {
            return this.outflowMessageHandler.cacheOutflowMsg(clientId,message);
        }
        //持久化操作
        return this.sessionStore.cacheOutflowMsg(clientId,message);
    }

    protected Message releaseOutflowMsg(String clientId, int msgId){
        if (highPerformance) {
            return this.outflowMessageHandler.releaseOutflowMsg(clientId,msgId);
        }
        return this.sessionStore.releaseOutflowMsg(clientId,msgId);
    }

    protected Collection<Message> getAllOutflowMsg(String clientId){
        if (highPerformance) {
            return this.outflowMessageHandler.getAllOutflowMsg(clientId);
        }
        return this.sessionStore.getAllOutflowMsg(clientId);
    }

    protected boolean cacheOutflowSecMsgId(String clientId, int msgId) {
        if (highPerformance) {
            return this.outflowSecMessageHandler.cacheOutflowSecMsgId(clientId,msgId);
        }
        return this.sessionStore.cacheOutflowSecMsgId(clientId,msgId);
    }

    protected boolean releaseOutflowSecMsgId(String clientId, int msgId) {
        if (highPerformance) {
            return this.outflowSecMessageHandler.releaseOutflowSecMsgId(clientId,msgId);
        }
        return this.sessionStore.releaseOutflowSecMsgId(clientId,msgId);
    }

    protected Collection<Integer> getAllOutflowSecMsgId(String clientId) {
        if (highPerformance) {
            return this.outflowSecMessageHandler.getAllOutflowSecMsgId(clientId);
        }
        return this.sessionStore.getAllOutflowSecMsgId(clientId);
    }

}
