package com.mortals.mqtt.broker.processor.protocol;

import com.alibaba.fastjson.JSONObject;
import com.mortals.mqtt.broker.services.MqttBrokerService;
import com.mortals.mqtt.broker.common.model.Message;
import com.mortals.mqtt.broker.common.model.MessageHeader;
import com.mortals.common.cluster.dispatcher.ClusterEventHandler;
import com.mortals.mqtt.broker.processor.MemoryMessageHandler;
import com.mortals.common.cluster.dispatcher.event.Event;
import com.mortals.common.cluster.dispatcher.event.EventCode;
import com.mortals.mqtt.broker.store.MessageStore;

/**
 * 通用消息分发处理器
 * TODO mqtt5实现
 */
public abstract class AbstractMessageProcessor extends MemoryMessageHandler {

    protected MessageStore messageStore;
    protected ClusterEventHandler clusterEventHandler;
    protected String currentIp;

    public AbstractMessageProcessor(MqttBrokerService mqttBrokerService) {
        super(mqttBrokerService);
        this.messageStore = mqttBrokerService.getMessageStore();
        this.clusterEventHandler = mqttBrokerService.getClusterEventHandler();
        this.currentIp = mqttBrokerService.getCurrentIp();
    }

    protected void processMessage(Message message) {

        // 1. retain消息逻辑
        boolean retain = (boolean) message.getHeader(MessageHeader.RETAIN);
        if (retain) {
            int qos = (int) message.getHeader(MessageHeader.QOS);
            byte[] payload = message.getPayload();
            String topic = (String) message.getHeader(MessageHeader.TOPIC);
            //qos == 0 or payload is none,then clear previous retain message
            if (qos == 0 || payload == null || payload.length == 0) {
                this.messageStore.clearRetainMessage(topic);
            } else {
                this.messageStore.storeRetainMessage(topic, message);
            }
        }
        // 2. 向集群中分发消息：第一阶段
        sendMessage2Cluster(message);
    }

    /**
     * 向集群分发消息:第一阶段
     */
    private void sendMessage2Cluster(Message message) {
        Event event = new Event(EventCode.DISPATCHER_CLIENT_MESSAGE.getCode(), JSONObject.toJSONString(message),System.currentTimeMillis(),currentIp);
        this.clusterEventHandler.sendEvent(event);
    }


    /**
     * 向集群分发消息:第一阶段
     */
    private void sendContrlMessage2Cluster(Message message) {
        Event event = new Event(EventCode.DISPATCHER_CONTROL_CLIENT_MESSAGE.getCode(), JSONObject.toJSONString(message),System.currentTimeMillis(),currentIp);
        this.clusterEventHandler.sendEvent(event);
    }


    /**
     * 向集群分发客户端订阅topic消息
     */
    protected void sendSubsMessage2Cluster(Message message) {
        Event event = new Event(EventCode.DISPATCHER_CLIENT_SUBS_MESSAGE.getCode(), JSONObject.toJSONString(message),System.currentTimeMillis(),currentIp);
        this.clusterEventHandler.sendEvent(event);
    }

}
