package com.mortals.mqtt.broker.processor.protocol;

import com.mortals.mqtt.broker.services.MqttBrokerService;
import com.mortals.mqtt.broker.common.log.MqttLogger;
import com.mortals.mqtt.broker.common.log.LogUtil;
import com.mortals.mqtt.broker.processor.RequestProcessor;
import com.mortals.mqtt.broker.bootstrap.util.MessageUtil;
import com.mortals.mqtt.broker.bootstrap.util.NettyUtil;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.codec.mqtt.MqttMessage;
import org.slf4j.Logger;

/**
 * 出栈消息接收到的qos2消息的pubComp报文：清除缓存的出栈消息
 */
public class PubCompProcessor extends AbstractMessageProcessor implements RequestProcessor {

    private Logger log = MqttLogger.messageTraceLog;

    public PubCompProcessor(MqttBrokerService mqttBrokerService) {
        super(mqttBrokerService);
    }

    @Override
    public void processRequest(ChannelHandlerContext ctx, MqttMessage mqttMessage) {
        String clientId = NettyUtil.getClientId(ctx.channel());
        int messageId = MessageUtil.getMessageId(mqttMessage);
        boolean flag = releaseOutflowSecMsgId(clientId, messageId);
        LogUtil.debug(log, "[PubComp] -> Receive PubCom and remove the flow message,clientId={},msgId={}", clientId, messageId);
        if (!flag) {
            LogUtil.warn(log, "[PubComp] -> The message is not in Flow cache,clientId={},msgId={}", clientId, messageId);
        }
    }
}
