package com.mortals.mqtt.broker.processor.protocol;

import com.mortals.mqtt.broker.services.MqttBrokerService;
import com.mortals.mqtt.broker.common.log.MqttLogger;
import com.mortals.mqtt.broker.common.log.LogUtil;
import com.mortals.mqtt.broker.processor.RequestProcessor;
import com.mortals.mqtt.broker.bootstrap.util.MessageUtil;
import com.mortals.mqtt.broker.bootstrap.util.NettyUtil;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.codec.mqtt.MqttMessage;
import org.slf4j.Logger;

/**
 * 出栈消息接收到qos2第一阶段的的pubRec报文: 丢弃消息，存储报文标志符
 */
public class PubRecProcessor extends AbstractMessageProcessor implements RequestProcessor {

    private Logger log = MqttLogger.messageTraceLog;

    public PubRecProcessor(MqttBrokerService mqttBrokerService) {
        super(mqttBrokerService);
    }

    @Override
    public void processRequest(ChannelHandlerContext ctx, MqttMessage mqttMessage) {
        String clientId = NettyUtil.getClientId(ctx.channel());
        int messageId = MessageUtil.getMessageId(mqttMessage);

        releaseOutflowMsg(clientId, messageId);
        cacheOutflowSecMsgId(clientId, messageId);

        LogUtil.debug(log, "[PubRec] -> Receive PubRec message,clientId={},msgId={}", clientId, messageId);
        MqttMessage pubRelMessage = MessageUtil.getPubRelMessage(messageId);
        ctx.writeAndFlush(pubRelMessage);
    }
}
