package com.mortals.mqtt.broker.processor.protocol;

import com.mortals.mqtt.broker.services.MqttBrokerService;
import com.mortals.mqtt.broker.common.log.MqttLogger;
import com.mortals.mqtt.broker.common.log.LogUtil;
import com.mortals.mqtt.broker.common.model.Message;
import com.mortals.mqtt.broker.processor.RequestProcessor;
import com.mortals.mqtt.broker.bootstrap.session.ConnectManager;
import com.mortals.mqtt.broker.bootstrap.util.MessageUtil;
import com.mortals.mqtt.broker.bootstrap.util.NettyUtil;
import com.mortals.mqtt.broker.bootstrap.util.RemotingHelper;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.codec.mqtt.MqttMessage;
import org.slf4j.Logger;

import java.util.Objects;

/**
 * 客户端的pubRel报文：入栈消息qos2的第二阶段：
 * 1. 消息所有者转换为broker,开始分发消息
 * 2. 返回pubCom报文
 */
public class PubRelProcessor extends AbstractMessageProcessor implements RequestProcessor {

    private static final Logger log = MqttLogger.messageTraceLog;

    public PubRelProcessor(MqttBrokerService controller) {
        super(controller);
    }

    @Override
    public void processRequest(ChannelHandlerContext ctx, MqttMessage mqttMessage) {
        String clientId = NettyUtil.getClientId(ctx.channel());

        LogUtil.warn(log, "[PubRelMessage] ->clientId={},mqttMessage={}.", clientId, mqttMessage.toString());

        int msgId = MessageUtil.getMessageId(mqttMessage);
        if (ConnectManager.getInstance().containClient(clientId)) {
            Message message = releaseInflowMsg(clientId, msgId);
            if (Objects.nonNull(message)) {
                super.processMessage(message);
            } else {
                LogUtil.warn(log, "[PubRelMessage] -> the message is not exist,clientId={},messageId={}.", clientId, msgId);
            }
            MqttMessage pubComMessage = MessageUtil.getPubComMessage(msgId);
            ctx.writeAndFlush(pubComMessage);
        } else {
            LogUtil.warn(log, "[PubRelMessage] -> the client：{} disconnect to this server.", clientId);
            RemotingHelper.closeChannel(ctx.channel());
        }
    }
}
