package com.mortals.mqtt.broker.processor.protocol;

import com.alibaba.fastjson.JSON;
import com.mortals.mqtt.broker.common.log.MqttLogger;
import com.mortals.mqtt.broker.common.log.LogUtil;
import com.mortals.mqtt.broker.processor.RequestProcessor;
import com.mortals.mqtt.broker.bootstrap.session.ClientSession;
import com.mortals.mqtt.broker.bootstrap.session.ConnectManager;
import com.mortals.mqtt.broker.bootstrap.util.MessageUtil;
import com.mortals.mqtt.broker.bootstrap.util.NettyUtil;
import com.mortals.mqtt.broker.store.SessionStore;
import com.mortals.mqtt.broker.subscribe.SubscriptionMatcher;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.codec.mqtt.MqttMessage;
import io.netty.handler.codec.mqtt.MqttUnsubAckMessage;
import io.netty.handler.codec.mqtt.MqttUnsubscribeMessage;
import io.netty.handler.codec.mqtt.MqttUnsubscribePayload;
import org.slf4j.Logger;

import java.util.List;
import java.util.Objects;

/**
 * 取消订阅
 */
public class UnSubscribeProcessor implements RequestProcessor {

    private Logger log = MqttLogger.messageTraceLog;

    private SubscriptionMatcher subscriptionMatcher;
    private SessionStore sessionStore;

    public UnSubscribeProcessor(SubscriptionMatcher subscriptionMatcher, SessionStore sessionStore) {
        this.subscriptionMatcher = subscriptionMatcher;
        this.sessionStore = sessionStore;
    }

    @Override
    public void processRequest(ChannelHandlerContext ctx, MqttMessage mqttMessage) {
        MqttUnsubscribeMessage unsubscribeMessage = (MqttUnsubscribeMessage) mqttMessage;
        MqttUnsubscribePayload unsubscribePayload = unsubscribeMessage.payload();
        List<String> topics = unsubscribePayload.topics();
        String clientId = NettyUtil.getClientId(ctx.channel());
        ClientSession clientSession = ConnectManager.getInstance().getClient(clientId);
        if (Objects.isNull(clientSession)) {
            LogUtil.warn(log, "[UnSubscribe] -> The client is not online.clientId={}", clientId);
        }
        topics.forEach(topic -> {
            subscriptionMatcher.unSubscribe(topic, clientId);
           // sessionStore.delSubscription(clientId, topic);
        });
        MqttUnsubAckMessage unsubAckMessage = MessageUtil.getUnSubAckMessage(MessageUtil.getMessageId(mqttMessage));
        ctx.writeAndFlush(unsubAckMessage);

        LogUtil.debug(log, "[UnSubscribe] -> currentNode Subs={}", JSON.toJSONString(subscriptionMatcher.getAllNode()));

    }
}
