package com.mortals.mqtt.broker.services.auth;

import com.mortals.mqtt.broker.common.model.ClientModel;

import java.util.Map;

/**
 * 连接鉴权管理
 */
public interface AuthService {

    void start();

    void shutdown();

    /**
     * verify the clientId whether it meets the requirements or not
     * 校验clientId是否必选
     */
    boolean clientIdVerify(String clientId);

    /**
     * 校验设备是否首次激活
     * @param clientId
     * @return
     */
    boolean clientVerifyActive(String clientId);

    /**
     * if the client is on blacklist,is not allowed to connect
     * 黑名单
     */
    boolean onBlacklist(String remoteAddr,String clientId);

    /**
     * verify the clientId,username,password whether true or not
     * 校验用户名密码等
     */
    boolean authentication(ClientModel clientModel) ;

    /**
     * verify the client's heartbeat time whether the compliance
     * 校验心跳时间
     */
    boolean verifyHeartbeatTime(String clientId,int time);

    /**
     * verify the clientId whether can publish message to the topic
     * 发布topic校验
     */
    boolean publishVerify(String clientId,String topic);

    /**
     * verify the clientId whether can subscribe the topic
     * 订阅topic校验
     */
    boolean subscribeVerify(String clientId,String topic);

    boolean updateDeviceOnline(String clientId,Integer online);

}
