package com.mortals.mqtt.broker.services.auth;


import com.alibaba.fastjson.JSON;
import com.mortals.mqtt.broker.common.BrokerConstants;
import com.mortals.mqtt.broker.common.code.DeviceOnlineEnum;
import com.mortals.mqtt.broker.common.model.ClientModel;
import com.mortals.mqtt.broker.common.model.DeviceBrokerInfo;
import com.mortals.mqtt.broker.common.model.DeviceStatusInfo;
import com.mortals.mqtt.broker.common.util.CryptoUtil;
import com.mortals.mqtt.broker.common.RedisCacheKeys;
import com.mortals.mqtt.broker.common.log.MqttLogger;
import com.mortals.mqtt.broker.common.model.DeviceModel;
import com.mortals.mqtt.broker.store.redis.RedisUtils;
import org.slf4j.Logger;

import java.util.Map;

public class DefaultAuthServiceImpl implements AuthService {
    private static final Logger log = MqttLogger.clientTraceLog;

    @Override
    public void start() {
    }

    @Override
    public void shutdown() {
    }

    @Override
    public boolean clientIdVerify(String clientId) {
        //校验clientId合法性
        String[] split = clientId.split("\\|");
        if (split.length == 0) {
            return false;
        }
        return true;
    }

    @Override
    public boolean clientVerifyActive(String clientId) {

        DeviceModel deviceModel = JSON.parseObject(RedisUtils.getInstance().getRedisImpl().operate(jedis -> jedis.hget(RedisCacheKeys.getDeviceModelLockKey(), clientId)), DeviceModel.class);
        if (null == deviceModel) {
            return false;
        }

        if (DeviceOnlineEnum.UNACTIVE.getValue() == deviceModel.getOnline()) {
            return true;
        }
        return false;
    }

    @Override
    public boolean onBlacklist(String remoteAddr, String clientId) {
        return false;
    }

    @Override
    public boolean authentication(ClientModel clientModel) {
        DeviceBrokerInfo deviceModel = JSON.parseObject(RedisUtils.getInstance().getRedisImpl().operate(jedis -> jedis.hget(RedisCacheKeys.getBrokerDeviceInfo(), clientModel.getClientId())), DeviceBrokerInfo.class);
        if (null == deviceModel) {
            return false;
        }
        String[] split = clientModel.getUserName().split("&");
        if (split.length <= 1) {
            return false;
        }
        String productKey = split[1];
        String deviceName = split[0];

        String signmethod = clientModel.getExtParamsMap().get(BrokerConstants.BROKER_SIGNMETHOD);
        String securemode = clientModel.getExtParamsMap().get(BrokerConstants.BROKER_SECUREMODE);

        String deviceSecret = deviceModel.getDeviceSecret();
        String plainPasswd = "clientId" + productKey + ":" + deviceName + "deviceName" +
                deviceName + "productKey" + productKey + "timestamp" + clientModel.getExtParamsMap().get(BrokerConstants.BROKER_TIMESTAMP);
        //签名认证方式
        String psd = null;
        try {
            String remotePsd = CryptoUtil.hmacSha256(plainPasswd, deviceSecret);
            psd = new String(clientModel.getPassword(), "UTF-8");
            // log.info("device:"+deviceName+" psd:"+psd);
            // log.info("device:"+deviceName+" psd:"+remotePsd);
            if (psd.endsWith(remotePsd)) {
                return true;
            }
        } catch (Exception e) {
            log.error("异常：", e);
        }
        return false;
    }

    @Override
    public boolean verifyHeartbeatTime(String clientId, int time) {
        return true;
    }

    @Override
    public boolean publishVerify(String clientId, String topic) {
        return true;
    }

    @Override
    public boolean subscribeVerify(String clientId, String topic) {
        return true;
    }

    @Override
    public boolean updateDeviceOnline(String clientId, Integer online) {
        DeviceStatusInfo deviceStatusInfo = new DeviceStatusInfo();
        deviceStatusInfo.setOnline(online);
        deviceStatusInfo.setTime(System.currentTimeMillis());
//        DeviceModel deviceModel = JSON.parseObject(RedisUtils.getInstance().getRedisImpl().operate(jedis -> jedis.hget(RedisCacheKeys.getDeviceModelLockKey(), clientId)), DeviceModel.class);
//        if (null == deviceModel) {
//            return false;
//        }
//        deviceModel.setOnline(online);
        RedisUtils.getInstance().getRedisImpl().operate(jedis -> jedis.hset(RedisCacheKeys.getBrokerDeviceStstus(), clientId, JSON.toJSONString(deviceStatusInfo)));

        return true;
    }
}
