
package com.mortals.mqtt.broker.store;


import com.mortals.mqtt.broker.common.config.BrokerConfig;
import com.mortals.mqtt.broker.common.model.Message;
import com.mortals.mqtt.broker.common.model.Subscription;

import java.util.Collection;
import java.util.List;
import java.util.Set;


public interface SessionStore {

    void start(BrokerConfig brokerConfig);

    void shutdown();

    /**
     * 从集群中查询该clientId之前的连接状态
     */
    SessionState getSession(String clientId);

    /**
     * 1. 保存会话到 Jmqtt集群
     * 2. 通知集群其它服务器，把该连接的本地会话信息清理掉
     */
    boolean storeSession(String clientId, SessionState sessionState);

    /**
     * 清理会话信息：
     * 1. 入栈出栈中的过程消息
     * 2. 离线消息
     * 3. 订阅关系
     * 4. 订阅状态{@link SessionState}
     */
    default void clearSession(String clientId, boolean clearOfflineMsg) {
        storeSession(clientId, new SessionState(SessionState.StateEnum.NULL));
        clearSubscription(clientId);
        if (clearOfflineMsg) {
            clearOfflineMsg(clientId);
        }
    }

    /**
     * 存储订阅关系
     */
    boolean storeSubscription(String clientId, Subscription subscription);

    /**
     * 移除订阅关系
     */
    boolean delSubscription(String clientId, String topic);

    /**
     * 清理订阅信息
     */
    boolean clearSubscription(String clientId);

    /**
     * 获取该clientId的所有的订阅关系
     */
    Set<Subscription> getSubscriptions(String clientId);

    /**
     * 缓存qos2 publish报文消息-入栈消息
     *
     * @return true:缓存成功   false:缓存失败
     */
    boolean cacheInflowMsg(String clientId, Message message);

    /**
     * 获取并删除接收到的qos2消息-入栈消息
     */
    Message releaseInflowMsg(String clientId, int msgId);

    /**
     * 获取所有的入栈消息
     */
    Collection<Message> getAllInflowMsg(String clientId);

    /**
     * 缓存出栈消息-分发给客户端的qos1,qos2消息
     */
    boolean cacheOutflowMsg(String clientId, Message message);

    /**
     * 获取所有的出栈消息
     */
    Collection<Message> getAllOutflowMsg(String clientId);

    /**
     * 获取并删除发送的出栈消息
     */
    Message releaseOutflowMsg(String clientId, int msgId);

    /**
     * 出栈qos2第二阶段，缓存msgId
     */
    boolean cacheOutflowSecMsgId(String clientId, int msgId);

    /**
     * 出栈qos2第二阶段，释放msgId
     * 若为false，说明msgId不存在（异常情况）
     */
    boolean releaseOutflowSecMsgId(String clientId, int msgId);

    /**
     * 获取所有的信息，进行发送
     */
    List<Integer> getAllOutflowSecMsgId(String clientId);

    /**
     * 缓存离线消息
     */
    boolean storeOfflineMsg(String clientId, Message message);

    /**
     * 获取所有的离线消息
     */
    Collection<Message> getAllOfflineMsg(String clientId);

    /**
     * 清理该客户端的离线消息
     */
    boolean clearOfflineMsg(String clientId);
}
