package com.mortals.mqtt.broker.store.rdb;

import com.mortals.mqtt.broker.common.config.BrokerConfig;
import com.mortals.mqtt.broker.common.log.MqttLogger;
import com.mortals.mqtt.broker.store.rdb.mapper.*;
import org.apache.ibatis.session.SqlSession;
import org.slf4j.Logger;

public abstract class AbstractDBStore {
    protected final static Logger log = MqttLogger.storeLog;

    protected static final Class<SessionMapper> sessionMapperClass = SessionMapper.class;
    protected static final Class<SubscriptionMapper> subscriptionMapperClass = SubscriptionMapper.class;
    protected static final Class<EventMapper> eventMapperClass = EventMapper.class;
    protected static final Class<InflowMessageMapper> inflowMessageMapperClass = InflowMessageMapper.class;
    protected static final Class<OutflowMessageMapper> outflowMessageMapperClass = OutflowMessageMapper.class;
    protected static final Class<OutflowSecMessageMapper> outflowSecMessageMapperClass = OutflowSecMessageMapper.class;
    protected static final Class<RetainMessageMapper> retainMessageMapperClass = RetainMessageMapper.class;

    protected static final Class<OfflineMessageMapper> offlineMessageMapperClass = OfflineMessageMapper.class;
    protected static final Class<WillMessageMapper> willMessageMapperClass = WillMessageMapper.class;


    protected void start(BrokerConfig brokerConfig) {
        DBUtils.getInstance().start(brokerConfig);
    }

    protected void shutdown() {
        DBUtils.getInstance().shutdown();
    }

    protected <T> T getMapper(SqlSession sqlSession, Class<T> clazz) {
        return sqlSession.getMapper(clazz);
    }

    protected Object operate(DBCallback dbCallback) {
        return DBUtils.getInstance().operate(dbCallback);
    }

    public SqlSession getSqlSessionWithTrans() {
        return DBUtils.getInstance().getSqlSessionWithTrans();
    }
}
