package com.mortals.mqtt.broker.store.redis;

import com.alibaba.fastjson.JSONObject;
import com.mortals.mqtt.broker.common.RedisCacheKeys;
import com.mortals.mqtt.broker.common.config.BrokerConfig;
import com.mortals.mqtt.broker.common.model.Subscription;
import com.mortals.mqtt.broker.store.SubscriptionStore;
import com.mortals.mqtt.broker.store.redis.support.RedisImpl;

import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * @author: finegirl
 * @date: 2021/5/18 21:51
 * @description: redis client 存储订阅关系
 **/
public class RedisSubscriptionStore implements SubscriptionStore {
    private RedisImpl redisImpl;

    @Override
    public void start(BrokerConfig brokerConfig) {
        this.redisImpl = RedisUtils.getInstance().createSupport(brokerConfig);
    }

    @Override
    public void shutdown() {
        RedisUtils.getInstance().close();
    }

    @Override
    public boolean storeSubscription(String clientId, Subscription subscription) {
        redisImpl.operate(jedis -> jedis.hset(RedisCacheKeys.getSubscriptionKey() + clientId, subscription.getTopic(), JSONObject.toJSONString(subscription)));
        return true;
    }

    @Override
    public boolean delSubscription(String clientId, String topic) {
        redisImpl.operate(jedis -> jedis.hdel(RedisCacheKeys.getSubscriptionKey() + clientId, topic));
        return true;
    }

    @Override
    public boolean clearSubscription(String clientId) {
        redisImpl.operate(jedis -> jedis.del(RedisCacheKeys.getSubscriptionKey() + clientId));
        return true;
    }

    @Override
    public Set<Subscription> getSubscriptions(String clientId) {
        Map<String, String> subscriptions = redisImpl.operate(jedis -> jedis.hgetAll(RedisCacheKeys.getSubscriptionKey()+ clientId));
        return subscriptions.values().stream().map(val -> JSONObject.parseObject(val, Subscription.class)).collect(Collectors.toSet());

    }
}
