package com.mortals.mqtt.broker.store.redis;

import com.mortals.mqtt.broker.common.config.BrokerConfig;
import com.mortals.mqtt.broker.store.redis.support.RedisImpl;
import lombok.Getter;

import java.util.concurrent.atomic.AtomicBoolean;

/**
 * redis 访问模板方法类
 */

public class RedisUtils {
    private static final RedisUtils redisUtils = new RedisUtils();
    @Getter
    private volatile RedisImpl redisImpl;

    private AtomicBoolean start = new AtomicBoolean(false);

    private RedisUtils() {
    }

    public static RedisUtils getInstance() {
        return redisUtils;
    }

    public RedisImpl createSupport(BrokerConfig brokerConfig) {
        if (start.compareAndSet(false, true)) {
            this.redisImpl = new RedisImpl(brokerConfig);
            this.redisImpl.init();
        }
        return redisImpl;
    }

    public void close() {
        RedisImpl redisImpl = this.redisImpl;
        if (start.compareAndSet(true, false) && redisImpl != null) {
            redisImpl.close();
        }
    }
}
