package com.mortals.mqtt.broker.store.redis.support;

import com.mortals.mqtt.broker.common.config.BrokerConfig;
import com.mortals.mqtt.broker.common.log.LogUtil;
import com.mortals.mqtt.broker.common.log.MqttLogger;
import com.mortals.mqtt.broker.store.redis.RedisCallBack;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import redis.clients.jedis.Jedis;
import redis.clients.jedis.JedisPool;
import redis.clients.jedis.JedisPoolConfig;

/**
 * @author: finegirl
 * @date: 2021/5/14 9:17
 * @description: redis实现类
 **/
public class RedisImpl implements IRedis {

    private static final Logger log = MqttLogger.storeLog;
    private BrokerConfig brokerConfig;
    private JedisPool jedisPool;

    public static final String PROJECT = "MQTT";

    public RedisImpl(BrokerConfig brokerConfig) {
        this.brokerConfig = brokerConfig;
    }

   public void init() {
        try {
            JedisPoolConfig jedisPoolConfig = new JedisPoolConfig();
            jedisPoolConfig.setMinIdle(brokerConfig.getMinIdle());
            jedisPoolConfig.setMaxTotal(brokerConfig.getMaxTotal());
            jedisPoolConfig.setTestOnBorrow(brokerConfig.isTestOnBorrow());
            jedisPoolConfig.setMaxTotal(jedisPoolConfig.getMaxIdle());
            jedisPoolConfig.setMaxWaitMillis(jedisPoolConfig.getMaxWaitMillis());
            if (StringUtils.isEmpty(brokerConfig.getRedisPassword())) {
                jedisPool = new JedisPool(jedisPoolConfig, brokerConfig.getRedisHost(), brokerConfig.getRedisPort());
            } else {
                jedisPool = new JedisPool(jedisPoolConfig, brokerConfig.getRedisHost(), brokerConfig.getRedisPort(), 10000, brokerConfig.getRedisPassword());
            }
        } catch (Exception ex) {
            LogUtil.error(log, "[Redis handle error],ex:{}", ex);
        }
    }

    @Override
    public <T> T operate(RedisCallBack redisCallBack) {
       // LogUtil.debug(log, "[Cluster] redis operate begin");
        long startTime = System.currentTimeMillis();
        Jedis jedis = null;
        try {
            jedis = jedisPool.getResource();
            jedis.select(brokerConfig.getRedisDb());
            return (T) redisCallBack.operate(jedis);
        } catch (Exception ex) {
            LogUtil.error(log, "[Cluster] redis operate error,ex:{}", ex);
        } finally {
          //  LogUtil.debug(log, "[Cluster] redis operate cost:{}", (System.currentTimeMillis() - startTime));
            if (jedis != null) {
                jedis.close();
            }
        }
        return null;
    }

  public   void close(){
        LogUtil.info(log,"[Cluster] redis close");
        if (jedisPool != null) {
            jedisPool.close();
        }
    }

}
