package mortals.mqtt;

import com.alibaba.fastjson.JSON;
import com.mortals.mqtt.broker.common.util.MqttSign;
import lombok.extern.apachecommons.CommonsLog;
import lombok.extern.slf4j.Slf4j;
import org.eclipse.paho.client.mqttv3.*;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;
import org.junit.Before;
import org.junit.Test;

/**
 * @author: finegirl
 * @date: 2021/5/17 9:34
 * @description: //TODO 请完善注释信息
 **/
@Slf4j
public class PahoClient {

    public static final String HOST = "tcp://127.0.0.1:1883";
    public static final String TOPIC = "toclient/124";
    private static final String clientid = "client124";
    private  String productKey = "cdbao";
    private String deviceName = "device1";
    private String deviceSecret = "ga7XA6KdlEeiPXQPpRbAjOZXwG8y";

    private MqttClient sampleClient;
    MqttConnectOptions connOpts;

    @Before
    public void testConnectWithPassword() throws MqttException, InterruptedException {
        // 计算MQTT连接参数。
            MqttSign sign = new MqttSign();
            sign.calculate(productKey, deviceName, deviceSecret);

            System.out.println("username: " + sign.getUsername());
            System.out.println("password: " + sign.getPassword());
            System.out.println("clientid: " + sign.getClientid());
        //接入物联网平台的域名。
        // Paho MQTT客户端。  MemoryPersistence设置clientid的保存形式，默认为以内存保存
        sampleClient = new MqttClient(HOST, sign.getClientid(), new MemoryPersistence());

       // sampleClient.
        // Paho MQTT连接参数。
        connOpts = new MqttConnectOptions();
        connOpts.setCleanSession(true);
        connOpts.setKeepAliveInterval(180);
        connOpts.setUserName(sign.getUsername());
        connOpts.setPassword(sign.getPassword().toCharArray());
       // sampleClient.connect(connOpts);
        System.out.println("HOST: " + HOST + " Connected");

    }

    @Test
    public void publishTest(){
        String str="{111111}";
        subscribe(TOPIC);
      //  publishMessage(str.getBytes(),TOPIC);

    }

    public  void publishMessage(byte[] messageContent, String topic) {
        try {
            // 创建消息
            MqttMessage message = new MqttMessage(messageContent);
            // 设置消息的服务质量  QOS0 QOS1 QOS2 等具体查看 Mqtt主题那点事对应的文章
            message.setQos(0);
            // 发布消息
            sampleClient.publish(topic, message);
            // 断开连接
         //   sampleClient.disconnect();
            // 关闭客户端
          //  sampleClient.close();
        } catch (MqttException me) {
            me.printStackTrace();
        }
    }


    public  void subscribe(String... topic) {
        try {

            sampleClient.setCallback(new MqttCallbackExtended() {
                @Override
                public void connectComplete(boolean b, String s) {
                    //链接成功时触发 用于重现连接
                    log.info("connect success " + s);
                    try {
                        //client 服务中需要订阅的主题
                        sampleClient.subscribe(TOPIC);
                    } catch (MqttException e) {
                        e.printStackTrace();
                    }
                }
                /**
                 * 连接断开  触发的函数
                 */
                @Override
                public void connectionLost(Throwable throwable) {
                    log.error("device connect lost", throwable);
                    if (!sampleClient.isConnected()) {
                        try {
                            //出现意外断开 触发重新连接
                            sampleClient.reconnect();
                        } catch (Exception e) {
                            log.error("Mqtt client close error", e);
                        }
                    }

                }
                /**
                 * 接收订阅主题的内容
                 */
                @Override
                public void messageArrived(String topic, MqttMessage mqttMessage) {
                    //处理消息内容
                    log.info("receive:"+ JSON.toJSONString(mqttMessage));
                   // handleArriveMessage(topic, mqttMessage);
                }

                @Override
                public void deliveryComplete(IMqttDeliveryToken iMqttDeliveryToken) {
                    log.info("deliveryComplete");
                }
            });
            sampleClient.connect(connOpts);
            //订阅消息
            for (String t : topic) {
                log.info(t);
                sampleClient.subscribe(t, 0);
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
    }

    private static MqttConnectOptions getMqttConnectOptions() {
        // 创建链接参数
        MqttConnectOptions connOpts = new MqttConnectOptions();
        // 设置是否清空session,这里如果设置为false表示服务器会保留客户端的连接记录，这里设置为true表示每次连接到服务器都以新的身份连接
        connOpts.setCleanSession(true);
        // 设置连接的用户名  username 和 password 是安装好mqtt 服务后再后台设置的
        connOpts.setUserName("username");
        connOpts.setPassword("password".toCharArray());
        // 设置超时时间 单位为秒
        connOpts.setConnectionTimeout(120);
        // 设置会话心跳时间 单位为秒 服务器会每隔1.5*20秒的时间向客户端发送个消息判断客户端是否在线，但这个方法并没有重连的机制
        connOpts.setKeepAliveInterval(20);
        return connOpts;
    }

}
