package mortals.mqtt.broker;

import akka.actor.typed.ActorRef;
import akka.actor.typed.ActorSystem;
import akka.actor.typed.Behavior;
import akka.actor.typed.javadsl.Behaviors;
import akka.actor.typed.pubsub.Topic;
import akka.actor.typed.pubsub.Topic.Command;
import com.mortals.common.cluster.dispatcher.event.Event;
import com.typesafe.config.Config;
import com.typesafe.config.ConfigFactory;

import java.util.HashMap;
import java.util.Map;
import java.util.Random;
import java.util.concurrent.TimeUnit;

public class AkkaMain {

    public static void main(String[] args) {

        startup(2551);
    }


    private static void startup(int port) {
        // Override the configuration of the port
        Map<String, Object> overrides = new HashMap<>();
        overrides.put("akka.remote.artery.canonical.port", port);

        Config config = ConfigFactory.parseMap(overrides)
                .withFallback(ConfigFactory.load("conf/akka"));


        // Create an Akka system
        ActorSystem.create(rootBehavior(), "MqttDispatcherSystem", config);
    }


    public static Behavior<Void> rootBehavior() {
        return Behaviors.setup(
                context -> {
                    ActorRef<Command<Event>> topic =
                            context.spawn(Topic.create(Event.class, "mqtt-event"), "MqttEvent");
                    ActorRef<Event> pubSub = context
                            .spawn(PubSubExample.create(new ConsumerExample()), "MqttPubSub");

                    topic.tell(Topic.subscribe(pubSub));

                    for (int i = 0; i < 10; i++) {
                        TimeUnit.SECONDS.sleep(3);
                        Event event = new Event(new Random().nextInt(),
                                "random" + new Random().nextLong(),
                                new Random().nextLong(), "");
                        System.out.println("begin publish " + event);
                        topic.tell(Topic.publish(event));
                    }

                    return Behaviors.empty();
                });
    }
}
