package mortals.mqtt.broker;

import com.mortals.mqtt.broker.common.config.BrokerConfig;
import com.mortals.common.queue.DefaultTbQueueMsg;
import com.mortals.common.queue.TbQueueMsg;
import com.mortals.common.queue.kafka.DefaultTbQueueMsgHeaders;
import com.mortals.common.queue.kafka.TbKafkaProducerTemplate;
import com.mortals.common.queue.TopicPartitionInfo;
import mortals.mqtt.KafkaMessageCollector;
import org.apache.kafka.clients.producer.ProducerConfig;
import org.apache.kafka.common.serialization.ByteArraySerializer;
import org.apache.kafka.common.serialization.StringSerializer;
import org.awaitility.Awaitility;
import org.junit.Before;
import org.junit.Test;

import java.io.UnsupportedEncodingException;
import java.util.Properties;
import java.util.UUID;

/**
 * @author: finegirl
 * @date: 2021/5/20 10:02
 * @description: //TODO 请完善注释信息
 **/
public class KafkaTest {

    BrokerConfig brokerConfig;
    TbKafkaProducerTemplate producerTemplate;


    @Before
    public void setUp() {
        brokerConfig = new BrokerConfig();

        brokerConfig.setKafkaServerAddr("192.168.2.95:17070");
        TbKafkaProducerTemplate.TbKafkaProducerTemplateBuilder<TbQueueMsg> builder = TbKafkaProducerTemplate.builder();
        //初始化生产者
        Properties props = new Properties();
        props.put(ProducerConfig.BOOTSTRAP_SERVERS_CONFIG, brokerConfig.getKafkaServerAddr());
        props.put(ProducerConfig.ACKS_CONFIG, brokerConfig.getKafkaAcks());
        props.put(ProducerConfig.RETRIES_CONFIG, brokerConfig.getKafkaRetries());
        props.put(ProducerConfig.BATCH_SIZE_CONFIG, brokerConfig.getKafkaBatchSize());
        props.put(ProducerConfig.LINGER_MS_CONFIG, brokerConfig.getKafkaLingerMs());
        props.put(ProducerConfig.BUFFER_MEMORY_CONFIG, brokerConfig.getKafkaBufferMemory());
        props.put(ProducerConfig.KEY_SERIALIZER_CLASS_CONFIG, StringSerializer.class.getName());
        props.put(ProducerConfig.VALUE_SERIALIZER_CLASS_CONFIG, ByteArraySerializer.class.getName());
        producerTemplate = builder.props(props).defaultTopic("topicTest").build();

    }

    @Test
    public void produceMsg() throws UnsupportedEncodingException, InterruptedException {

       String product="topicTest";

       String clientId="amy.device1";
        //HashFunction hashFunction = Hashing.murmur3_32();

       // int hash = hashFunction.newHasher().putString(clientId, StandardCharsets.UTF_8).hash().asInt();

       int hash=clientId.hashCode();
       // System.out.println("hash:"+hash);
        Integer partitionSize = 10;
        int partition;
        if (partitionSize != null) {
            partition = Math.abs(hash % partitionSize);
        } else {
            partition = 0;
        }
        System.out.println("partition:"+partition);
        TopicPartitionInfo info = TopicPartitionInfo.builder().topic(product).partition(partition).build();

        String body = "this is kafka produce msg";

        DefaultTbQueueMsgHeaders headers = new DefaultTbQueueMsgHeaders();
        headers.put("kkk", "ddd".getBytes());

        DefaultTbQueueMsg defaultTbQueueMsg = new DefaultTbQueueMsg(UUID.randomUUID(), body.getBytes("utf-8"), headers);

        KafkaMessageCollector kafkaMessageCollector = new KafkaMessageCollector();

        for(int i=0;i<500;i++){

            producerTemplate.send(info, defaultTbQueueMsg, kafkaMessageCollector);

             Awaitility.await().until(kafkaMessageCollector::isMessageReceived);
              System.out.println(kafkaMessageCollector.getMetadata().toString());
        }


    }
}
