package mortals.mqtt.broker;

import com.alibaba.fastjson.JSON;
import com.mortals.mqtt.broker.common.code.DeviceOnlineEnum;
import com.mortals.mqtt.broker.common.model.DeviceBrokerInfo;
import com.mortals.mqtt.broker.services.device.DeviceServiceImpl;
import com.mortals.mqtt.broker.services.device.DeviceService;
import com.mortals.mqtt.broker.common.RedisCacheKeys;
import com.mortals.mqtt.broker.common.config.BrokerConfig;
import com.mortals.mqtt.broker.common.helper.DateUtils;
import com.mortals.mqtt.broker.common.model.DeviceModel;
import com.mortals.mqtt.broker.common.model.Subscription;
import com.mortals.mqtt.broker.store.SubscriptionStore;
import com.mortals.mqtt.broker.store.redis.RedisSubscriptionStore;
import com.mortals.mqtt.broker.store.redis.RedisUtils;
import com.mortals.mqtt.broker.store.redis.support.RedisImpl;
import com.mortals.mqtt.broker.subscribe.DefaultSubscriptionTreeMatcher;
import com.mortals.mqtt.broker.subscribe.SubscriptionMatcher;
import lombok.extern.slf4j.Slf4j;
import org.junit.Before;
import org.junit.Test;

import java.util.Set;
import java.util.stream.Collectors;

import static org.junit.jupiter.api.Assertions.assertEquals;

/**
 * @author: finegirl
 * @date: 2021/5/13 10:45
 * @description: //TODO 请完善注释信息
 **/
@Slf4j
public class RedisTest {

    RedisImpl support;
    BrokerConfig brokerConfig;

    private DeviceService deviceService;

    private SubscriptionMatcher subscriptionMatcher;

    private SubscriptionStore subscriptionStore;

    @Before
    public void setUp() {
        brokerConfig = new BrokerConfig();
        brokerConfig.setRedisHost("127.0.0.1");
        brokerConfig.setRedisPort(6379);
        brokerConfig.setRedisDb(2);
        brokerConfig.setPassword("");

        support = RedisUtils.getInstance().createSupport(brokerConfig);

        this.deviceService = new DeviceServiceImpl();
        this.subscriptionMatcher = new DefaultSubscriptionTreeMatcher();
        this.subscriptionStore = new RedisSubscriptionStore();
        this.subscriptionStore.start(brokerConfig);

    }


    @Test
    public void defaultConfig() {
        support.init();
        String key = "com:mortail:device:";
        String key1 = "com:mortail:device.model";

        String value = "value";
        String clientId = "adadfafd11111";

        Object operate = support.operate(jedis -> {
            return jedis.set(key, value);
        });

        System.out.println(operate);

    }

    @Test
    public void redisCreateDevice() {
        BrokerConfig brokerConfig = new BrokerConfig();
        brokerConfig.setRedisHost("127.0.0.1");
        brokerConfig.setRedisPort(6379);
        brokerConfig.setRedisDb(2);
        brokerConfig.setPassword("");
        RedisImpl support = RedisUtils.getInstance().createSupport(brokerConfig);

        for (int i = 0; i < 100; i++) {
            String clientId = "cdbao:device" + i;


            DeviceBrokerInfo deviceModel = new DeviceBrokerInfo();
            deviceModel.setClientId(clientId);
            deviceModel.setDeviceSecret("ga7XA6KdlEeiPXQPpRbAjOZXwG8y");
            deviceModel.setProductKey("cdbao");
            deviceModel.setDeviceName("device" + i);
            deviceModel.setAvgFlowRate(52);
            deviceModel.setMaxFlowRate(104);
            support.operate(jedis -> jedis.hset(RedisCacheKeys.getBrokerDeviceInfo(), clientId, JSON.toJSONString(deviceModel)));

        }

    }

    //每个设备订阅3个主题
    @Test
    public void redisSubsTest() {

        RedisSubscriptionStore redisSubscriptionStore = new RedisSubscriptionStore();
        redisSubscriptionStore.start(brokerConfig);

        for (int i = 0; i < 100; i++) {

            String clientId = "cdbao.device" + i;
            Subscription subscription = new Subscription();
            subscription.setClientId(clientId);
            subscription.setTopic("/topic/a");
            subscription.setQos(1);

            redisSubscriptionStore.storeSubscription(clientId, subscription);


            subscription.setTopic("/topic/b");
            subscription.setQos(1);

            redisSubscriptionStore.storeSubscription(clientId, subscription);

            subscription.setTopic("/topic/c");
            subscription.setQos(1);

            redisSubscriptionStore.storeSubscription(clientId, subscription);


            subscription.setTopic("/topic/"+clientId);
            subscription.setQos(1);

            redisSubscriptionStore.storeSubscription(clientId, subscription);
        }


        //获取指定订阅者
        Set<Subscription> subscriptions = subscriptionStore.getSubscriptions("cdbao.device1");

        System.out.println(JSON.toJSONString(subscriptions));
        log.info("subs:{}", JSON.toJSONString(subscriptions));

    }


    /**
     * 初始化订阅树
     */
    @Test
    public void initSubsTest() {

        subscriptionMatcher.initNode();

        System.out.println(DateUtils.getCurrStrDateTime());
        //获取所有设备
        Set<String> allDeviceClients = this.deviceService.getAllDeviceClients();

        System.out.println(DateUtils.getCurrStrDateTime());
        System.out.println("clients size" + allDeviceClients.size());
        //根据设备获取设备所有订阅topic
        Set<Subscription> allSubs = allDeviceClients.parallelStream()
                .flatMap(item -> this.subscriptionStore.getSubscriptions(item).stream()).collect(Collectors.toSet());
        //根据订阅的top,初始化订阅树

        System.out.println(DateUtils.getCurrStrDateTime());
        System.out.println("subs size" + allSubs.size());

        long count = allSubs.parallelStream().map(item -> subscriptionMatcher.subscribe(item)).count();
        String str = JSON.toJSONString(subscriptionMatcher.getAllNode());
        System.out.println(DateUtils.getCurrStrDateTime());
        System.out.println(count);
        System.out.println(str.length());
        // System.out.println(str);


    }


    @Test
    public void redisObject() {
        BrokerConfig brokerConfig = new BrokerConfig();
        brokerConfig.setRedisHost("127.0.0.1");
        brokerConfig.setRedisPort(6379);
        brokerConfig.setRedisDb(2);
        brokerConfig.setPassword("");


        RedisImpl support = RedisUtils.getInstance().createSupport(brokerConfig);
        support.init();

        String clientId = "cdbao.device1";
        DeviceModel deviceModel = new DeviceModel();
        deviceModel.setClientId(clientId);
        deviceModel.setOnline(DeviceOnlineEnum.UNACTIVE.getValue());
        deviceModel.setDeviceSecret("ga7XA6KdlEeiPXQPpRbAjOZXwG8y");
        deviceModel.setProductKey("cdbao");
        deviceModel.setDeviceName("device1");
        deviceModel.setFilterTopics("/aaa/vvvv;/dddd/eeeee");


        Object test = support.operate(jedis -> jedis.hset(RedisCacheKeys.getDeviceModelLockKey(), clientId, JSON.toJSONString(deviceModel)));
//        System.out.println(test);
//        System.out.println((Long) test>0);

        DeviceModel after = JSON.parseObject(support.operate(jedis -> jedis.hget(RedisCacheKeys.getDeviceModelLockKey(), clientId)), DeviceModel.class);

        assertEquals(JSON.toJSONString(deviceModel), JSON.toJSONString(after), "redis set object fail");


    }
}
