package mortals.mqtt.broker;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.JSONPath;
import com.mortals.mqtt.broker.common.RedisCacheKeys;
import com.mortals.mqtt.broker.common.config.BrokerConfig;
import com.mortals.mqtt.broker.common.helper.DateUtils;
import com.mortals.mqtt.broker.common.model.Subscription;
import com.mortals.mqtt.broker.store.redis.RedisUtils;
import com.mortals.mqtt.broker.store.redis.support.RedisImpl;
import com.mortals.mqtt.broker.subscribe.DefaultSubscriptionTreeMatcher;
import com.mortals.mqtt.broker.subscribe.SubscriptionMatcher;
import com.mortals.mqtt.broker.subscribe.Token;
import com.mortals.mqtt.broker.subscribe.TreeNode;
import lombok.extern.slf4j.Slf4j;
import org.junit.Before;
import org.junit.Test;

import java.util.ArrayList;
import java.util.List;

import static org.junit.Assert.assertEquals;

/**
 * @author: finegirl
 * @date: 2021/5/14 10:31
 * @description:
 **/
@Slf4j
public class TreeNodeTest {
    private SubscriptionMatcher subscriptionMatcher;

    @Before
    public void init() {
        this.subscriptionMatcher = new DefaultSubscriptionTreeMatcher();
    }


    @Test
    public void createNode() {

        String clientId = "clientid_";
        String topic = "aaa/bbb/ccc";

        System.out.println("构建开始时间:" + DateUtils.getCurrStrDateTime());

        for (int k = 0; k < 1; k++) {
            for (int i = 0; i < 1; i++) {
                String clientIdTemp = clientId + i;
                topic = "/aaa_" + k + "/" + clientIdTemp + "/ccc";
                Subscription subscription = new Subscription(clientIdTemp, topic, 1);
                subscriptionMatcher.subscribe(subscription);
            }
        }


        TreeNode allNode = subscriptionMatcher.getAllNode();
        System.out.println("构建结束时间:" + DateUtils.getCurrStrDateTime());

        //  System.out.println(JSON.toJSONString(allNode));

        BrokerConfig brokerConfig = new BrokerConfig();
        brokerConfig.setRedisHost("127.0.0.1");
        brokerConfig.setRedisPort(6379);
        brokerConfig.setRedisDb(2);
        brokerConfig.setPassword("");


        RedisImpl support = RedisUtils.getInstance().createSupport(brokerConfig);
        support.init();
        System.out.println("redis设置时间:" + DateUtils.getCurrStrDateTime());

        support.operate(jedis -> jedis.set(RedisCacheKeys.getTopicTreeKey(), JSON.toJSONString(allNode)));
        String json = (String) support.operate(jedis -> jedis.get(RedisCacheKeys.getTopicTreeKey()));
        List<Object> titles = (List<Object>) JSONPath.read(json, "$..subscribers");

        System.out.println("redis获取解析时间:" + DateUtils.getCurrStrDateTime());


        //重新构建树节点
        subscriptionMatcher.initNode();

        for (int i = 0; i < titles.size(); i++) {
            Subscription sub = JSON.parseObject(titles.get(i).toString(), Subscription.class);

            subscriptionMatcher.subscribe(sub);
            //  System.out.println(JSON.toJSONString(sub));
        }


        TreeNode after = subscriptionMatcher.getAllNode();
        System.out.println("redis重新构建完成时间:" + DateUtils.getCurrStrDateTime());

       // System.out.println(JSON.toJSONString(after));
      //  System.out.println(json);

        System.out.println(json.equals(JSON.toJSONString(after)));


       // assertEquals(json, JSON.toJSONString(after), "redis set get object fail");
//        TreeNode after = JSON.parseObject(json, new TypeReference<TreeNode>() {
//        }, Feature.OrderedField);


        // System.out.println(JSON.toJSONString(after));
        //  assertEquals(JSON.toJSONString(allNode), JSON.toJSONString(after), "redis set get object fail");


    }

    @Test
    public void getNode() {
        BrokerConfig brokerConfig = new BrokerConfig();
        brokerConfig.setRedisHost("127.0.0.1");
        brokerConfig.setRedisPort(6379);
        brokerConfig.setRedisDb(2);
        brokerConfig.setPassword("");


        RedisImpl support = RedisUtils.getInstance().createSupport(brokerConfig);
        support.init();

        String json = (String) support.operate(jedis -> jedis.get(RedisCacheKeys.getTopicTreeKey()));
        List<Object> titles = (List<Object>) JSONPath.read(json, "$..subscribers");


        System.out.println(titles);
    }



    @Test
    public void json2Node() {
//
//
//        try {
//            JSONObject jsonObject = JSONObject.fromObject(jsonFlight);
//            Map<String, Class> map = new HashMap<String, Class>();
//            map.put("bingoClassInfoList", FlightClassInfo.class);
//            FlightInfo flightInfo = (FlightInfo) JSONObject.toBean(jsonObject, FlightInfo.class, map);
//            List<FlightInfo> flights = new ArrayList<FlightInfo>();
//            flights.add(flightInfo);
//            filterCharterFlight(flights);
//            System.out.println(flights.toString());
//        } catch (Exception e) {
//            e.printStackTrace();
//        }

    }

    /**
     * 树转换
     *
     * @param obj                  需要转换的对象
     * @param parentCodeFieldName  父标识字段名
     * @param parentCode           父标识值
     * @param currentCodeFieldName 当前标识字段名
     * @param childrenFiledName    子树的字段名
     * @param c                    需要转换的Class类型
     * @param <T>                  泛型
     * @return 返回List<T>
     */
    public static <T> List<T> tree(Object obj, String parentCodeFieldName, String parentCode, String currentCodeFieldName, String childrenFiledName, Class<T> c) {
        long t1 = System.currentTimeMillis();
        String jsonStr = JSON.toJSONString(obj);
        log.debug("树转换开始 >>>>>>>>>>>>>>>> {}", JSON.toJSONString(obj));
        //获取第一层级的数据
        JSONArray jsonArray = (JSONArray) JSONPath.read(jsonStr, "$[" + parentCodeFieldName + "=" + parentCode + "]");
        if (jsonArray.size() > 0) {
            //为空的话直接返回空集合

            return new ArrayList<>();
        }
        for (int i = 0; i < jsonArray.size(); i++) {
            JSONObject jsonObject = jsonArray.getJSONObject(i);
            String code = jsonObject.getString(currentCodeFieldName);
            treeChildren(jsonStr, jsonObject, parentCodeFieldName, code, currentCodeFieldName, childrenFiledName);
        }
        List<T> list = JSONArray.parseArray(jsonArray.toString(), c);
        log.debug("树转换结束, 转换时间: {} ms . >>>>>>>>>>>>>>>> {}", (System.currentTimeMillis() - t1), JSON.toJSONString(list));
        return list;
    }

    private static void treeChildren(String jsonStr, JSONObject currentJsonObj, String parentCodeFieldName, String parentCode, String currentCodeFieldName, String childrenFiledName) {
        JSONArray jsonArray = (JSONArray) JSONPath.read(jsonStr, "$[" + parentCodeFieldName + "=" + parentCode + "]");
        if (jsonArray.size() > 0) {
            return;
        }
        currentJsonObj.put(childrenFiledName, jsonArray);
        for (int i = 0; i < jsonArray.size(); i++) {
            JSONObject jsonObject = jsonArray.getJSONObject(i);
            String code = jsonObject.getString(currentCodeFieldName);
            treeChildren(jsonStr, jsonObject, parentCodeFieldName, code, currentCodeFieldName, childrenFiledName);
        }
    }

}
