package mortals.mqtt.integration;


import com.mortals.common.queue.TopicPartitionInfo;
import com.mortals.mqtt.broker.services.MqttBrokerService;
import com.mortals.MQServer;
import com.mortals.mqtt.broker.common.util.MqttSign;
import com.mortals.mqtt.broker.common.config.BrokerConfig;
import com.mortals.mqtt.broker.common.model.Message;
import com.mortals.mqtt.broker.common.model.MessageHeader;
import com.mortals.common.queue.DefaultTbQueueMsg;
import com.mortals.common.queue.TbQueueMsg;
import com.mortals.common.queue.kafka.*;
import lombok.extern.slf4j.Slf4j;
import mortals.mqtt.KafkaMessageCollector;
import mortals.mqtt.MessageCollector;
import org.apache.kafka.clients.producer.ProducerConfig;
import org.apache.kafka.common.serialization.ByteArraySerializer;
import org.apache.kafka.common.serialization.StringSerializer;
import org.awaitility.Awaitility;
import org.eclipse.paho.client.mqttv3.*;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;
import org.junit.Before;
import org.junit.Test;

import java.io.UnsupportedEncodingException;
import java.util.Properties;
import java.util.UUID;

import static mortals.mqtt.ServerIntegrationPahoTest.HOST;

/**
 * @author: finegirl
 * @date: 2021/5/20 16:54
 * @description: kafka集成测试
 **/
@Slf4j
public class ServerIntegrationKafkaTest {
    IMqttClient m_subscriber;
    IMqttClient m_publisher;
    MessageCollector m_callback;
    MessageCollector m_callback_p;
    private String productKey = "cdbao";
    private String deviceName = "device1";
    private String deviceSecret = "ga7XA6KdlEeiPXQPpRbAjOZXwG8y";

    private String deviceName2 = "device2";

    private MqttSign sign;
    private MqttSign sign2;

    MqttBrokerService mqttBrokerService;
    TbKafkaProducerTemplate producerTemplate;


    @Before
    public void setUp() throws Exception {

        mqttBrokerService = MQServer.startMqttService(new String[]{});
        sign = new MqttSign();
        sign.calculate(productKey, deviceName, deviceSecret);

        m_subscriber = new MqttClient(HOST, sign.getClientid(), new MemoryPersistence());
        m_callback = new MessageCollector();
        m_subscriber.setCallback(m_callback);
        MqttConnectOptions options = new MqttConnectOptions();
        options.setCleanSession(false);
        options.setKeepAliveInterval(180);
        options.setUserName(sign.getUsername());
        options.setPassword(sign.getPassword().toCharArray());
        m_subscriber.connect(options);

        BrokerConfig brokerConfig = new BrokerConfig();

        brokerConfig.setKafkaServerAddr("192.168.2.95:17070");
        TbKafkaProducerTemplate.TbKafkaProducerTemplateBuilder<TbQueueMsg> builder = TbKafkaProducerTemplate.builder();
        //初始化生产者
        Properties props = new Properties();
        props.put(ProducerConfig.BOOTSTRAP_SERVERS_CONFIG, brokerConfig.getKafkaServerAddr());
        props.put(ProducerConfig.ACKS_CONFIG, brokerConfig.getKafkaAcks());
        props.put(ProducerConfig.RETRIES_CONFIG, brokerConfig.getKafkaRetries());
        props.put(ProducerConfig.BATCH_SIZE_CONFIG, brokerConfig.getKafkaBatchSize());
        props.put(ProducerConfig.LINGER_MS_CONFIG, brokerConfig.getKafkaLingerMs());
        props.put(ProducerConfig.BUFFER_MEMORY_CONFIG, brokerConfig.getKafkaBufferMemory());
        props.put(ProducerConfig.KEY_SERIALIZER_CLASS_CONFIG, StringSerializer.class.getName());
        props.put(ProducerConfig.VALUE_SERIALIZER_CLASS_CONFIG, ByteArraySerializer.class.getName());
        producerTemplate = builder.props(props).defaultTopic("cdbao").build();
    }


    /**
     * 1.客户端订阅topic主题（非kafka topic）
     * 2.kafka针对一个topic生成消息
     * 3.broker中的kafka消费生产的消息
     * 4.broker获取消息中的topic信息
     * 5.broker把消息写入客户端A
     */
    @Test
    public void kafkaProducePubMessageToServerToClientTest() throws MqttException, UnsupportedEncodingException, InterruptedException {
        log.info("*** checkSubscriberQoS0ReceiveQoS0publishes ***");
        m_subscriber.subscribe("/topic/kafka", 0);

        //kafka生成消息
        String product = "topicTest";

        String clientId = "amy.device1";
       // HashFunction hashFunction = Hashing.murmur3_32();

       // int hash = hashFunction.newHasher().putString(clientId, StandardCharsets.UTF_8).hash().asInt();
        //System.out.println("hash:" + hash);

        int hash=clientId.hashCode();

        Integer partitionSize = 10;
        int partition;
        if (partitionSize != null) {
            partition = Math.abs(hash % partitionSize);
        } else {
            partition = 0;
        }
        TopicPartitionInfo info = TopicPartitionInfo.builder().topic(product).partition(partition).build();

        String body = "this is kafka produce msg";
        body.getBytes("utf-8");

        DefaultTbQueueMsgHeaders headers = new DefaultTbQueueMsgHeaders();
        headers.put(MessageHeader.CLIENTID,deviceName.getBytes());
        headers.put(MessageHeader.TOPIC,"/topic/kafka".getBytes());
        headers.put(MessageHeader.MESSAGETYPE,new byte[]{(byte)Message.Type.PUBLISH.value()});
        headers.put(MessageHeader.QOS,new byte[]{(byte)1});

        DefaultTbQueueMsg defaultTbQueueMsg = new DefaultTbQueueMsg(UUID.randomUUID(), body.getBytes("utf-8"), headers);

        KafkaMessageCollector kafkaMessageCollector = new KafkaMessageCollector();
        for(int i=0;i<10;i++){
            producerTemplate.send(info, defaultTbQueueMsg, kafkaMessageCollector);
            Awaitility.await().until(kafkaMessageCollector::isMessageReceived);
          //  log.info(kafkaMessageCollector.getMetadata().toString());
        }
        //Thread.sleep(10000);

        Awaitility.await().until(m_callback::isMessageReceived);
        MqttMessage message = m_callback.retrieveMessage();
        log.info("receive message:"+message.toString());
//        assertEquals("Hello world MQTT QoS0", message.toString());
//        assertEquals(0, message.getQos());

        Thread.sleep(10000);

    }

    @Test
    public void testType(){
        Message.Type.PUBACK.value();
    }
}
