/*
 * Copyright (c) 2012-2018 The original author or authors
 * ------------------------------------------------------
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * and Apache License v2.0 which accompanies this distribution.
 *
 * The Eclipse Public License is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * The Apache License v2.0 is available at
 * http://www.opensource.org/licenses/apache2.0.php
 *
 * You may elect to redistribute this code under either of these licenses.
 */

package mortals.mqtt.integration;

import com.mortals.mqtt.broker.common.util.MqttSign;
import com.mortals.mqtt.broker.services.MqttBrokerService;
import lombok.extern.log4j.Log4j;
import mortals.mqtt.MessageCollector;
import org.awaitility.Awaitility;
import org.eclipse.paho.client.mqttv3.IMqttClient;
import org.eclipse.paho.client.mqttv3.MqttClient;
import org.eclipse.paho.client.mqttv3.MqttConnectOptions;
import org.eclipse.paho.client.mqttv3.MqttMessage;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import java.util.concurrent.TimeUnit;

import static mortals.mqtt.ServerIntegrationPahoTest.HOST;

/**
 * 管理端下发设备控制消息测试
 */
@Log4j
public class ServerIntegrationManagerTest {

    IMqttClient m_subscriber;
    MessageCollector m_callback;
    private String productKey = "cdbao";
    private String deviceName = "device1";
    private String deviceSecret = "ga7XA6KdlEeiPXQPpRbAjOZXwG8y";

    private MqttSign sign;

    MqttBrokerService mqttBrokerService;

    @BeforeEach
    public void setUp() throws Exception {

        //brokerService = MQServer.start(new String[]{});
        // 计算MQTT连接参数。
        sign = new MqttSign();
        sign.calculate(productKey, deviceName, deviceSecret);

        m_subscriber = new MqttClient(HOST, sign.getClientid(), new MemoryPersistence());
        m_callback = new MessageCollector();

        m_subscriber.setCallback(m_callback);

        MqttConnectOptions options = new MqttConnectOptions();
        options.setCleanSession(false);
        options.setKeepAliveInterval(30);
        options.setUserName(sign.getUsername());
        options.setPassword(sign.getPassword().toCharArray());
        m_subscriber.connect(options);


    }

    @AfterEach
    public void tearDown() throws Exception {

        if (m_subscriber.isConnected()) {
            m_subscriber.disconnect();
        }
        //brokerService.shutdown();
    }

    /**
     * 测试设备连接，管理端下发消息，设备端接收消息并打印
     *
     * @throws Exception
     */
    @Test
    public void checkManagerControlMsgToDevice() throws Exception {
        log.info("*** checkManagerControlMsgToDevice ***");
//        while (true){
//
//        }

        Awaitility.await().atMost(1000, TimeUnit.MINUTES).until(m_callback::isMessageReceived);
        MqttMessage message = m_callback.retrieveMessage();
        log.info("receive:"+message.toString());
        log.info("count:"+m_callback.getCount());
        System.out.println("receive:"+message.toString());
        //assertEquals("this is kafka produce msg", message.toString());



    }



}
