/*
 * Copyright (c) 2012-2018 The original author or authors
 * ------------------------------------------------------
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * and Apache License v2.0 which accompanies this distribution.
 *
 * The Eclipse Public License is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * The Apache License v2.0 is available at
 * http://www.opensource.org/licenses/apache2.0.php
 *
 * You may elect to redistribute this code under either of these licenses.
 */

package mortals.mqtt.integration;

import com.mortals.mqtt.broker.common.util.MqttSign;
import com.mortals.mqtt.broker.services.MqttBrokerService;
import mortals.mqtt.MessageCollector;
import org.eclipse.paho.client.mqttv3.MqttClient;
import org.eclipse.paho.client.mqttv3.MqttConnectOptions;
import org.eclipse.paho.client.mqttv3.MqttException;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

import static java.nio.charset.StandardCharsets.UTF_8;
import static mortals.mqtt.ServerIntegrationPahoTest.HOST;
import static org.junit.jupiter.api.Assertions.assertTrue;

public class ServerIntegrationMultiConnectTest {

    private String productKey = "cdbao";
    private String deviceName = "device";
    private String deviceSecret = "ga7XA6KdlEeiPXQPpRbAjOZXwG8y";


    private MqttSign sign;

    Map<MqttClient, MessageCollector> map;
    List<MqttClient> list;
    List<MessageCollector> messageCollectorList;

    MqttBrokerService mqttBrokerService;

    /**
     * device1 订阅者
     * device2 发布者
     *
     * @throws Exception
     */
    @BeforeEach
    public void setUp() throws Exception {
      //  mqttBrokerService = MQServer.startMqttService(new String[]{});
    }

    @AfterEach
    public void tearDown() throws Exception {

            map.entrySet().stream().forEach(item -> {
                if (item.getKey().isConnected()) {
                    //打印接收的消息
                    String temp = item.getKey().getClientId().split("\\|")[0];
                    System.out.println("clientId:" + temp + "receive msg count:"+item.getValue().getCount());

                    try {
                        item.getKey().disconnect();
                    } catch (MqttException e) {
                        e.printStackTrace();
                    }
                }
            });


       // mqttBrokerService.shutdown();
    }

    @Test
    public void testMultipleClientConnectsWithSameClientId() throws MqttException, IOException, InterruptedException {
        map = new HashMap<>();

        for (int i = 0; i < 100; i++) {
            // 计算MQTT连接参数。
            sign = new MqttSign();
            sign.calculate(productKey, deviceName + i, deviceSecret);
            MqttClient m_subscriber = new MqttClient(HOST, sign.getClientid(), new MemoryPersistence());
            MessageCollector m_callback = new MessageCollector();

            m_subscriber.setCallback(m_callback);

            MqttConnectOptions options = new MqttConnectOptions();
            options.setCleanSession(false);
            options.setKeepAliveInterval(30);
            options.setUserName(sign.getUsername());
            options.setPassword(sign.getPassword().toCharArray());
            m_subscriber.connect(options);

            map.put(m_subscriber, m_callback);

        }

        TimeUnit.SECONDS.sleep(5);

        //随机发布消息
        int num = 0;
        while (num < 200) {
            map.keySet().parallelStream().peek(item -> {
                try {
                    String temp = item.getClientId().split("\\|")[0];
                    item.publish("/topic/" + temp, "Hello world MQTT QoS0".getBytes(UTF_8), 2, false);
                } catch (MqttException e) {
                    e.printStackTrace();
                }
            }).count();

            TimeUnit.SECONDS.sleep(1);
            num++;
        }
    }
}
