package mortals.mqtt.metric;

import static java.lang.Double.isInfinite;
import static java.lang.Double.isNaN;

/**
 * @author: finegirl
 * @date: 2021/5/24 15:33
 * @description: //TODO 请完善注释信息
 **/
public abstract class RatioGauge implements Gauge<Double> {
    /**
     * 定义Ration类，，包含两个成员变量，通过两个变量做除法，得到一个比率信息。
     */
    public static class Ratio {
        public static Ratio of(double numerator, double denominator) {
            return new Ratio(numerator, denominator);
        }

        private final double numerator;
        private final double denominator;

        private Ratio(double numerator, double denominator) {
            this.numerator = numerator;
            this.denominator = denominator;
        }

        /**
         * 返回Ration的比率信息
         *
         * @return the ratio
         */
        public double getValue() {
            final double d = denominator;
            if (isNaN(d) || isInfinite(d) || d == 0) {
                return Double.NaN;
            }
            return numerator / d;
        }

        @Override
        public String toString() {
            return numerator + ":" + denominator;
        }
    }

    /**
     * Returns the {@link Ratio} which is the gauge's current value.
     *
     * @return the {@link Ratio} which is the gauge's current value
     */
    protected abstract Ratio getRatio();

    @Override
    public Double getValue() {
        return getRatio().getValue();
    }
}
