package com.mortals.xhx.module.consumer.web;

import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.DbOrderKind;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.common.code.PushMessTypeEnum;
import com.mortals.xhx.common.code.SubTypeEnum;
import com.mortals.xhx.module.consumer.model.ConsumerGroupEntity;
import com.mortals.xhx.module.consumer.model.ConsumerGroupQuery;
import com.mortals.xhx.module.consumer.service.ConsumerGroupService;
import com.mortals.xhx.module.customer.model.CustomerEntity;
import com.mortals.xhx.module.customer.service.CustomerService;
import com.mortals.xhx.module.product.model.ProductEntity;
import com.mortals.xhx.module.product.service.ProductService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.mortals.framework.model.Context;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.mortals.framework.web.BaseCRUDJsonMappingController;
import com.mortals.xhx.module.consumer.model.ConsumerSubEntity;
import com.mortals.xhx.module.consumer.service.ConsumerSubService;

import java.util.HashMap;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * ConsumerSubController
 * <p>
 * 服务端订阅 控制器
 *
 * @author zxfei
 * @date 2021-11-09
 */
@RestController
@RequestMapping("consumer/sub")
public class ConsumerSubController extends BaseCRUDJsonMappingController<ConsumerSubService, ConsumerSubForm, ConsumerSubEntity, Long> {

    @Autowired
    private ParamService paramService;

    @Autowired
    private ProductService productService;

    @Autowired
    private ConsumerGroupService consumerGroupService;

    @Autowired
    private CustomerService customerService;

    public ConsumerSubController() {
        super.setFormClass(ConsumerSubForm.class);
        super.setModuleDesc("服务端订阅");
    }

    @Override
    protected void init(HttpServletRequest request, HttpServletResponse response, ConsumerSubForm form,
                        Map<String, Object> model, Context context) {
        this.addDict(model, "subType", SubTypeEnum.getEnumMap());
        this.addDict(model, "pushMessType", PushMessTypeEnum.getEnumMap());
        this.addDict(model, "productId",
                productService.getCacheList().stream().collect(Collectors.toMap(e -> e.getId().toString(), ProductEntity::getProductName)));
        ConsumerGroupQuery query = new ConsumerGroupQuery();
        query.setCustomerId(getCurUser().getId());
        this.addDict(model, "consumerGroupId",
                consumerGroupService.find(query, new Context()).stream().collect(Collectors.toMap(e -> e.getId().toString(), ConsumerGroupEntity::getConsumerGroupName)));
        this.addDict(model, "customerId",
                customerService.getCacheList().stream().collect(Collectors.toMap(e -> e.getId().toString(), CustomerEntity::getLoginName)));
        super.init(request, response, form, model, context);
    }

    @Override
    protected void doListBefore(HttpServletRequest request, HttpServletResponse response, ConsumerSubForm form, Map<String, Object> model, Context context) throws AppException {
        Map<String, String> orderCols = new HashMap<>();
        orderCols.put("id", DbOrderKind.DESC.name());
        form.getQuery().setOrderCols(orderCols);
        super.doListBefore(request, response, form, model, context);
    }
}