package com.mortals.xhx.module.customer.service.impl;

import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.module.customer.model.CustomerQuery;
import com.mortals.xhx.utils.KeyUtil;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDCacheServiceImpl;
import com.mortals.xhx.module.customer.dao.CustomerDao;
import com.mortals.xhx.module.customer.model.CustomerEntity;
import com.mortals.xhx.module.customer.service.CustomerService;

import java.util.List;


/**
 * CustomerService
 * 客户 service实现
 *
 * @author zxfei
 * @date 2021-11-09
 */
@Service("customerService")
public class CustomerServiceImpl extends AbstractCRUDCacheServiceImpl<CustomerDao, CustomerEntity, Long> implements CustomerService {
    @Override
    protected String getExtKey(CustomerEntity data) {
        return data.getAccessKey();
    }

    @Override
    protected void validData(CustomerEntity entity, Context context) throws AppException {
        CustomerQuery query = new CustomerQuery();
        query.setLoginName(entity.getLoginName());
        List<CustomerEntity> list = dao.getList(query);
        if (entity.getId() != null) {//修改
            // 校验重复编码
            if (!list.isEmpty() && list.get(0).getLoginName().equals(entity.getLoginName())
                    && list.get(0).getId().longValue() != entity.getId()) {
                throw new AppException("已存在相同账号");
            }
        } else {//新增
            CustomerQuery cQuery = new CustomerQuery();
            cQuery.setAccessKey(entity.getAccessKey());
            if (dao.getCount(cQuery) > 0) {
                throw new AppException("服务繁忙，请重新提交");
            }
            if (!list.isEmpty()) {
                throw new AppException("已存在相同账号");
            }
        }
        super.validData(entity, context);
    }

    @Override
    protected void saveBefore(CustomerEntity entity, Context context) throws AppException {
        entity.setAccessKey(KeyUtil.toHex());
        super.saveBefore(entity, context);
    }
}