package com.mortals.xhx.module.device.service.impl;

import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.service.ICacheService;
import com.mortals.framework.service.impl.AbstractCRUDCacheServiceImpl;
import com.mortals.xhx.common.code.DeviceStatusEnum;
import com.mortals.xhx.common.code.ProductFunctionTypeEnum;
import com.mortals.xhx.common.code.ProductStatusEnum;
import com.mortals.xhx.common.code.UseStatusEnum;
import com.mortals.xhx.common.utils.BeanUtil;
import com.mortals.xhx.module.device.model.DeviceQuery;
import com.mortals.xhx.module.device.model.DeviceTopicEntity;
import com.mortals.xhx.module.device.model.DeviceTopicQuery;
import com.mortals.xhx.module.device.service.DeviceTopicService;
import com.mortals.xhx.module.product.model.*;
import com.mortals.xhx.module.product.service.ProductFunctionItemService;
import com.mortals.xhx.module.product.service.ProductService;
import com.mortals.xhx.module.product.service.ProductTopicService;
import com.mortals.xhx.service.common.code.DeviceBrokerInfo;
import com.mortals.xhx.service.common.code.RedisKeys;
import com.mortals.xhx.service.common.code.Subscription;
import com.mortals.xhx.utils.KeyUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.xhx.module.device.dao.DeviceDao;
import com.mortals.xhx.module.device.model.DeviceEntity;
import com.mortals.xhx.module.device.service.DeviceService;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * DeviceService
 * 设备 service实现
 *
 * @author zxfei
 * @date 2021-11-09
 */
@Service("deviceService")
public class DeviceServiceImpl extends AbstractCRUDCacheServiceImpl<DeviceDao, DeviceEntity, Long> implements DeviceService {
    @Autowired
    private ProductService productService;

    @Autowired
    private ProductFunctionItemService productFunctionItemService;

    @Autowired
    private ProductTopicService productTopicService;

    @Autowired
    private DeviceTopicService deviceTopicService;

    @Autowired
    private ICacheService cacheService;

    @Override
    protected String getExtKey(DeviceEntity data) {
        return data.getProductKey() + ":" + data.getDeviceName();
    }

    @Override
    protected void validData(DeviceEntity entity, Context context) throws AppException {
        DeviceQuery query = new DeviceQuery();
        query.setProductId(entity.getProductId());
        query.setDeviceName(entity.getDeviceName());
        List<DeviceEntity> list = dao.getList(query);
        if (entity.getId() != null) {//修改
            // 校验重复编码
            if (!list.isEmpty() && list.get(0).getDeviceName().equals(entity.getDeviceName())
                    && list.get(0).getProductId().longValue() == entity.getProductId()
                    && list.get(0).getId().longValue() != entity.getId()) {
                throw new AppException("该产品下已存在相同的设备名称");
            }
        } else {//新增
            DeviceQuery dQuery = new DeviceQuery();
            dQuery.setDeviceSecret(entity.getDeviceSecret());
            if (dao.getCount(dQuery) > 0) {
                throw new AppException("服务繁忙，请重新提交");
            }
            if (!list.isEmpty()) {
                throw new AppException("该产品下已存在相同的设备名称");
            }
        }
        super.validData(entity, context);
    }

    @Override
    protected void saveBefore(DeviceEntity entity, Context context) throws AppException {
        ProductEntity cache = productService.getCache(String.valueOf(entity.getProductId()));
        entity.setProductKey(cache.getProductKey());
        entity.setCustomerId(cache.getCustomerId());
        entity.setDeviceSecret(KeyUtil.toHex());
        super.saveBefore(entity, context);
    }

    @Override
    protected void saveAfter(DeviceEntity entity, Context context) throws AppException {
        updateDeviceTopic(entity, context, true);
        //新增broker需要的设备缓存节点
        updateDeviceBrokerInfo(entity);
        super.saveAfter(entity, context);
    }

    @Override
    protected void updateAfter(DeviceEntity entity, Context context) throws AppException {
        //修改broker需要的设备缓存节点
        super.updateAfter(entity, context);
    }

    @Override
    public boolean updateDeviceTopic(DeviceEntity entity, Context context, boolean flag) throws AppException {
        //新增设备构建设备topic
        List<DeviceTopicEntity> dtList = new ArrayList<DeviceTopicEntity>();
        ProductQuery pQuery = new ProductQuery();
        pQuery.setId(entity.getProductId());
        pQuery.setProductStatus(ProductStatusEnum.PUBLISH.getValue());
        if (flag == true && productService.count(pQuery, context) == 0) {
            //对应产品未发布
            return false;
        }
        ProductFunctionItemQuery itemQuery = new ProductFunctionItemQuery();
        itemQuery.setProductId(entity.getProductId());
        List<ProductFunctionItemEntity> itemList = productFunctionItemService.find(itemQuery, context);
        if (null == itemList || itemList.size() == 0) {
            //设备对应的产品还没有物模型
            return false;
        }
        ProductTopicQuery ptQuery = new ProductTopicQuery();
        ptQuery.setProductId(entity.getProductId());
        List<ProductTopicEntity> ptList = productTopicService.find(ptQuery, context);
        if (null != ptList && ptList.size() != 0) {
            ptList.forEach((ptEntity) -> {
                String topic = ptEntity.getTopic().replace("${deviceName}", entity.getDeviceName());
                itemList.forEach((itemEntity) -> {
                    //服务
                    if (topic.contains("${tsl.service.identifier}")) {
                        if (itemEntity.getProductFunctionType() == ProductFunctionTypeEnum.SERVICE.getValue()) {
                            dtList.add(comDeviceTopic(entity, ptEntity, topic, itemEntity));
                        }
                    } else {//事件
                        if (itemEntity.getProductFunctionType() == ProductFunctionTypeEnum.EVENT.getValue()) {
                            dtList.add(comDeviceTopic(entity, ptEntity, topic, itemEntity));
                        }
                    }
                });
            });
        }
        deviceTopicService.save(dtList, context);
        //更新redis设备topic
        if (null != dtList && dtList.size() != 0) {
            Map<String, Object> hash = new HashMap<String, Object>();
            String clientId = entity.getProductKey() + ":" + entity.getDeviceName();
            dtList.forEach((dt) -> {
                Subscription subscription = new Subscription();
                subscription.setClientId(clientId);
                subscription.setTopic(dt.getTopic());
                subscription.setQos(dt.getQos());
                hash.put(subscription.getTopic(), subscription);
            });
            cacheService.hmset(RedisKeys.getDeviceTopicBrokerKey(clientId), hash);
        }
        return true;
    }

    /**
     * 组合设备topic实体
     *
     * @param entity
     * @param ptEntity
     * @param topic
     * @param itemEntity
     * @return
     */
    private DeviceTopicEntity comDeviceTopic(DeviceEntity entity, ProductTopicEntity ptEntity, String topic, ProductFunctionItemEntity itemEntity) {
        DeviceTopicEntity dtEntity = new DeviceTopicEntity();
        dtEntity.setCustomerId(entity.getCustomerId());
        dtEntity.setProductId(entity.getProductId());
        dtEntity.setDeviceTopicHandle(ptEntity.getDeviceTopicHandle());
        dtEntity.setDeviceTopicRemark(ptEntity.getProductTopicRemark());
        dtEntity.setCreateTime(entity.getCreateTime());
        dtEntity.setUpdateTime(entity.getUpdateTime());
        dtEntity.setDeviceId(entity.getId());
        dtEntity.setDeviceName(entity.getDeviceName());
        dtEntity.setTopic(topic.replace(topic.contains("${tsl.event.identifier}") ?
                "${tsl.event.identifier}" : "${tsl.service.identifier}", itemEntity.getFunctionIdentifier()));
        dtEntity.setQos(ptEntity.getQos());
        dtEntity.setProductKey(entity.getProductKey());
        return dtEntity;
    }

    @Override
    protected void removeBefore(Long[] ids, Context context) throws AppException {
        for (Long id : ids) {
            DeviceQuery query = new DeviceQuery();
            query.setId(id);
            query.setConnectStatus(DeviceStatusEnum.ONLINE.getValue());
            if (dao.getCount(query) > 0) {
                throw new AppException("在线的设备不能删除");
            }
            DeviceQuery query1 = new DeviceQuery();
            query1.setId(id);
            query1.setConnectStatus(UseStatusEnum.ONLINE.getValue());
            if (dao.getCount(query1) > 0) {
                throw new AppException("启用的设备不能删除");
            }
        }
        super.removeBefore(ids, context);
    }

    @Override
    protected void removeAfter(Long[] ids, Context context, int result) throws AppException {
        //删除设备相关的topic与缓存关系
        delDeviceTopicByDevIds(ids, context);
        super.removeAfter(ids, context, result);
    }


    @Override
    public void delDeviceTopicByDevIds(Long[] ids, Context context) throws AppException {
        for (Long id : ids) {
            DeviceTopicQuery dtQuery = new DeviceTopicQuery();
            dtQuery.setDeviceId(id);
            List<DeviceTopicEntity> dtList = deviceTopicService.find(dtQuery, context);
            if (null != dtList && dtList.size() != 0) {
                Long[] dtIds = dtList.stream().map(DeviceTopicEntity::getId).toArray(Long[]::new);
                deviceTopicService.remove(dtIds, context);
                cacheService.del(RedisKeys.getDeviceTopicBrokerKey(dtList.get(0).getProductKey()
                        + ":" + dtList.get(0).getDeviceName()));
            }
        }
    }

    public void updateDeviceBrokerInfo(DeviceEntity entity) throws AppException {
        DeviceBrokerInfo brokerInfo = BeanUtil.covert(entity, DeviceBrokerInfo.class);
        brokerInfo.setClientId(entity.getProductKey() + ":" + entity.getDeviceName());
        brokerInfo.setAvgFlowRate(50);
        brokerInfo.setMaxFlowRate(100);
        cacheService.hset(RedisKeys.getBrokerDeviceInfoKey(), brokerInfo.getClientId(), brokerInfo);
    }
}