package com.mortals.xhx.module.device.web;

import com.mortals.framework.ap.SysConstains;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.DbOrderKind;
import com.mortals.framework.service.ICacheService;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.common.code.DeviceStatusEnum;
import com.mortals.xhx.common.code.UseStatusEnum;
import com.mortals.xhx.module.customer.model.CustomerEntity;
import com.mortals.xhx.module.customer.service.CustomerService;
import com.mortals.xhx.module.product.model.ProductEntity;
import com.mortals.xhx.module.product.service.ProductService;
import com.mortals.xhx.service.common.code.DeviceStatusInfo;
import com.mortals.xhx.service.common.code.RedisKeys;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.mortals.framework.model.Context;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.mortals.framework.web.BaseCRUDJsonMappingController;
import com.mortals.xhx.module.device.model.DeviceEntity;
import com.mortals.xhx.module.device.service.DeviceService;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * DeviceController
 * <p>
 * 设备 控制器
 *
 * @author zxfei
 * @date 2021-11-09
 */
@RestController
@RequestMapping("device")
public class DeviceController extends BaseCRUDJsonMappingController<DeviceService, DeviceForm, DeviceEntity, Long> {

    @Autowired
    private ProductService productService;

    @Autowired
    private CustomerService customerService;

    @Autowired
    private ICacheService cacheService;

    public DeviceController() {
        super.setFormClass(DeviceForm.class);
        super.setModuleDesc("设备表");
    }

    @Override
    protected void init(HttpServletRequest request, HttpServletResponse response, DeviceForm form,
                        Map<String, Object> model, Context context) {
        this.addDict(model, "productId",
                productService.getCacheList().stream().collect(Collectors.toMap(e -> e.getId().toString(), ProductEntity::getProductName)));
        this.addDict(model, "customerId",
                customerService.getCacheList().stream().collect(Collectors.toMap(e -> e.getId().toString(), CustomerEntity::getLoginName)));
        this.addDict(model, "useStatus", UseStatusEnum.getEnumMap());
        this.addDict(model, "connectStatus", DeviceStatusEnum.getEnumMap());
        super.init(request, response, form, model, context);
    }

    @Override
    protected void doListBefore(HttpServletRequest request, HttpServletResponse response, DeviceForm form, Map<String, Object> model, Context context) throws AppException {
        Map<String, String> orderCols = new HashMap<>();
        orderCols.put("id", DbOrderKind.DESC.name());
        form.getQuery().setOrderCols(orderCols);
        super.doListBefore(request, response, form, model, context);
    }

    /**
     * 查询后处理
     *
     * @param request
     * @param response
     * @param form
     * @return
     */
    protected int doListAfter(HttpServletRequest request, HttpServletResponse response, DeviceForm form, Map<String, Object> model, Context context) throws AppException {
        List<DeviceEntity> list = (List<DeviceEntity>) model.get(SysConstains.RESULT_KEY);
        list.forEach((e) -> {
            DeviceStatusInfo statusInfo = cacheService.hget(RedisKeys.getDeviceStatusBrokerKey(), e.getProductKey() + ":" + e.getDeviceName(), DeviceStatusInfo.class);
            if (null != statusInfo) {
                Date date = new Date(statusInfo.getTime());
                e.setConnectStatus(statusInfo.getOnline());
                if (statusInfo.getOnline() == DeviceStatusEnum.ONLINE.getValue()) {
                    e.setOnlineTime(date);
                } else {
                    e.setOfflineTime(date);
                }
            }
        });
        return super.doListAfter(request, response, form, model, context);
    }
}