package com.mortals.xhx.module.device.web;

import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.DbOrderKind;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.common.code.DeviceTopicHandleEnum;
import com.mortals.xhx.common.code.QosTypeEnum;
import com.mortals.xhx.module.customer.model.CustomerEntity;
import com.mortals.xhx.module.customer.service.CustomerService;
import com.mortals.xhx.module.product.model.ProductEntity;
import com.mortals.xhx.module.product.service.ProductService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.mortals.framework.model.Context;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.mortals.framework.web.BaseCRUDJsonMappingController;
import com.mortals.xhx.module.device.model.DeviceTopicEntity;
import com.mortals.xhx.module.device.service.DeviceTopicService;

import java.util.HashMap;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * DeviceTopicController
 * <p>
 * 设备topic 控制器
 *
 * @author zxfei
 * @date 2021-11-09
 */
@RestController
@RequestMapping("device/topic")
public class DeviceTopicController extends BaseCRUDJsonMappingController<DeviceTopicService, DeviceTopicForm, DeviceTopicEntity, Long> {

    @Autowired
    private ProductService productService;

    @Autowired
    private CustomerService customerService;

    public DeviceTopicController() {
        super.setFormClass(DeviceTopicForm.class);
        super.setModuleDesc("设备topic表");
    }

    @Override
    protected void init(HttpServletRequest request, HttpServletResponse response, DeviceTopicForm form,
                        Map<String, Object> model, Context context) {
        this.addDict(model, "productId",
                productService.getCacheList().stream().collect(Collectors.toMap(e -> e.getId().toString(), ProductEntity::getProductName)));
        this.addDict(model, "customerId",
                customerService.getCacheList().stream().collect(Collectors.toMap(e -> e.getId().toString(), CustomerEntity::getLoginName)));
        this.addDict(model, "deviceTopicHandle", DeviceTopicHandleEnum.getEnumMap());
        this.addDict(model, "qos", QosTypeEnum.getEnumMap());
        super.init(request, response, form, model, context);
    }

    @Override
    protected void doListBefore(HttpServletRequest request, HttpServletResponse response, DeviceTopicForm form, Map<String, Object> model, Context context) throws AppException {
        Map<String, String> orderCols = new HashMap<>();
        orderCols.put("id", DbOrderKind.DESC.name());
        form.getQuery().setOrderCols(orderCols);
        super.doListBefore(request, response, form, model, context);
    }
}