package com.mortals.xhx.module.product.model;
import java.util.List;
import java.util.ArrayList;
import com.mortals.framework.annotation.Excel;
import com.mortals.framework.model.BaseEntityLong;
import com.mortals.xhx.module.product.model.vo.ProductVo;
/**
* 产品实体对象
*
* @author zxfei
* @date 2021-11-09
*/

public class ProductEntity extends ProductVo {
    private static final long serialVersionUID = 1L;

    /**
    * 用户ID
    */
    private Long customerId;
    /**
    * 产品名称，为产品命名。产品名称在账号内具有唯一性。例如，可以填写为产品型号。支持中文、英文字母、日文、数字、下划线（_）、短划线（-）、at（@）和英文圆括号，长度限制4~30个字符。
    */
    private String productName;
    /**
    * 所属品类，相当于产品模板。1，标准品类：平台已为标准品类预定义了功能模板。例如，能源管理 > 电表品类已预定义用电量、电压、电流、总累积量等电表标准功能。选择该品类，创建的产品具有预定义的功能。您可以在该产品的产品详情页功能定义页签下，编辑、修改、新增功能。2，自定义品类：产品创建成功后，需根据实际需要，自定义物模型。
    */
    private Integer productType;
    /**
    * 节点类型，产品下设备的类型。1，直连设备：具有IP地址，可直接连接平台，且不能挂载子设备，但可作为子设备挂载到网关下的设备。2，网关子设备：不直接连接平台，而是通过网关设备接入平台的设备。3，网关设备：可以挂载子设备的直连设备。网关具有子设备管理模块，可以维持子设备的拓扑关系，将与子设备的拓扑关系同步到云端。
    */
    private Integer nodeType;
    /**
    * 接入网关协议，节点类型选择为网关子设备的参数。表示该产品下的设备作为子设备与网关的通讯协议类型。自定义：表示子设备和网关之间是其它标准或私有协议。1,Modbus：表示子设备和网关之间的通讯协议是Modbus。2,OPC UA：表示子设备和网关之间的通讯协议是OPC UA。3,ZigBee：表示子设备和网关之间的通讯协议是ZigBee。4,BLE：表示子设备和网关之间的通讯协议是BLE。
    */
    private Integer connGateProtocol;
    /**
    * 连网方式，直连设备和网关设备的连网方式。1,Wi-Fi2,蜂窝（2G/3G/4G/5G）3,以太网4,LoRaWAN5,其他
    */
    private Integer networkType;
    /**
    * 入网凭证
    */
    private String networkProof;
    /**
    * 数据格式，设备上下行的数据格式。1，标准数据格式：是平台为开发者提供的设备与云端的数据交换协议，采用JSON格式。2，透传/自定义：选择透传/自定义。需在定义数据解析脚本，将上行的自定义格式的数据转换为标准数据格式；将下行的标准数据格式数据解析为设备自定义格式，设备才能与云端进行通信。
    */
    private Integer dataType;
    /**
    * 校验类型，设备上报物模型数据时，平台将按设置的数据校验方式，根据物模型定义进行数据校验。1,弱校验：只校验设备数据的idetifier和dataType字段，不校验其他字段，流转全量数据。2,免校验：对上报的数据不进行校验，流转全量数据。
    */
    private Integer checkType;
    /**
    * 认证方式，设备接入平台的安全认证方式。产品创建成功后，认证方式不可变更。1，设备密钥：使用物联网平台为设备生成的DeviceSecret进行设备认证签名计算。使用DeviceSecret签名计算，2，ID²：ID²认证提供设备与物联网平台的双向身份认证能力，通过建立轻量化的安全链路（iTLS）来保障数据的安全性。3，X.509证书：使用X.509数字证书进行设备身份认证。
    */
    private Integer authType;
    /**
    * 产品备注
    */
    private String productRemark;
    /**
    * productKey，平台为产品颁发的全局唯一标识符
    */
    private String productKey;
    /**
    * 产品状态，1，开发中2，发布
    */
    private Integer productStatus;
    /**
    * 产品下发布的物模型
    */
    private String productTSL;
    /**
    * qos
    */
    private Integer qos;



    public ProductEntity(){}
    /**
    * 获取 用户ID
    * @return Long
    */
    public Long getCustomerId(){
        return customerId;
    }
    /**
    * 设置 用户ID
    * @param customerId
    */
    public void setCustomerId(Long customerId){
        this.customerId = customerId;
    }
    /**
    * 获取 产品名称，为产品命名。产品名称在账号内具有唯一性。例如，可以填写为产品型号。支持中文、英文字母、日文、数字、下划线（_）、短划线（-）、at（@）和英文圆括号，长度限制4~30个字符。
    * @return String
    */
    public String getProductName(){
        return productName;
    }
    /**
    * 设置 产品名称，为产品命名。产品名称在账号内具有唯一性。例如，可以填写为产品型号。支持中文、英文字母、日文、数字、下划线（_）、短划线（-）、at（@）和英文圆括号，长度限制4~30个字符。
    * @param productName
    */
    public void setProductName(String productName){
        this.productName = productName;
    }
    /**
    * 获取 所属品类，相当于产品模板。1，标准品类：平台已为标准品类预定义了功能模板。例如，能源管理 > 电表品类已预定义用电量、电压、电流、总累积量等电表标准功能。选择该品类，创建的产品具有预定义的功能。您可以在该产品的产品详情页功能定义页签下，编辑、修改、新增功能。2，自定义品类：产品创建成功后，需根据实际需要，自定义物模型。
    * @return Integer
    */
    public Integer getProductType(){
        return productType;
    }
    /**
    * 设置 所属品类，相当于产品模板。1，标准品类：平台已为标准品类预定义了功能模板。例如，能源管理 > 电表品类已预定义用电量、电压、电流、总累积量等电表标准功能。选择该品类，创建的产品具有预定义的功能。您可以在该产品的产品详情页功能定义页签下，编辑、修改、新增功能。2，自定义品类：产品创建成功后，需根据实际需要，自定义物模型。
    * @param productType
    */
    public void setProductType(Integer productType){
        this.productType = productType;
    }
    /**
    * 获取 节点类型，产品下设备的类型。1，直连设备：具有IP地址，可直接连接平台，且不能挂载子设备，但可作为子设备挂载到网关下的设备。2，网关子设备：不直接连接平台，而是通过网关设备接入平台的设备。3，网关设备：可以挂载子设备的直连设备。网关具有子设备管理模块，可以维持子设备的拓扑关系，将与子设备的拓扑关系同步到云端。
    * @return Integer
    */
    public Integer getNodeType(){
        return nodeType;
    }
    /**
    * 设置 节点类型，产品下设备的类型。1，直连设备：具有IP地址，可直接连接平台，且不能挂载子设备，但可作为子设备挂载到网关下的设备。2，网关子设备：不直接连接平台，而是通过网关设备接入平台的设备。3，网关设备：可以挂载子设备的直连设备。网关具有子设备管理模块，可以维持子设备的拓扑关系，将与子设备的拓扑关系同步到云端。
    * @param nodeType
    */
    public void setNodeType(Integer nodeType){
        this.nodeType = nodeType;
    }
    /**
    * 获取 接入网关协议，节点类型选择为网关子设备的参数。表示该产品下的设备作为子设备与网关的通讯协议类型。自定义：表示子设备和网关之间是其它标准或私有协议。1,Modbus：表示子设备和网关之间的通讯协议是Modbus。2,OPC UA：表示子设备和网关之间的通讯协议是OPC UA。3,ZigBee：表示子设备和网关之间的通讯协议是ZigBee。4,BLE：表示子设备和网关之间的通讯协议是BLE。
    * @return Integer
    */
    public Integer getConnGateProtocol(){
        return connGateProtocol;
    }
    /**
    * 设置 接入网关协议，节点类型选择为网关子设备的参数。表示该产品下的设备作为子设备与网关的通讯协议类型。自定义：表示子设备和网关之间是其它标准或私有协议。1,Modbus：表示子设备和网关之间的通讯协议是Modbus。2,OPC UA：表示子设备和网关之间的通讯协议是OPC UA。3,ZigBee：表示子设备和网关之间的通讯协议是ZigBee。4,BLE：表示子设备和网关之间的通讯协议是BLE。
    * @param connGateProtocol
    */
    public void setConnGateProtocol(Integer connGateProtocol){
        this.connGateProtocol = connGateProtocol;
    }
    /**
    * 获取 连网方式，直连设备和网关设备的连网方式。1,Wi-Fi2,蜂窝（2G/3G/4G/5G）3,以太网4,LoRaWAN5,其他
    * @return Integer
    */
    public Integer getNetworkType(){
        return networkType;
    }
    /**
    * 设置 连网方式，直连设备和网关设备的连网方式。1,Wi-Fi2,蜂窝（2G/3G/4G/5G）3,以太网4,LoRaWAN5,其他
    * @param networkType
    */
    public void setNetworkType(Integer networkType){
        this.networkType = networkType;
    }
    /**
    * 获取 入网凭证
    * @return String
    */
    public String getNetworkProof(){
        return networkProof;
    }
    /**
    * 设置 入网凭证
    * @param networkProof
    */
    public void setNetworkProof(String networkProof){
        this.networkProof = networkProof;
    }
    /**
    * 获取 数据格式，设备上下行的数据格式。1，标准数据格式：是平台为开发者提供的设备与云端的数据交换协议，采用JSON格式。2，透传/自定义：选择透传/自定义。需在定义数据解析脚本，将上行的自定义格式的数据转换为标准数据格式；将下行的标准数据格式数据解析为设备自定义格式，设备才能与云端进行通信。
    * @return Integer
    */
    public Integer getDataType(){
        return dataType;
    }
    /**
    * 设置 数据格式，设备上下行的数据格式。1，标准数据格式：是平台为开发者提供的设备与云端的数据交换协议，采用JSON格式。2，透传/自定义：选择透传/自定义。需在定义数据解析脚本，将上行的自定义格式的数据转换为标准数据格式；将下行的标准数据格式数据解析为设备自定义格式，设备才能与云端进行通信。
    * @param dataType
    */
    public void setDataType(Integer dataType){
        this.dataType = dataType;
    }
    /**
    * 获取 校验类型，设备上报物模型数据时，平台将按设置的数据校验方式，根据物模型定义进行数据校验。1,弱校验：只校验设备数据的idetifier和dataType字段，不校验其他字段，流转全量数据。2,免校验：对上报的数据不进行校验，流转全量数据。
    * @return Integer
    */
    public Integer getCheckType(){
        return checkType;
    }
    /**
    * 设置 校验类型，设备上报物模型数据时，平台将按设置的数据校验方式，根据物模型定义进行数据校验。1,弱校验：只校验设备数据的idetifier和dataType字段，不校验其他字段，流转全量数据。2,免校验：对上报的数据不进行校验，流转全量数据。
    * @param checkType
    */
    public void setCheckType(Integer checkType){
        this.checkType = checkType;
    }
    /**
    * 获取 认证方式，设备接入平台的安全认证方式。产品创建成功后，认证方式不可变更。1，设备密钥：使用物联网平台为设备生成的DeviceSecret进行设备认证签名计算。使用DeviceSecret签名计算，2，ID²：ID²认证提供设备与物联网平台的双向身份认证能力，通过建立轻量化的安全链路（iTLS）来保障数据的安全性。3，X.509证书：使用X.509数字证书进行设备身份认证。
    * @return Integer
    */
    public Integer getAuthType(){
        return authType;
    }
    /**
    * 设置 认证方式，设备接入平台的安全认证方式。产品创建成功后，认证方式不可变更。1，设备密钥：使用物联网平台为设备生成的DeviceSecret进行设备认证签名计算。使用DeviceSecret签名计算，2，ID²：ID²认证提供设备与物联网平台的双向身份认证能力，通过建立轻量化的安全链路（iTLS）来保障数据的安全性。3，X.509证书：使用X.509数字证书进行设备身份认证。
    * @param authType
    */
    public void setAuthType(Integer authType){
        this.authType = authType;
    }
    /**
    * 获取 产品备注
    * @return String
    */
    public String getProductRemark(){
        return productRemark;
    }
    /**
    * 设置 产品备注
    * @param productRemark
    */
    public void setProductRemark(String productRemark){
        this.productRemark = productRemark;
    }
    /**
    * 获取 productKey，平台为产品颁发的全局唯一标识符
    * @return String
    */
    public String getProductKey(){
        return productKey;
    }
    /**
    * 设置 productKey，平台为产品颁发的全局唯一标识符
    * @param productKey
    */
    public void setProductKey(String productKey){
        this.productKey = productKey;
    }
    /**
    * 获取 产品状态，1，开发中2，发布
    * @return Integer
    */
    public Integer getProductStatus(){
        return productStatus;
    }
    /**
    * 设置 产品状态，1，开发中2，发布
    * @param productStatus
    */
    public void setProductStatus(Integer productStatus){
        this.productStatus = productStatus;
    }
    /**
    * 获取 产品下发布的物模型
    * @return String
    */
    public String getProductTSL(){
        return productTSL;
    }
    /**
    * 设置 产品下发布的物模型
    * @param productTSL
    */
    public void setProductTSL(String productTSL){
        this.productTSL = productTSL;
    }
    /**
    * 获取 qos
    * @return Integer
    */
    public Integer getQos(){
        return qos;
    }
    /**
    * 设置 qos
    * @param qos
    */
    public void setQos(Integer qos){
        this.qos = qos;
    }




    @Override
    public int hashCode() {
         return this.getId().hashCode();
    }
    @Override
    public boolean equals(Object obj) {
        if (obj == null) return false;
        if (obj instanceof ProductEntity) {
            ProductEntity tmp = (ProductEntity) obj;
            if (this.getId() == tmp.getId()) {
                 return true;
            }
        }
        return false;
    }

    public String toString(){
        StringBuilder sb = new StringBuilder("");
        sb.append(",customerId:").append(getCustomerId());
        sb.append(",productName:").append(getProductName());
        sb.append(",productType:").append(getProductType());
        sb.append(",nodeType:").append(getNodeType());
        sb.append(",connGateProtocol:").append(getConnGateProtocol());
        sb.append(",networkType:").append(getNetworkType());
        sb.append(",networkProof:").append(getNetworkProof());
        sb.append(",dataType:").append(getDataType());
        sb.append(",checkType:").append(getCheckType());
        sb.append(",authType:").append(getAuthType());
        sb.append(",productRemark:").append(getProductRemark());
        sb.append(",productKey:").append(getProductKey());
        sb.append(",productStatus:").append(getProductStatus());
        sb.append(",productTSL:").append(getProductTSL());
        sb.append(",qos:").append(getQos());
        return sb.toString();
    }

    public void initAttrValue(){
            this.customerId = null;
            this.productName = null;
            this.productType = null;
            this.nodeType = null;
            this.connGateProtocol = null;
            this.networkType = null;
            this.networkProof = null;
            this.dataType = null;
            this.checkType = null;
            this.authType = null;
            this.productRemark = null;
            this.productKey = null;
            this.productStatus = null;
            this.productTSL = null;
            this.qos = null;
    }
}