package com.mortals.xhx.module.product.service.impl;

import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.common.code.ProductFunctionStatusEnum;
import com.mortals.xhx.module.product.model.ProductFunctionItemDetailQuery;
import com.mortals.xhx.module.product.model.ProductFunctionItemEntity;
import com.mortals.xhx.module.product.model.ProductFunctionItemQuery;
import com.mortals.xhx.module.product.service.ProductFunctionItemService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.xhx.module.product.dao.ProductFunctionItemDetailDao;
import com.mortals.xhx.module.product.model.ProductFunctionItemDetailEntity;
import com.mortals.xhx.module.product.service.ProductFunctionItemDetailService;

import javax.annotation.Resource;
import java.util.List;


/**
 * ProductFunctionItemDetailService
 * 产品物模型节点明细 service实现
 *
 * @author zxfei
 * @date 2021-11-09
 */
@Service("productFunctionItemDetailService")
public class ProductFunctionItemDetailServiceImpl extends AbstractCRUDServiceImpl<ProductFunctionItemDetailDao, ProductFunctionItemDetailEntity, Long> implements ProductFunctionItemDetailService {
    @Autowired
    private ProductFunctionItemService productFunctionItemService;

    @Override
    protected void validData(ProductFunctionItemDetailEntity entity, Context context) throws AppException {
        ProductFunctionItemDetailQuery query = new ProductFunctionItemDetailQuery();
        query.setFunctionItemId(entity.getFunctionItemId());
        query.setDetailIdentifier(entity.getDetailIdentifier());
        List<ProductFunctionItemDetailEntity> list = dao.getList(query);
        //判断物模型节点状态
        ProductFunctionItemQuery itemQuery = new ProductFunctionItemQuery();
        itemQuery.setId(entity.getFunctionItemId());
        itemQuery.setProductFunctionStatus(ProductFunctionStatusEnum.ROUGH.getValue());
        if (productFunctionItemService.count(itemQuery, new Context()) == 0) {
            throw new AppException("对应物模型节点不是草稿状态,不能编辑新增物模型明细");
        }
        if (entity.getId() != null) {//修改
            // 校验重复标识符
            if (!list.isEmpty() && list.get(0).getDetailIdentifier().equals(entity.getDetailIdentifier())
                    && list.get(0).getFunctionItemId().longValue() == entity.getFunctionItemId().longValue()
                    && list.get(0).getId().longValue() != entity.getId()) {
                throw new AppException("该物模型节点已存在相同的明细标识符");
            }
        } else {//新增
            if (!list.isEmpty()) {
                throw new AppException("该物模型节点已存在相同的明细标识符");
            }
        }
        super.validData(entity, context);
    }

    @Override
    protected void removeBefore(Long[] ids, Context context) throws AppException {
        for (Long id : ids) {
            ProductFunctionItemDetailEntity entity = dao.get(id);
            ProductFunctionItemQuery query = new ProductFunctionItemQuery();
            query.setId(entity.getFunctionItemId());
            query.setProductFunctionStatus(ProductFunctionStatusEnum.USER.getValue());
            if (productFunctionItemService.count(query, new Context()) > 0) {
                throw new AppException("对应物模型节点已发布不能删除节点下明细");
            }
        }
        super.removeBefore(ids, context);
    }

    @Override
    protected void saveBefore(ProductFunctionItemDetailEntity entity, Context context) throws AppException {
        ProductFunctionItemEntity functionItemEntity = productFunctionItemService.get(entity.getFunctionItemId(), context);
        entity.setProductId(functionItemEntity.getProductId());
        entity.setProductKey(functionItemEntity.getProductKey());
        super.saveBefore(entity, context);
    }
}