package com.mortals.xhx.module.product.service.impl;

import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.common.code.ProductFunctionTypeEnum;
import com.mortals.xhx.common.code.ProductStatusEnum;
import com.mortals.xhx.module.product.model.ProductEntity;
import com.mortals.xhx.module.product.model.ProductFunctionItemQuery;
import com.mortals.xhx.module.product.model.ProductQuery;
import com.mortals.xhx.module.product.service.ProductService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.xhx.module.product.dao.ProductFunctionItemDao;
import com.mortals.xhx.module.product.model.ProductFunctionItemEntity;
import com.mortals.xhx.module.product.service.ProductFunctionItemService;

import javax.annotation.Resource;
import java.util.List;


/**
 * ProductFunctionItemService
 * 产品物模型节点 service实现
 *
 * @author zxfei
 * @date 2021-11-09
 */
@Service("productFunctionItemService")
public class ProductFunctionItemServiceImpl extends AbstractCRUDServiceImpl<ProductFunctionItemDao, ProductFunctionItemEntity, Long> implements ProductFunctionItemService {
    @Autowired
    private ProductService productService;

    @Override
    protected void validData(ProductFunctionItemEntity entity, Context context) throws AppException {
        ProductFunctionItemQuery query = new ProductFunctionItemQuery();
        query.setProductId(entity.getProductId());
        query.setFunctionIdentifier(entity.getFunctionIdentifier());
        List<ProductFunctionItemEntity> list = dao.getList(query);
        //判断产品状态
        ProductQuery productQuery = new ProductQuery();
        productQuery.setId(entity.getProductId());
        productQuery.setProductStatus(ProductStatusEnum.PUBLISH.getValue());
        if (productService.count(productQuery, new Context()) > 0) {
            throw new AppException("对应产品已发布，不能新增编辑物模型");
        }
        if (entity.getId() != null) {//修改
            // 校验重复标识符
            if (!list.isEmpty() && list.get(0).getFunctionIdentifier().equals(entity.getFunctionIdentifier())
                    && list.get(0).getProductId().longValue() == entity.getProductId().longValue()
                    && list.get(0).getId().longValue() != entity.getId()) {
                throw new AppException("该产品已存在相同的物模型节点标识符");
            }
        } else {//新增
            if (!list.isEmpty()) {
                throw new AppException("该产品已存在相同的物模型节点标识符");
            }
        }
        super.validData(entity, context);
    }

    @Override
    protected void removeBefore(Long[] ids, Context context) throws AppException {
        for (Long id : ids) {
            ProductFunctionItemEntity entity = dao.get(id);
            ProductQuery query = new ProductQuery();
            query.setId(entity.getProductId());
            query.setProductStatus(ProductStatusEnum.PUBLISH.getValue());
            if (productService.count(query, new Context()) > 0) {
                throw new AppException("对应产品已发布，不能删除");
            }
        }
        super.removeBefore(ids, context);
    }

    @Override
    protected void saveBefore(ProductFunctionItemEntity entity, Context context) throws AppException {
        ProductEntity cache = productService.getCache(String.valueOf(entity.getProductId()));
        entity.setProductKey(cache.getProductKey());
        entity.setCustomerId(cache.getCustomerId());
        entity.setProductFunctionMethod(ProductFunctionTypeEnum.getByValue(entity.getProductFunctionType()).getRemark()
                + "." + entity.getFunctionIdentifier());
        super.saveBefore(entity, context);
    }
}