package com.mortals.xhx.module.product.service.impl;

import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.xhx.common.code.ProductStatusEnum;
import com.mortals.xhx.common.code.ProductTopicEnum;
import com.mortals.xhx.module.device.model.DeviceQuery;
import com.mortals.xhx.module.device.service.DeviceService;
import com.mortals.xhx.module.product.model.*;
import com.mortals.xhx.module.product.service.ProductFunctionItemDetailService;
import com.mortals.xhx.module.product.service.ProductFunctionItemService;
import com.mortals.xhx.module.product.service.ProductTopicService;
import com.mortals.xhx.utils.KeyUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.mortals.framework.service.impl.AbstractCRUDCacheServiceImpl;
import com.mortals.xhx.module.product.dao.ProductDao;
import com.mortals.xhx.module.product.service.ProductService;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;


/**
 * ProductService
 * 产品 service实现
 *
 * @author zxfei
 * @date 2021-11-09
 */
@Service("productService")
public class ProductServiceImpl extends AbstractCRUDCacheServiceImpl<ProductDao, ProductEntity, Long> implements ProductService {

    @Autowired
    private ProductTopicService productTopicService;

    @Autowired
    private DeviceService deviceService;

    @Autowired
    private ProductFunctionItemService productFunctionItemService;

    @Autowired
    private ProductFunctionItemDetailService productFunctionItemDetailService;

    @Override
    protected String getExtKey(ProductEntity data) {
        return data.getProductKey();
    }

    @Override
    protected void validData(ProductEntity entity, Context context) throws AppException {
        ProductQuery query = new ProductQuery();
        query.setProductName(entity.getProductName());
        List<ProductEntity> list = dao.getList(query);
        if (entity.getId() != null) {//修改
            // 校验重复编码
            if (!list.isEmpty() && list.get(0).getProductName().equals(entity.getProductName())
                    && list.get(0).getId().longValue() != entity.getId()) {
                throw new AppException("已存在相同的产品名称");
            }
        } else {//新增
            ProductQuery pQuery = new ProductQuery();
            pQuery.setProductKey(entity.getProductKey());
            if (dao.getCount(pQuery) > 0) {
                throw new AppException("服务繁忙，请重新提交");
            }
            if (!list.isEmpty()) {
                throw new AppException("已存在相同的产品名称");
            }
        }
        super.validData(entity, context);
    }

    @Override
    protected void removeBefore(Long[] ids, Context context) throws AppException {
        for (Long id : ids) {
            ProductEntity entity = new ProductEntity();
            entity.setId(id);
            entity.setProductStatus(ProductStatusEnum.PUBLISH.getValue());
            if (dao.getCount(entity) > 0) {
                throw new AppException("已发布的产品不能删除");
            }
            DeviceQuery query = new DeviceQuery();
            query.setProductId(id);
            if (deviceService.count(query, new Context()) > 0) {
                throw new AppException("该产品下还有设备，不能删除");
            }
        }
        super.removeBefore(ids, context);
    }

    @Override
    protected void removeAfter(Long[] ids, Context context, int result) throws AppException {
        //删除产品相关的物模型与topic
        for (Long id : ids) {
            //删除产品topic
            ProductTopicQuery ptQuery = new ProductTopicQuery();
            ptQuery.setProductId(id);
            List<ProductTopicEntity> ptList = productTopicService.find(ptQuery, context);
            if (null != ptList && ptList.size() != 0) {
                Long[] ptIds = ptList.stream().map(ProductTopicEntity::getId).toArray(Long[]::new);
                productTopicService.remove(ptIds, context);
            }
            //删除产品物模型节点
            ProductFunctionItemQuery pfiQuery = new ProductFunctionItemQuery();
            pfiQuery.setProductId(id);
            List<ProductFunctionItemEntity> pfiList = productFunctionItemService.find(pfiQuery, context);
            if (null != pfiList && pfiList.size() != 0) {
                Long[] pfiIds = pfiList.stream().map(ProductFunctionItemEntity::getId).toArray(Long[]::new);
                productFunctionItemService.remove(pfiIds, context);
            }
            //删除产品物模型详情
            ProductFunctionItemDetailQuery pfidQuery = new ProductFunctionItemDetailQuery();
            pfidQuery.setProductId(id);
            List<ProductFunctionItemDetailEntity> pfidList = productFunctionItemDetailService.find(pfidQuery, context);
            if (null != pfidList && pfidList.size() != 0) {
                Long[] pfidIds = pfidList.stream().map(ProductFunctionItemDetailEntity::getId).toArray(Long[]::new);
                productFunctionItemDetailService.remove(pfidIds, context);
            }
        }
        super.removeAfter(ids, context, result);
    }

    @Override
    protected void saveBefore(ProductEntity entity, Context context) throws AppException {
        //TODO TSL
        entity.setProductTSL("");
        entity.setProductKey(KeyUtil.toHex());
        entity.setProductStatus(ProductStatusEnum.DEVELOP.getValue());//新建默认开发中状态
        entity.setCustomerId(entity.getCreateUserId());
        super.saveBefore(entity, context);
    }

    @Override
    protected void saveAfter(ProductEntity entity, Context context) throws AppException {
        //新增产品topic
        List<ProductTopicEntity> list = new ArrayList<ProductTopicEntity>(4);
        for (ProductTopicEnum pfEnum : ProductTopicEnum.values()) {
            ProductTopicEntity ptEntity = new ProductTopicEntity();
            ptEntity.setCustomerId(entity.getCustomerId());
            ptEntity.setProductId(entity.getId());
            ptEntity.setTopic(pfEnum.getMsg().replace("${productKey}", entity.getProductKey()));
            ptEntity.setDeviceTopicHandle(pfEnum.getHandle());
            ptEntity.setProductTopicRemark(pfEnum.getRemark());
            ptEntity.setCreateTime(entity.getCreateTime());
            ptEntity.setUpdateTime(entity.getUpdateTime());
            ptEntity.setQos(entity.getQos());
            ptEntity.setProductKey(entity.getProductKey());
            list.add(ptEntity);
        }
        productTopicService.save(list, context);
        super.saveAfter(entity, context);
    }

    @Override
    protected void updateAfter(ProductEntity entity, Context context) throws AppException {
        super.updateAfter(entity, context);
    }
}