package com.mortals.xhx.module.product.web;

import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.DbOrderKind;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.common.code.*;
import com.mortals.xhx.module.customer.model.CustomerEntity;
import com.mortals.xhx.module.customer.service.CustomerService;
import com.mortals.xhx.module.product.model.ProductEntity;
import com.mortals.xhx.module.product.service.ProductService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.mortals.framework.model.Context;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.mortals.framework.web.BaseCRUDJsonMappingController;
import com.mortals.xhx.module.product.model.ProductFunctionItemEntity;
import com.mortals.xhx.module.product.service.ProductFunctionItemService;

import java.util.HashMap;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * ProductFunctionItemController
 * <p>
 * 产品物模型节点 控制器
 *
 * @author zxfei
 * @date 2021-11-09
 */
@RestController
@RequestMapping("product/function/item")
public class ProductFunctionItemController extends BaseCRUDJsonMappingController<ProductFunctionItemService, ProductFunctionItemForm, ProductFunctionItemEntity, Long> {

    @Autowired
    private ProductService productService;

    @Autowired
    private CustomerService customerService;

    public ProductFunctionItemController() {
        super.setFormClass(ProductFunctionItemForm.class);
        super.setModuleDesc("产品物模型节点表");
    }

    @Override
    protected void init(HttpServletRequest request, HttpServletResponse response, ProductFunctionItemForm form,
                        Map<String, Object> model, Context context) {
        this.addDict(model, "eventType", EventTypeEnum.getEnumMap());
        this.addDict(model, "readOrWrite", ReadOrWriteTypeEnum.getEnumMap());
        this.addDict(model, "invokingType", InvokingTypeEnum.getEnumMap());
        this.addDict(model, "productFunctionType", ProductFunctionTypeEnum.getEnumMap());
        this.addDict(model, "productFunctionStatus", ProductFunctionStatusEnum.getEnumMap());
        this.addDict(model, "productId",
                productService.getCacheList().stream().collect(Collectors.toMap(e -> e.getId().toString(), ProductEntity::getProductName)));
        this.addDict(model, "customerId",
                customerService.getCacheList().stream().collect(Collectors.toMap(e -> e.getId().toString(), CustomerEntity::getLoginName)));
        super.init(request, response, form, model, context);
    }

    @Override
    protected void doListBefore(HttpServletRequest request, HttpServletResponse response, ProductFunctionItemForm form, Map<String, Object> model, Context context) throws AppException {
        Map<String, String> orderCols = new HashMap<>();
        orderCols.put("id", DbOrderKind.DESC.name());
        form.getQuery().setOrderCols(orderCols);
        super.doListBefore(request, response, form, model, context);
    }
}