package com.mortals.xhx.module.product.web;

import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.DbOrderKind;
import com.mortals.xhx.base.system.param.service.ParamService;
import com.mortals.xhx.common.code.DataUnitEnum;
import com.mortals.xhx.common.code.ParameterTypeEnum;
import com.mortals.xhx.module.product.model.ProductEntity;
import com.mortals.xhx.module.product.model.ProductFunctionItemEntity;
import com.mortals.xhx.module.product.model.ProductFunctionItemQuery;
import com.mortals.xhx.module.product.service.ProductFunctionItemService;
import com.mortals.xhx.module.product.service.ProductService;
import com.mortals.xhx.service.common.code.DataTypeEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.mortals.framework.model.Context;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import com.mortals.framework.web.BaseCRUDJsonMappingController;
import com.mortals.xhx.module.product.model.ProductFunctionItemDetailEntity;
import com.mortals.xhx.module.product.service.ProductFunctionItemDetailService;

import java.util.HashMap;
import java.util.Map;
import java.util.stream.Collectors;

/**
* ProductFunctionItemDetailController
*
* 产品物模型节点明细 控制器
*
* @author zxfei
* @date 2021-11-09
*/
@RestController
@RequestMapping("product/function/item/detail")
public class ProductFunctionItemDetailController extends BaseCRUDJsonMappingController<ProductFunctionItemDetailService,ProductFunctionItemDetailForm,ProductFunctionItemDetailEntity,Long> {

    @Autowired
    private ProductService productService;

    @Autowired
    private ProductFunctionItemService productFunctionItemService;

    public ProductFunctionItemDetailController(){
        super.setFormClass(ProductFunctionItemDetailForm.class);
        super.setModuleDesc("物模型节点明细表");
    }

    @Override
    protected void init(HttpServletRequest request, HttpServletResponse response, ProductFunctionItemDetailForm form,
                        Map<String, Object> model, Context context) {
        this.addDict(model, "dataType", DataTypeEnum.getEnumMap());
        this.addDict(model, "dataUnit", DataUnitEnum.getEnumMap());
        this.addDict(model, "parameterType", ParameterTypeEnum.getEnumMap());
        this.addDict(model, "productId",
                productService.getCacheList().stream().collect(Collectors.toMap(e -> e.getId().toString(), ProductEntity::getProductName)));
        this.addDict(model, "functionItemId",
                productFunctionItemService.find(new ProductFunctionItemQuery(), new Context()).stream().collect(Collectors.toMap(e -> e.getId().toString(), ProductFunctionItemEntity::getFunctionIdentifier)));
        super.init(request, response, form, model, context);
    }

    @Override
    protected void doListBefore(HttpServletRequest request, HttpServletResponse response, ProductFunctionItemDetailForm form, Map<String, Object> model, Context context) throws AppException {
        Map<String, String> orderCols = new HashMap<>();
        orderCols.put("id", DbOrderKind.DESC.name());
        form.getQuery().setOrderCols(orderCols);
        super.doListBefore(request, response, form, model, context);
    }
}