package com.mortals.xhx.service.app.model;

/**
 * cmd-client.jar与Mall HTTP响应定义{@link AppMsgRsp}<br/>
 * 特殊码：0成功<br/>
 * 返回类型+系统编号+错误码
 * 返回类型：1：参数错误 2：权限不足 3：业务错误
 * 系统编号：51智连服务
 * 错误码：001~199 为公司统一定义的全局返回码，200及以后为业务系统自定义返回码
 * 举例：151001
 */
public enum AppMsgRspCode {

    /** 成功 */
    SUCCESS(0, "成功"),
    /** 未知错误 */
    UNKNOWN_ERROR(351999, "未知错误"),

    // 1***~5***为mall系统http响应编码
    /** 参数错误 */
    PARAM_ERROR(151001, "参数错误"),
    /** AccessKey错误，找不到对应的customer信息 */
    ACCESSKEY_ERROR(151008, "AccessKey错误"),
    /** 设备找不到 */
    DEVICE_ERROR(351017, "找不到设备"),
    /** 签名错误 */
    SIGN_ERROR(151007, "签名错误"),
    /** 超时错误 */
    TIMEOUT_ERROR(151009, "超时错误"),
    /** 消息类型错误 */
    MSGTYPE_ERROR(351206, "消息类型错误"),
    /** 命令编码错误 */
    CMDCODE_ERROR(351208, "命令编码错误"),
    /** sessionId和deviceNum不能同时为空 */
    SESSION_DEVICENUM_ERROR(351209, "sessionId和deviceNum不能同时为空"),
    /** 设备离线 */
    DEVICE_OFFLINE(351018, "设备离线"),

    // 6***~8***为业务系统回复mall系统http响应编码
    /** 签名错误 */
    M_SIGN_ERROR(351007, "签名错误"),
    /** 非法的cmdCode */
    M_CMDCODE_ERROR(351202, "非法的cmdCode"),
    /** 设备繁忙 */
    DEVICE_BUSY(351203, "设备繁忙"),;

    private int value;
    private String desc;

    AppMsgRspCode(int value, String desc) {
        this.value = value;
        this.desc = desc;
    }

    public int getValue() {
        return value;
    }

    public String getDesc() {
        return desc;
    }
}
