package com.mortals.xhx.service.app.web;

import javax.annotation.Resource;

import com.mortals.xhx.common.code.DeviceStatusEnum;
import com.mortals.xhx.common.utils.BeanUtil;
import com.mortals.xhx.module.customer.model.CustomerEntity;
import com.mortals.xhx.module.device.model.DeviceEntity;
import com.mortals.xhx.service.app.model.AppInvokeServiceReq;
import com.mortals.xhx.service.app.model.AppMsgRsp;
import com.mortals.xhx.service.app.model.AppMsgRspCode;
import com.mortals.xhx.service.common.code.Constants;
import com.mortals.xhx.service.common.code.KafkaKey;
import com.mortals.xhx.service.handle.MsgCodecHelperToDB;
import com.mortals.xhx.service.handle.MsgCodecHelperToKafka;
import com.mortals.xhx.service.msgcodec.model.DwMsg;
import com.mortals.xhx.service.msgcodec.model.Function;
import com.mortals.xhx.service.msgcodec.proto.MsgCodec;
import com.mortals.xhx.service.queue.TbQueueMsg;
import com.mortals.xhx.service.queue.TbQueueProducer;
import com.mortals.xhx.service.queue.TopicPartitionInfo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;
import com.alibaba.fastjson.JSON;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.service.ICacheService;
import com.mortals.framework.util.MD5Util;

/**
 * 接收第三方应用下发的命令消息
 */
@RestController
@RequestMapping(value = "/ctp")
public class DwCmdController {

    private Logger log = LoggerFactory.getLogger(DwCmdController.class);

    @Autowired
    private TbQueueProducer<TbQueueMsg> producer;
    @Resource
    private ICacheService cacheService;
    @Resource
    protected MsgCodecHelperToKafka helperKa;
	@Resource
	protected MsgCodecHelperToDB helperDB;

    @RequestMapping(value = "/invokeService", method = RequestMethod.POST)
    public String invokeService(@RequestBody AppInvokeServiceReq req) {
        AppMsgRsp appRsp = new AppMsgRsp();
        // 检查Customer信息
        CustomerEntity customer = helperDB.getCustomer(req.getAccessKey());
        if (customer == null) {
            appRsp.setCode(AppMsgRspCode.ACCESSKEY_ERROR.getValue());
            appRsp.setMsg(AppMsgRspCode.ACCESSKEY_ERROR.getDesc());
            return JSON.toJSONString(appRsp);
        }
        // 验证签名
        String systemSign = MD5Util.strToMd5(req.getAction() + req.getAccessKey() + req.getArgs(), "utf-8");
        if (!req.getSign().equalsIgnoreCase(systemSign)) {
            appRsp.setCode(AppMsgRspCode.SIGN_ERROR.getValue());
            appRsp.setMsg(AppMsgRspCode.SIGN_ERROR.getDesc());
            return JSON.toJSONString(appRsp);
        }
        //TODO 判断时间,收到消息与服务器时间超过1分钟不处理，后续时间改为可配置 
        if((System.currentTimeMillis() - req.getTime()) / (1000 * 60)  > 1){
        	appRsp.setCode(AppMsgRspCode.TIMEOUT_ERROR.getValue());
            appRsp.setMsg(AppMsgRspCode.TIMEOUT_ERROR.getDesc());
            return JSON.toJSONString(appRsp);
        }
        try {
        	//判断设备状态
        	DeviceEntity device = helperDB.getDevice(req.getDeviceName(), req.getProductKey());
        	if (device == null) {
                appRsp.setCode(AppMsgRspCode.DEVICE_ERROR.getValue());
                appRsp.setMsg(AppMsgRspCode.DEVICE_ERROR.getDesc());
                return JSON.toJSONString(appRsp);
            }else if(device.getConnectStatus() != DeviceStatusEnum.ONLINE.getValue()){
            	appRsp.setCode(AppMsgRspCode.DEVICE_OFFLINE.getValue());
                appRsp.setMsg(AppMsgRspCode.DEVICE_OFFLINE.getDesc());
                return JSON.toJSONString(appRsp);
            }
        	// 检查物模型标识
        	Function functionItem = helperDB.selectProductFunctionItem(req.getProductKey(), req.getIdentifier(), Constants.SERVICE);
            if(null == functionItem){
            	appRsp.setCode(AppMsgRspCode.PARAM_ERROR.getValue());
                appRsp.setMsg(AppMsgRspCode.PARAM_ERROR.getDesc());
                return JSON.toJSONString(appRsp);
            }else if(null == functionItem.getServices()){
            	appRsp.setCode(AppMsgRspCode.PARAM_ERROR.getValue());
                appRsp.setMsg(AppMsgRspCode.PARAM_ERROR.getDesc());
                return JSON.toJSONString(appRsp);
            }
            MsgCodec msgCodec = helperKa.getMsgCodec(req.getProductKey(), req.getProtocolType());
            DwMsg dwMsg = BeanUtil.covert(req, DwMsg.class);
            // 下行命令处理
            TbQueueMsg queueMsg = msgCodec.doDeviceDwCmd(dwMsg, functionItem.getServices());
            TopicPartitionInfo info = TopicPartitionInfo.builder().topic(
            		KafkaKey.getDwEncodedTopicName(req.getProductKey())).build();
            producer.send(info, queueMsg, null);
            helperDB.insertDwDeviceLog(dwMsg, queueMsg.getData());
            // 回写serialNumber
            appRsp.setMsgId(dwMsg.getMsgId());
            // 返回响应
            appRsp.setCode(AppMsgRspCode.SUCCESS.getValue());
            appRsp.setMsg(AppMsgRspCode.SUCCESS.getDesc());
            return JSON.toJSONString(appRsp);
        } catch (Exception e) {
            log.error("[" + req.getDeviceName() + "]" + "处理收到的下行服务报错：" + req, e);
            throw new AppException(e);
        }
    }
}
