package com.mortals.xhx.service.common.util;

import java.util.Arrays;
import java.util.List;

import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.stereotype.Component;

import com.google.common.collect.Lists;

/**
 * spring容器获取bean工具
 */
@Component
public class SpringUtil implements ApplicationContextAware {

    private static ApplicationContext applicationContext;

    @Override
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        if (SpringUtil.applicationContext == null) {
            SpringUtil.applicationContext = applicationContext;
        }
    }

    /**
     * 获取applicationContext
     *
     * @return
     */
    public static ApplicationContext getApplicationContext() {
        return applicationContext;
    }

    /**
     * 通过name获取 Bean
     *
     * @param name
     * @return
     */
    public static Object getBean(String name) {
        return getApplicationContext().getBean(name);
    }

    /**
     * 通过class获取Bean
     *
     * @param clazz
     * @param <T>
     * @return
     */
    public static <T> T getBean(Class<T> clazz) {
        return getApplicationContext().getBean(clazz);
    }


    /**
     * 通过name,以及Clazz返回指定的Bean
     *
     * @param name
     * @param clazz
     * @param <T>
     * @return
     */
    public static <T> T getBean(String name, Class<T> clazz) {
        return getApplicationContext().getBean(name, clazz);
    }

    /**
     * 查询所有的beanDefinition List
     * @return
     */
    public static List<String> getBeanDefinitionNames() {
        String[] beanDefinitionNames = getApplicationContext().getBeanDefinitionNames();
        if(beanDefinitionNames.length == 0){
            return Lists.newArrayList();
        }else{
            return Arrays.asList(beanDefinitionNames);
        }
    }

    /**
     * 判断beanDefinitionName是否存在
     * @param beanDefinitionName
     * @return
     */
    public static boolean existBeanDefinitionName(String beanDefinitionName){
        return getBeanDefinitionNames().contains(beanDefinitionName) ;
    }
}
