package com.mortals.xhx.service.handle;

import com.mortals.framework.util.StringUtils;
import com.mortals.xhx.service.msgcodec.proto.CodecException;

import java.io.Serializable;
import java.util.concurrent.ThreadLocalRandom;

/**
 * 消息解析辅助类基类
 * @author shaosheng
 * @date 2021/5/17 下午5:03
 */
public abstract class AbstractMsgCodecHelper {

    /**
     * 解析Topic
     * @param
     * @return
     */
    public Topic parseTopics(String topics) {
        return new Topic(topics);
    }
    
    /**
     * 构造Topic
     * @return
     */
    public Topic buildTopic(String productKey, String deviceName, String topicType, String identifier, String operation) {
        return new Topic(productKey,deviceName,topicType,identifier,operation);
    }
    
    /**
     * 解析Client
     * @param
     * @return
     */
    public Client parseClientId(String clientId) {
        return new Client(clientId);
    }
    
    /**
     * 构造Client
     * @param productKey
     * @param deviceName
     * @return
     */
    public Client buildClient(String productKey, String deviceName) {
        return new Client(productKey, deviceName);
    }
    
    /**
     * clientId类
     */
    public final static class Client implements Serializable {

		private static final long serialVersionUID = 1326453979766195982L;
    	
		/** Topic类长度 */
        private static final int clientIdLen = 2;
        
		/** 产品全局唯一标识符 */
        private String productKey;
        
        /** 设备名称 */
        private String deviceName;
        
        Client(String productKey, String deviceName) {
            if (StringUtils.isEmpty(productKey)) {
                throw new CodecException("productKey is null");
            }
            if (StringUtils.isEmpty(deviceName)) {
                throw new CodecException("deviceName is null");
            }
            this.productKey = productKey;
            this.deviceName = deviceName;
        }
        
        Client(String clientId) {
            if (StringUtils.isEmpty(clientId)) {
                throw new CodecException("clientId is null");
            }
            String[] arr = clientId.split(":");
            if (arr.length != Client.clientIdLen) {
                throw new CodecException("clientId error");
            }
            this.productKey = arr[0];
            this.deviceName = arr[1];
        }

        /**
         * 获取Client
         * @return
         */
        public String getClient() {
            return String.format("%s:%s", productKey, deviceName);
        }

		public String getProductKey() {
			return productKey;
		}

		public String getDeviceName() {
			return deviceName;
		}

		public static int getClientidlen() {
			return clientIdLen;
		}

		@Override
		public String toString() {
			return "Client [productKey=" + productKey + ", deviceName=" + deviceName + "]";
		}
    }

    /**
     * Topic类
     */
    public final static class Topic implements Serializable {

        private static final long serialVersionUID = -8588428300989061856L;
        //  /sys/${productKey}/${deviceName}/thing/event/${tsl.event.identifier}/dev_post
        //  /sys/${productKey}/${deviceName}/thing/service/${tsl.service.identifier}/ser_post
        /** Topic类长度 */
        private static final int topicLen = 8;
        /** 标识 */
        private String sys;
        /** 产品全局唯一标识符 */
        private String productKey;
        /** 设备名称 */
        private String deviceName;
        /** 符号 */
        private String thing;
        /** 物模型Topic功能类型  属性  事件  服务*/
        private String topicType;
        /** 物模型标识符 */
        private String identifier;
        /** 操作类型： 发布 订阅 */
        private String operation;

        Topic(String productKey, String deviceName, String topicType, String identifier, String operation) {
            if (StringUtils.isEmpty(productKey)) {
                throw new CodecException("productKey is null");
            }
            if (StringUtils.isEmpty(deviceName)) {
                throw new CodecException("deviceName is null");
            }
            if (StringUtils.isEmpty(topicType)) {
                throw new CodecException("topicType is null");
            }
            if (StringUtils.isEmpty(identifier)) {
                throw new CodecException("identifier is null");
            }
            if (StringUtils.isEmpty(operation)) {
                throw new CodecException("operation is null");
            }
            this.sys = "sys";
            this.productKey = productKey;
            this.deviceName = deviceName;
            this.thing = "thing";
            this.topicType = topicType;
            this.identifier = identifier;
            this.operation = operation;
        }

        Topic(String topics) {
            if (StringUtils.isEmpty(topics)) {
                throw new CodecException("topic is null");
            }
            String[] arr = topics.split("\\/");
            if (arr.length != Topic.topicLen) {
                throw new CodecException("topic error");
            }
            this.sys = arr[1];
            this.productKey = arr[2];
            this.deviceName = arr[3];
            this.thing = arr[4];
            this.topicType = arr[5];
            this.identifier = arr[6];
            this.operation = arr[7];
        }

        /**
         * 获取Topic类
         * @return
         */
        public String getTopic() {
            return String.format("/%s/%s/%s/%s/%s/%s/%s", sys, productKey, deviceName, thing, topicType, identifier, operation);
        }

        public String getSys() {
            return sys;
        }

        public String getProductKey() {
            return productKey;
        }

        public String getDeviceName() {
            return deviceName;
        }

        public String getThing() {
            return thing;
        }

        public String getTopicType() {
            return topicType;
        }

        public String getIdentifier() {
            return identifier;
        }

        public String getOperation() {
            return operation;
        }
    }
    
    /**
     * 生成消息唯一序列号<br/>
     * (默认方式为随机产生一整数(-2147483648—2147483647)之间,周期性循环
     * @return
     */
    public String generalMsgId() {
        return "" + ThreadLocalRandom.current().nextInt();
    }
}
