package com.mortals.xhx.service.handle;

import com.mortals.xhx.common.code.DeviceLogTypeStatusEnum;
import com.mortals.xhx.common.code.DeviceStatusEnum;
import com.mortals.xhx.module.customer.model.CustomerEntity;
import com.mortals.xhx.module.customer.service.CustomerService;
import com.mortals.xhx.module.device.dao.DeviceDao;
import com.mortals.xhx.module.device.dao.DeviceLogDao;
import com.mortals.xhx.module.device.model.DeviceEntity;
import com.mortals.xhx.module.device.model.DeviceLogEntity;
import com.mortals.xhx.module.device.model.DeviceQuery;
import com.mortals.xhx.module.device.service.DeviceService;
import com.mortals.xhx.module.product.model.ProductEntity;
import com.mortals.xhx.module.product.model.ProductFunctionItemEntity;
import com.mortals.xhx.module.product.model.ProductFunctionItemQuery;
import com.mortals.xhx.module.product.service.ProductFunctionItemDetailService;
import com.mortals.xhx.module.product.service.ProductFunctionItemService;
import com.mortals.xhx.module.product.service.ProductService;
import com.mortals.xhx.service.common.code.Constants;
import com.mortals.xhx.service.common.code.ProductFunctionStatusEnum;
import com.mortals.xhx.service.handle.dbHandle.FuncItemEventHandle;
import com.mortals.xhx.service.handle.dbHandle.FuncItemServiceHandle;
import com.mortals.xhx.service.msgcodec.model.BrokerMsg;
import com.mortals.xhx.service.msgcodec.model.DwMsg;
import com.mortals.xhx.service.msgcodec.model.Function;
import com.mortals.xhx.service.msgcodec.model.func.FuncItemEvent;
import com.mortals.xhx.service.msgcodec.model.func.FuncItemService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import java.util.Date;
import java.util.List;
import javax.annotation.Resource;

/**
 * 消息解析中需要操作数据库的控制类
 * @author shaosheng
 * @date 2021/5/17 下午4:49
 */
@Service
public class MsgCodecHelperToDB<T> extends AbstractMsgCodecHelper {

    public static Logger log = LoggerFactory.getLogger(MsgCodecHelperToDB.class);

    @Autowired
    private ProductFunctionItemDetailService productFunctionItemDetailService;

    @Autowired
    private ProductFunctionItemService productFunctionItemService;
    
    @Resource
    private CustomerService customerService;
    
    @Resource
    private DeviceService deviceService;
    
    @Resource
    private DeviceDao deviceDao;
    
    @Resource
    private ProductService productService;
    
    @Resource
    private DeviceLogDao deviceLogDao;

    /**
     * 获取物模型解析
     * @param productKey
     * @param identifier
     */
    public Function getFunctByProKey(String productKey, String identifier, Topic topic) {
    	
        return selectProductFunctionItem(productKey,identifier,topic.getTopicType());
    }

    public Function selectProductFunctionItem(String productKey, String identifier,String topicType) {
        ProductFunctionItemQuery productFunctionItemQuery = new ProductFunctionItemQuery();
        productFunctionItemQuery.setFunctionIdentifier(identifier);
        productFunctionItemQuery.setProductKey(productKey);
        productFunctionItemQuery.setProductFunctionStatus(ProductFunctionStatusEnum.FUNCTION_STATUS_PUB.getValue());

        List<ProductFunctionItemEntity> productFunctionItemList = productFunctionItemService.find(productFunctionItemQuery,null);
        if (null == productFunctionItemList || productFunctionItemList.isEmpty()) {
            return null;
        }

        Function function = new Function();

        ProductFunctionItemEntity productFunctionItemEntity = productFunctionItemList.get(0);


        if (topicType.equals(Constants.EVENT)) { // 事件
            FuncItemEventHandle funcItemHandle = new FuncItemEventHandle(productFunctionItemEntity);
            FuncItemEvent funcItemEvent = funcItemHandle.buildFuncItemEvent();
            function.getEvents().add(funcItemEvent);
        }

        if (topicType.equals(Constants.SERVICE)) { // 服务
            FuncItemServiceHandle funcItemServiceHandle = new FuncItemServiceHandle(productFunctionItemEntity);
            FuncItemService funcItemService = funcItemServiceHandle.buildFuncServiceEvent();
            function.getServices().add(funcItemService);
        }
        return function;
    }

	public CustomerEntity getCustomer(String accessKey) {
		return customerService.getCache(accessKey);
	}

	public DeviceEntity getDevice(String deviceName, String productKey) {
		return deviceService.getCache(productKey + ":" + deviceName);
	}
	
	public ProductEntity getProduct(String productKey) {
		return productService.getCache(productKey);
	}
	
	public void updateDeviceStatus(BrokerMsg bkMsg, int status) {
		Client client = parseClientId(bkMsg.getClientId());
		DeviceEntity entity = new DeviceEntity();
		entity.setConnectStatus(status);
		Date date = new Date(Long.valueOf(bkMsg.getTimestamp()));
		if(status == DeviceStatusEnum.ONLINE.getValue()){
			entity.setOnlineTime(date);
		}else{
			entity.setOfflineTime(date);
		}
		entity.setUpdateTime(date);
		DeviceQuery query = new DeviceQuery();
		query.setDeviceName(client.getDeviceName());
		query.setProductKey(client.getProductKey());
		deviceDao.update(entity, query);
	}
	
	public void insertUpDeviceLog(BrokerMsg bkMsg) throws Exception {
		DeviceLogEntity entity = new DeviceLogEntity();
		Date date = new Date(Long.valueOf(bkMsg.getTimestamp()));
		Client clientId = parseClientId(bkMsg.getClientId());
		entity.setDeviceName(clientId.getDeviceName());
		entity.setProductKey(clientId.getProductKey());
		entity.setContent(new String(bkMsg.getPayload(), "utf-8"));
		entity.setLogType(DeviceLogTypeStatusEnum.UPEVENT.getValue());
		entity.setCreateTime(date);
		deviceLogDao.insert(entity);
	}
	
	public void insertDwDeviceLog(DwMsg dwMsg, byte[] data) throws Exception {
		DeviceLogEntity entity = new DeviceLogEntity();
		Date date = new Date(dwMsg.getTime());
		entity.setDeviceName(dwMsg.getDeviceName());
		entity.setProductKey(dwMsg.getProductKey());
		entity.setContent(new String(data, "utf-8"));
		entity.setLogType(DeviceLogTypeStatusEnum.DWSERVICE.getValue());
		entity.setCreateTime(date);
		deviceLogDao.insert(entity);
	}
}
