package com.mortals.xhx.service.handle;

import java.nio.charset.StandardCharsets;
import javax.annotation.Resource;

import com.mortals.framework.util.StringUtils;
import com.mortals.xhx.module.product.model.ProductEntity;
import com.mortals.xhx.service.common.code.MsgProtocolType;
import com.mortals.xhx.service.common.util.SpringUtil;
import com.mortals.xhx.service.msgcodec.proto.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.stereotype.Service;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.common.code.YesNo;

/**
 * 消息解析中需要操作Kafka的控制类
 */
@Service
public class MsgCodecHelperToKafka extends AbstractMsgCodecHelper {

    public static Logger log = LoggerFactory.getLogger(MsgCodecHelperToKafka.class);
    
    @Resource
	protected MsgCodecHelperToDB helperDB;
    
    @Resource
    private KafkaTemplate<Integer, String> kafkaTemplate;
    
    /**
     * 推送消息到kafka
     * @param topic
     * @param msg
     * @param partition
     */
    public void pushToKafka(String topic, Object msg, int... partition) {
		log.info("推送消息到kafka：{}-{}：{}", topic, partition, msg.toString());
        if (partition == null || partition.length != 1) {
            kafkaTemplate.send(topic, JSON.toJSONString(msg));
        } else {
            kafkaTemplate.send(topic, partition[0], JSON.toJSONString(msg));
        }
    }
    
    /**
     * 将kafka消息转换clz实体
     * @param message
     * @param clz
     * @return
     */
    public <T> T parseKafkaMsg(Object message, Class<T> clz) {
        if (message == null) {
            log.warn("kafka消息为null");
            return null;
        }
        String msg = new String((byte[]) message, StandardCharsets.UTF_8);
        if (StringUtils.isEmpty(msg)) {
            log.warn("kafka消息为空");
            return null;
        }
        try {
            T request = JSONObject.parseObject(msg, clz);
            log.info("接收到kafka消息:{}", request);
            return request;
        } catch (Exception e) {
            log.error("kafka消息转为{}实例失败, message:{}", clz, msg, e);
        }
        return null;
    }
    
    public MsgCodec getMsgCodec(String productKey, int messageProtocl) {
    	ProductEntity productEntity = helperDB.getProduct(productKey);
        if (productEntity == null) {
            throw new CodecException("产品key错误：" + productKey);
        }
    	switch (MsgProtocolType.getByValue(messageProtocl)){
        case MQTT:
        	if(productEntity.getDataType() == YesNo.YES.getValue()){
        		return SpringUtil.getBean(MqttCodecCustom.class);
        	}
        	return SpringUtil.getBean(MqttMsgCodecBase.class);
        case COAP:
        	if(productEntity.getDataType() == YesNo.YES.getValue()){
        		return SpringUtil.getBean(CoapCodecCustom.class);
        	}
        	return SpringUtil.getBean(CoapMsgCodecBase.class);
        /*case LWM2M:
        	return SpringUtil.getBean(MqttCodecCustom.class);
        case HTTP:
        	return SpringUtil.getBean(MqttCodecCustom.class);*/
        default:
            log.error("非法的协议类型");
            return SpringUtil.getBean(MsgCodec.class);
        }
    }
}
