package com.mortals.xhx.service.handle.dbHandle;

import com.alibaba.fastjson.JSONObject;
import com.mortals.xhx.module.product.model.ProductFunctionItemDetailEntity;
import com.mortals.xhx.module.product.model.ProductFunctionItemDetailQuery;
import com.mortals.xhx.module.product.model.ProductFunctionItemEntity;
import com.mortals.xhx.module.product.service.ProductFunctionItemDetailService;
import com.mortals.xhx.module.product.service.ProductFunctionItemService;
import com.mortals.xhx.service.common.code.DataTypeEnum;
import com.mortals.xhx.service.msgcodec.model.func.*;
import org.apache.poi.ss.formula.functions.T;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;

/**
 * @author shaosheng
 * @date 2021/5/28 下午4:16
 */
public abstract class AbstractFuncItemHandle {

    public static Logger log = LoggerFactory.getLogger(AbstractFuncItemHandle.class);

    @Autowired
    private ProductFunctionItemDetailService productFunctionItemDetailService;

    @Autowired
    private ProductFunctionItemService productFunctionItemService;

    protected ProductFunctionItemEntity entity;

    public AbstractFuncItemHandle(ProductFunctionItemEntity entity) {
        this.entity = entity;
    }

    protected void buildFuncItemBase(FuncItemBase funcItemBase) {
        funcItemBase.setCustomerId(entity.getCustomerId());
        funcItemBase.setFunctionIdentifier(entity.getFunctionIdentifier());
        funcItemBase.setFunctionName(entity.getFunctionName());
        funcItemBase.setProductId(entity.getProductId());
        funcItemBase.setProductKey(entity.getProductKey());
        funcItemBase.setFuncItemId(entity.getId());
        funcItemBase.setProductFunctionMethod(entity.getProductFunctionMethod());
    }

    protected List<ProductFunctionItemDetailEntity> selectProductFunctionItemDetail(long productFunctionItemId) {
        ProductFunctionItemDetailQuery productFunctionItemDetailQuery = new ProductFunctionItemDetailQuery();
        productFunctionItemDetailQuery.setFunctionItemId(productFunctionItemId);
        List<ProductFunctionItemDetailEntity> detailEntityList = productFunctionItemDetailService.find(productFunctionItemDetailQuery,null);
        if (null == detailEntityList || detailEntityList.isEmpty()) {
            return null;
        }
        return detailEntityList;
    }

    protected void addFunctionItemDetailToFuncItem(List<ProductFunctionItemDetailEntity> detailEntityList, FuncItemBase funcItemEvent) {
        for(ProductFunctionItemDetailEntity entity : detailEntityList) {
            FuncItemDetail funcItemDetail = buildFuncItemDetail(entity);
            funcItemEvent.getFuncItemDetailList().add(funcItemDetail);
        }
    }

    private FuncItemDetail buildFuncItemDetail(ProductFunctionItemDetailEntity entity) {

        int dataType = entity.getDataType();

        FuncItemDetail funcItemDetail = new FuncItemDetail();
        funcItemDetail.setDetailIdentifier(entity.getDetailIdentifier());
        funcItemDetail.setParameterName(entity.getParameterName());
        funcItemDetail.setDataTypeDetail(entity.getDataTypeDetail());
        funcItemDetail.setDataType(dataType);
        funcItemDetail.setProductId(entity.getProductId());
        funcItemDetail.setProductKey(entity.getProductKey());
        funcItemDetail.setFunctionItemId(entity.getFunctionItemId());

        funcItemDetail.setDataDetail(buildFuncItemDetailData(entity.getDataTypeDetail(),dataType));

        if (DataTypeEnum.TSL_STRUCT.getValue() == dataType || DataTypeEnum.TSL_ARRAY.getValue() == dataType) {
            buildDetailDataToStructAndArray((T) funcItemDetail.getDataDetail(),dataType);
        }
        return funcItemDetail;
    }

    private void analyzeFuncItemDetail(FuncItemDetailDataTypeToComplex complex) {
        List<ProductFunctionItemDetailEntity> detailEntityList = selectProductFunctionItemDetail(complex.getFunctionItemDetailId());
        for(ProductFunctionItemDetailEntity entity : detailEntityList) {
            FuncItemDetail funcItemDetail = buildFuncItemDetail(entity);
            complex.getDataDetail().add(funcItemDetail);
        }
    }

    private void buildDetailDataToStructAndArray(Object dataDetail,int dataType) {
        try {
            if (DataTypeEnum.TSL_STRUCT.getValue() == dataType ) {
                FuncItemDetailDataTypeToStruct structData = (FuncItemDetailDataTypeToStruct)dataDetail;
                analyzeFuncItemDetail(structData);
            } else if (DataTypeEnum.TSL_ARRAY.getValue() == dataType) {
                FuncItemDetailDataTypeToArray arrayData = (FuncItemDetailDataTypeToArray)dataDetail;
                if (arrayData.getElementType() == DataTypeEnum.TSL_ARRAY.getValue()) {
                    analyzeFuncItemDetail(arrayData);
                }
            }
        } catch (Exception e) {
            log.error("解析STRUCT和ARRAY出错",e);
        }
    }

    private Object buildFuncItemDetailData(String dataTypeDetail,int dataType) {
        Object data = null;
        try {
            switch (DataTypeEnum.getByValue(dataType)) {
                case TSL_INT_32: case TSL_FLOAT: case TSL_DOUBLE:
                    data = JSONObject.parseObject(dataTypeDetail, FuncItemDetailDataTypeToBasic.class);
                    break;
                case TSL_ENUM:
                    data = JSONObject.parseObject(dataTypeDetail, FuncItemDetailDataTypeToEnum.class);
                    break;
                case TSL_BOOL:
                    data = JSONObject.parseObject(dataTypeDetail, FuncItemDetailDataTypeToBool.class);
                    break;
                case TSL_TEXT:
                    data = JSONObject.parseObject(dataTypeDetail, FuncItemDetailDataTypeToText.class);
                    break;
                case TSL_DATE:
                    data = JSONObject.parseObject(dataTypeDetail, FuncItemDetailDataTypeToDate.class);
                    break;
                case TSL_STRUCT: case TSL_ARRAY:
                    data = JSONObject.parseObject(dataTypeDetail,  FuncItemDetailDataTypeToComplex.class);
                    break;
                default:
                    throw new IllegalStateException("Unexpected value: " + dataType);
            }
        } catch (Exception e) {
            log.error("解析物模型中对应的数据类型数据失败",e);
        }
        return data;
    }
}
