package com.mortals.xhx.service.msgcodec.consumer;

import javax.annotation.Resource;

import com.mortals.xhx.service.common.code.MsgProtocolType;
import com.mortals.xhx.service.common.code.MsgType;
import com.mortals.xhx.service.handle.AbstractMsgCodecHelper;
import com.mortals.xhx.service.handle.MsgCodecHelperToDB;
import com.mortals.xhx.service.handle.MsgCodecHelperToKafka;
import com.mortals.xhx.service.msgcodec.model.BrokerMsg;
import com.mortals.xhx.service.msgcodec.proto.MsgCodec;
import com.mortals.xhx.service.queue.kafka.KafkaTbQueueMsg;
import lombok.SneakyThrows;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.stereotype.Service;
import com.alibaba.fastjson.JSONObject;
import java.io.IOException;
import java.util.*;

/**
 * 上行待解码命令队列-解码消费者
 */
@Service
public class UpDecodeConsumer {

    private Logger log = LoggerFactory.getLogger(this.getClass());

    @Resource
    private MsgCodecHelperToKafka helper;

    @Resource
    private MsgCodecHelperToDB helperDB;
    /**
     * 处理设备上行的待解码指令队列消息者<br/>
     * 从kafka(上行命令待解码队列)获取上行命令； <br/>
     * topic跟产品相关
     * @param
     */
    @SneakyThrows
    @KafkaListener(topicPattern = "Cust_UpDecode_.*", groupId = "iot-aep-service")
    protected void handle(List<ConsumerRecord<String, byte[]>> recordList) {
        if (!recordList.isEmpty()) {
        	//创建5个线程处理批量操作
        	ThreadPools.getPool("Cust_UpDecode", 5).execute(() -> {
	        	try {
		            List<KafkaTbQueueMsg> kafkaTbQueueMsgs = decodeRecords(recordList);
		            for (KafkaTbQueueMsg item : kafkaTbQueueMsgs){
		                BrokerMsg bkMsg = JSONObject.parseObject(JSONObject.toJSONString(item.getHeaders().getData()),
		                		BrokerMsg.class);
		                byte[] data = item.getData();
		                bkMsg.setPayload(data);
		                if(bkMsg.getProtocol() == MsgProtocolType.SYSTEM.getValue()){
		                    log.info("接收到kafka上行系统消息:" + new String(data,"utf-8"));
		                }else{
		                    log.info("接收到kafka上行设备消息:" + bkMsg.toString());
		                    AbstractMsgCodecHelper.Topic topics = helper.parseTopics(bkMsg.getTopic());
		                    MsgCodec msgCodec = helper.getMsgCodec(topics.getProductKey(), bkMsg.getProtocol());
		                    helperDB.insertUpDeviceLog(bkMsg);
		                    switch (MsgType.getByValue(bkMsg.getMessageType())) {
		                        case ConnCreate:// 连接创建在线
		                        	msgCodec.doConnectCreate(bkMsg);
		                            break;
		                        case ConnClose:// 连接断开离线
		                        	msgCodec.doConnectClose(bkMsg);
		                            break;
		                        case PubCmd:// 设备上行发布消息
		                        	msgCodec.doDeviceUpCmd(bkMsg);
		                            break;
		                        case SubCmd:// 设备上行订阅消息
		                        	//TODO
		                            break;
		                        default:
		                            log.error("非法的上行消息");
		                            break;
		                    }
		                }
		            }
	        	} catch (Exception e) {
	                log.error("解析转发命令失败", e);
	            }
        	});
        }
    }

    List<KafkaTbQueueMsg> decodeRecords(List<ConsumerRecord<String, byte[]>> records) {
        List<KafkaTbQueueMsg> result = new ArrayList<>(records.size());
        records.forEach(record -> {
            try {
            	if (record == null) {
                    log.warn("kafka消息为null");
                    return;
                }
                result.add(decode(record));
            } catch (IOException e) {
                log.error("Failed decode record: [{}]", record);
                throw new RuntimeException("Failed to decode record: ", e);
            }
        });
        return result;
    }

    public KafkaTbQueueMsg decode(ConsumerRecord<String, byte[]> record) throws IOException {
        return new KafkaTbQueueMsg(record);
    }
}
