package com.mortals.xhx.service.msgcodec.proto;

import java.io.UnsupportedEncodingException;
import java.util.Iterator;
import java.util.List;
import java.util.UUID;
import java.util.Map.Entry;
import javax.annotation.Resource;

import com.mortals.xhx.common.code.DeviceStatusEnum;
import com.mortals.xhx.common.utils.BeanUtil;
import com.mortals.xhx.module.device.model.DeviceTopicEntity;
import com.mortals.xhx.service.common.code.*;
import com.mortals.xhx.service.handle.AbstractMsgCodecHelper;
import com.mortals.xhx.service.msgcodec.model.*;
import com.mortals.xhx.service.msgcodec.model.func.FuncItemDetail;
import com.mortals.xhx.service.msgcodec.model.func.FuncItemDetailDataTypeToBasic;
import com.mortals.xhx.service.msgcodec.model.func.FuncItemEvent;
import com.mortals.xhx.service.msgcodec.model.func.FuncItemService;
import com.mortals.xhx.service.queue.DefaultTbQueueMsg;
import com.mortals.xhx.service.queue.TbQueueMsg;
import com.mortals.xhx.service.queue.TbQueueMsgHeaders;
import com.mortals.xhx.service.queue.kafka.DefaultTbQueueMsgHeaders;
import org.springframework.stereotype.Service;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.service.ICacheService;

/**
 * 需解码协议消息处理服务类 到此的消息是已经鉴权成功的消息
 */
@Service
public class MqttCodecCustom extends MqttMsgCodecBase {

	@Resource
	private ICacheService cacheService;



	@Override
	public void doConnectCreate(BrokerMsg bkMsg) throws CodecException {
		// 修改设备在线离线历史记录，实时在线状态从redis里面取
		helperDB.updateDeviceStatus(bkMsg, DeviceStatusEnum.ONLINE.getValue());
	}

	@Override
	public void doConnectClose(BrokerMsg bkMsg) throws CodecException {
		// 修改设备在线离线记录，实时在线状态从redis里面取
		helperDB.updateDeviceStatus(bkMsg, DeviceStatusEnum.OFFLINE.getValue());
	}

	@Override
	public void doDeviceUpCmd(BrokerMsg bkMsg) throws CodecException {
		String topics = bkMsg.getTopic();
		if (topics.endsWith(Constants.REPLY)) {
			// TODO mqtt响应消息,暂不处理
			// 有响应失败，失败的消息要考虑是否重发
			return;
		}
		MsgDataReq upMsgData = JSON.parseObject(bkMsg.getPayload(), MsgDataReq.class);
		try {
			// 解析topic内容
			AbstractMsgCodecHelper.Topic topic = helperDB.parseTopics(topics);
			// 根据产品key与物模型标识符查找具体物模型命令
			Function function = helperDB.getFunctByProKey(topic.getProductKey(), topic.getIdentifier(),topic);
			if (topic.getTopicType().equals(Constants.EVENT)) {
				// 事件
				MsgDataRsp dataRsp = doUpEventCheck(function.getEvents(), upMsgData, topic, bkMsg);
				doSubmitDwDecoded(bkMsg, topic, dataRsp);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	@Override
	public TbQueueMsg doDeviceDwCmd(DwMsg dwMsg, List<FuncItemService> services) throws Exception {
		//物模型解析校验
		MsgDataReq req = new MsgDataReq();
		req.setId(Constants.RSP_MSG_ID.equals(dwMsg.getMsgId()) ? helperDB.generalMsgId() : Constants.RSP_MSG_ID);
		req.setMsgId(Constants.RSP_MSG_ID.equals(dwMsg.getMsgId()) ? Constants.RSP_MSG_ID : dwMsg.getMsgId());
		FuncItemService service = services.get(0);
		req.setMethod(service.getProductFunctionMethod());
		req.setParams(dwMsg.getArgs());
		//生成消息ID
		dwMsg.setMsgId(Constants.RSP_MSG_ID.equals(dwMsg.getMsgId()) ? req.getId() : req.getMsgId());
		AbstractMsgCodecHelper.Topic topic = helperKa.buildTopic(dwMsg.getProductKey(), dwMsg.getDeviceName(), Constants.SERVICE,
				dwMsg.getIdentifier(), Constants.SER_POST);
		DeviceTopicEntity topicEntity = cacheService.hget(RedisKeys.getDeviceTopicBrokerKey(dwMsg.getProductKey() + ":" +dwMsg.getDeviceName()),
				topic.getTopic(), DeviceTopicEntity.class);
		TbQueueMsgHeaders header = new DefaultTbQueueMsgHeaders();
		header.put(MessageHeader.CLIENTID, (dwMsg.getProductKey() + ":" +dwMsg.getDeviceName()).getBytes());
        header.put(MessageHeader.MESSAGEPROTOCOL, new byte[]{(byte)dwMsg.getProtocolType()});
        header.put(MessageHeader.MESSAGETYPE, new byte[]{(byte) MsgType.DwCmd.getValue()});
        header.put(MessageHeader.TOPIC, topic.getTopic().getBytes());
        header.put(MessageHeader.QOS, new byte[]{(byte) topicEntity.getQos().intValue()});
        header.put(MessageHeader.TIMESTAMP, String.valueOf(dwMsg.getTime()).getBytes());
		TbQueueMsg queueMsg = new DefaultTbQueueMsg(UUID.randomUUID(), JSON.toJSONString(req).getBytes("utf-8"), header);
		return queueMsg;
	}

	/**
	 * 收到消息之后的响应
	 * 
	 * @param bkMsg
	 * @param topic
	 * @throws Exception
	 */
	public void doSubmitDwDecoded(BrokerMsg bkMsg, AbstractMsgCodecHelper.Topic topic, MsgDataRsp msgDataRsp) throws Exception {
		bkMsg.setPayload(JSONObject.toJSON(msgDataRsp).toString().getBytes());
		// 下行返回消息
		helperKa.pushToKafka(KafkaKey.getDwEncodedTopicName(topic.getProductKey()), bkMsg);
	}

	/**
	 * 事件消息校验
	 * @param events
	 * @param upMsgData
	 * @param topic
	 * @param bkMsg
	 * @return
	 * @throws Exception
	 */
	public MsgDataRsp doUpEventCheck(List<FuncItemEvent> events, MsgDataReq upMsgData, AbstractMsgCodecHelper.Topic topic, BrokerMsg bkMsg)
			throws Exception {
		MsgDataRsp msgDataRsp = BeanUtil.covert(upMsgData, MsgDataRsp.class);
		boolean match = events.stream().anyMatch(data -> data.getFunctionIdentifier().equals(topic.getIdentifier()));
		if (match) {
			for (FuncItemEvent funcItemEvent : events) {
				if (funcItemEvent.getFunctionIdentifier().equals(topic.getIdentifier())
						&& upMsgData.getMethod().equals(funcItemEvent.getProductFunctionMethod())) {
					JSONObject param = JSON.parseObject(upMsgData.getParams());
					// 参数校验
					Iterator<Entry<String, Object>> iterator = param.entrySet().iterator();
					while (iterator.hasNext()) {
						Entry<String, Object> entry = iterator.next();
						// 判断参数名
						boolean flag = funcItemEvent.getFuncItemDetailList().stream()
								.anyMatch(data -> data.getDetailIdentifier().equals(entry.getKey()));
						if (!flag) {
							buildMsgDataRsp(msgDataRsp,MsgCodeEnum.PARAM_ERROR.getValue(),MsgCodeEnum.PARAM_ERROR.getMsg());
							return msgDataRsp;
						}
					}
					// TODO 判断参数类型与值
					for (FuncItemDetail putData : funcItemEvent.getFuncItemDetailList()) {
						int dataType = putData.getDataType();
						if (dataType == DataTypeEnum.TSL_BOOL.getValue()) {
							int bool = (int) param.get(putData.getDetailIdentifier());
							if (bool != 1 && bool != 0) {
								// 参数值错误
								buildMsgDataRsp(msgDataRsp,MsgCodeEnum.DATA_TYPE_ERROR.getValue(),MsgCodeEnum.DATA_TYPE_ERROR.getMsg());
								return msgDataRsp;
							}
						} else if (dataType == DataTypeEnum.TSL_INT_32.getValue()) {
							int i = (int) param.get(putData.getDetailIdentifier());
							FuncItemDetailDataTypeToBasic dataBasic = (FuncItemDetailDataTypeToBasic)putData.getDataDetail();
							if (i < Integer.valueOf(dataBasic.getDataScopeMin())
									|| i > Integer.valueOf(dataBasic.getDataScopeMax())) {
								// 参数值错误
								buildMsgDataRsp(msgDataRsp,MsgCodeEnum.DATA_TYPE_ERROR.getValue(),MsgCodeEnum.DATA_TYPE_ERROR.getMsg());
								return msgDataRsp;
							}
						} else if (dataType == DataTypeEnum.TSL_TEXT.getValue()) {

						}
					}
					UpAppMsg upMsg = new UpAppMsg();
					upMsg.setProtocolType(bkMsg.getProtocol());
					upMsg.setClientId(bkMsg.getClientId());
			        upMsg.setTime(Long.valueOf(bkMsg.getTimestamp()));
					upMsg.setTopic(bkMsg.getTopic());
					upMsg.setIdentifier(funcItemEvent.getFunctionIdentifier());
					upMsg.setName(funcItemEvent.getFunctionName());
					upMsg.setEventType(String.valueOf(funcItemEvent.getEventType()));
					upMsg.setMsgId("0".equals(upMsgData.getMsgId()) ? upMsgData.getId() : upMsgData.getMsgId());
					upMsg.setData(param.toString());
					upMsg.setDeviceName(topic.getDeviceName());
					upMsg.setProductKey(topic.getProductKey());
					// 提交到上行已解码共享队列
					helperKa.pushToKafka(KafkaKey.getUpDecodedTopicName(topic.getProductKey()), upMsg);
					msgDataRsp.setCode(MsgCodeEnum.SUCCESS.getValue());
					msgDataRsp.setMessage(MsgCodeEnum.SUCCESS.getMsg());
				}
			}
		} else {
			// 事件标识错误
			msgDataRsp.setCode(MsgCodeEnum.EVENT_IDENTI_ERROR.getValue());
			msgDataRsp.setMessage(MsgCodeEnum.EVENT_IDENTI_ERROR.getMsg());
		}
		return msgDataRsp;
	}
}
