package com.mortals.xhx.service.msgcodec.proto;

import java.nio.charset.StandardCharsets;
import java.util.Arrays;
import java.util.List;

import javax.annotation.Resource;

import com.mortals.xhx.service.common.code.Constants;
import com.mortals.xhx.service.common.code.KafkaKey;
import com.mortals.xhx.service.handle.AbstractMsgCodecHelper;
import com.mortals.xhx.service.handle.MsgCodecHelperToDB;
import com.mortals.xhx.service.handle.MsgCodecHelperToKafka;
import com.mortals.xhx.service.msgcodec.model.*;
import com.mortals.xhx.service.msgcodec.model.func.FuncItemService;
import com.mortals.xhx.service.queue.TbQueueMsg;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;

/**
 * 不需解码协议透传消息处理服务类
 */
@Service
public class MqttMsgCodecBase implements MsgCodec {

    public static Logger log = LoggerFactory.getLogger(MqttMsgCodecBase.class);

    @Resource
    protected MsgCodecHelperToKafka helperKa;

    @Resource
    protected MsgCodecHelperToDB helperDB;

    @Override
    public void doConnectCreate(BrokerMsg bkMsg) throws CodecException {
    }

    @Override
    public void doConnectClose(BrokerMsg bkMsg) throws CodecException {
    }

    @Override
    public void doDeviceUpCmd(BrokerMsg bkMsg) throws CodecException {
        String topics = bkMsg.getTopic();
        if (topics.endsWith(Constants.REPLY)) {
            //TODO mqtt响应消息,暂不处理
            //有响应失败，失败的消息要考虑是否重发
            return;
        }
        AbstractMsgCodecHelper.Topic topic = helperKa.parseTopics(topics);
        UpAppMsg upMsg = new UpAppMsg();
        upMsg.setProtocolType(bkMsg.getProtocol());
        upMsg.setClientId(bkMsg.getClientId());
        upMsg.setTime(Long.valueOf(bkMsg.getTimestamp()));
        upMsg.setData(new String(bkMsg.getPayload(), StandardCharsets.UTF_8));
        upMsg.setTopic(topics);
        // 提交到上行已解码共享队列
        helperKa.pushToKafka(KafkaKey.getUpDecodedTopicName(topic.getProductKey()), upMsg);
    }

    protected void buildMsgDataRsp(MsgDataRsp msgDataRsp, int code, String message) {
        msgDataRsp.setCode(code);
        msgDataRsp.setMessage(message);

    }

    @Override
    public TbQueueMsg doDeviceDwCmd(DwMsg dwMsg, List<FuncItemService> services) throws Exception {
        return null;
    }


    public static void main(String[] args) {
        String string = "{" +
                "\"id\": \"123\"," +
                "\"version\": \"1.0\"," +
                "\"params\": [" +
                "{" +
                "\"attrKey\": \"Temperature\"," +
                "\"attrValue\": 36.8" +
                "}" +
                "]," +
                "\"method\": \"thing.deviceinfo.update\"" +
                "}";
        System.out.println(string);
        MsgDataReq parseObject = JSON.parseObject(string.getBytes(), MsgDataReq.class);
        System.out.println(parseObject.toString());
        System.out.println(Arrays.toString(string.getBytes()));
        System.out.println("------------------------");
        String string2 = JSONObject.toJSON(parseObject).toString();
        System.out.println(string2);
        MsgDataReq aa = JSON.parseObject(string2.getBytes(), MsgDataReq.class);
        System.out.println(aa.toString());
        System.out.println(Arrays.toString(string2.getBytes()));
    }
}
