package com.mortals.xhx.service.queue.kafka;

import com.mortals.xhx.service.queue.TbQueueCallback;
import com.mortals.xhx.service.queue.TbQueueMsg;
import com.mortals.xhx.service.queue.TbQueueProducer;
import com.mortals.xhx.service.queue.TopicPartitionInfo;
import lombok.extern.apachecommons.CommonsLog;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.apache.kafka.clients.producer.RecordMetadata;
import org.apache.kafka.common.header.Header;
import org.apache.kafka.common.header.internals.RecordHeader;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.kafka.support.ProducerListener;
import org.springframework.stereotype.Service;

import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;
@CommonsLog
@Service
public class TbKafkaProducerTemplate<T extends TbQueueMsg> implements TbQueueProducer<T> {

   // private KafkaProducer<String, byte[]> producer;
    @Autowired
    private KafkaTemplate<String, byte[]> producer;

    private String defaultTopic;
    private Set<TopicPartitionInfo> topics= ConcurrentHashMap.newKeySet();


    @Override
    public void init() {
    }

    @Override
    public String getDefaultTopic() {
        return null;
    }

    @Override
    public void send(TopicPartitionInfo tpi, T msg, TbQueueCallback callback) {
        String key = msg.getKey().toString();
        byte[] data = msg.getData();
        ProducerRecord<String, byte[]> record;
        if(tpi.getTopic()==null){
            tpi.setTopic(this.defaultTopic);
        }

        Iterable<Header> headers = msg.getHeaders().getData().entrySet().stream().map(e -> new RecordHeader(e.getKey(), e.getValue())).collect(Collectors.toList());
        record = new ProducerRecord<>(tpi.getTopic(), null, key, data, headers);

        producer.setProducerListener(new ProducerListener<String, byte[]>() {
            @Override
            public void onSuccess(ProducerRecord<String, byte[]> producerRecord, RecordMetadata recordMetadata) {
                if (callback != null) {
                    callback.onSuccess(new KafkaTbQueueMsgMetadata(recordMetadata));
                }
            }

            @Override
            public void onError(ProducerRecord<String, byte[]> producerRecord, Exception exception) {
                if (callback != null) {
                     callback.onFailure(exception);
                } else {
                     log.error("Producer template failure:", exception);
                }
            }
        });
        producer.send(record);
    }


    @Override
    public void stop() {
        if (producer != null) {
          //  producer.close();
        }
    }


}
